'///////////////////////////////////////////////////////////////////////
' CONSTANTS SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

CON

  _clkmode = xtal2 + pll8x            ' enable external clock and pll times 8
  _xinfreq = 10_000_000 + 0000         ' set frequency to 10 MHZ plus some error
  _stack = (64) >> 2            

  ' Size of graphics tile map
  X_TILES           = 16
  Y_TILES           = 12

  MaxFeeders = 6
  CogLimit = MaxFeeders -1

  
'///////////////////////////////////////////////////////////////////////
' VARIABLES SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////
VAR

  ' ScanLine Buffer
   byte ScanLine[256]
   long Params[14]
   long DoLine     
   long BallX
   long BallY
   long Debug

   byte Speed, DirX, DirY



'///////////////////////////////////////////////////////////////////////
' OBJECT DECLARATION SECTION ///////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////
OBJ
   tv       : "RGW_TV_DRV_011.spin"  ' instantiate a tv object
   Feeder   : "RGW_TV_FEEDER_DRV_011.spin"
   uart     : "RGW_FullDuplex.spin"


'///////////////////////////////////////////////////////////////////////
' PUBLIC FUNCTIONS /////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////
PUB start |  lX, lY

   ' Start TV driver
    TV_ScanBuff := @ScanLine
    TV_BackColor := $1b        ' blue
'  TV_BackColor := $02         ' black
    TV_Doline   := @DoLine
    TV_Debug    := @Debug  
    tv.start(@tvparams)

'  uart.start(31, 30, 2400)

   
   
    Params[00] := @TV_Status
    Params[01] := @Scanline
    Params[02] := @SpriteOffsets + MaxFeeders
    Params[03] := @Sprite_Size
    Params[04] := @Sprite_Data
    Params[05] := @BallX
    Params[06] := @BallY
    Params[07] := @Image_data
    Params[08] := @Palette_data
    Params[09] := @DoLine
    Params[10] := @Debug    
    Params[11] :=  MaxFeeders

    ' Startup the TV feeder cogs
    repeat ly from 0 to COGLimit
      Params[12] := ly
      Feeder.Start( @Params)
      repeat lx from 1 to $ff'ff
        lx := lx 

    
    
    ' BEGIN GAME LOOP ////////////////////////////////////////////////////
    repeat
      repeat lx from 0 to 4000
         lx:=lx
      Speed := 5
      If DirX == 1
        If BallX < CONSTANT(256 - 64)
          BallX := BallX + Speed
        Else
          DirX := 0
          BallX := BallX - Speed      
      Else
        If BallX > 0
          BallX := BallX - Speed
        Else
          DirX := 1
          BallX := BallX + Speed

      Speed := 3
      If DirY == 1
        If BallY < CONSTANT(192-64)
          BallY := BallY + Speed
        Else
          DirY := 0
          BallY := BallY - Speed      
      Else
        If BallY > 0
          BallY := BallY - Speed
        Else
          DirY := 1
          BallY := BallY + Speed
      
'      hex(debug,8)
    ' hex(@Sprite_Data+1,8,1)
    
' Debug use
PUB putc(txbyte)
  uart.tx(txbyte)

PUB hex(value, digits) ', Term)
  value <<= (8 - digits) << 2
  repeat digits
    putc(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))

  putc(13)
  putc(10)








'///////////////////////////////////////////////////////////////////////
' DATA SECTION /////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

DAT

' TV PARAMETERS FOR DRIVER /////////////////////////////////////////////
tvparams
TV_Status               long    0               ' status
                        long    1               ' enable
                        long    %011_0000       ' pins
                        long    %0000           ' mode
TV_ScanBuff             long    0               ' screen scan line buffer
TV_BackColor            long    0               ' colors
TV_HC                   long    x_tiles         ' hc
TV_VC                   long    y_tiles         ' vc
                        long    10              ' hx timing stretch
                        long    1               ' vx
                        long    0               ' ho
                        long    0               ' vo
                        long    55_250_000      ' broadcast on channel 2 VHF, each channel is 6 MHz above the previous
                        long    0               ' audio
TV_DoLine               long    0               ' Address of the doline buffer for the feeder cog
TV_Debug                long    0


SpriteOffsets           byte 0,1,2,3,4,5,0,1,2,3,4,5



'{
' Hydra Image
Palette_Data    LONG $5B5A4A02, $4C0304AD, $8E3EAE2A, $3D2C1D07

Image_Data      LONG $01001010, $11010101, $54432111, $11126661, $66536621, $56333553, $67777775, $32277778
                LONG $25449777, $73265556, $777999A7, $28228286, $68282282, $56666667, $34554494, $35445536
                LONG $66659456, $53649443, $56663655, $44555544, $45544544, $55555444, $22222666, $66663662
                LONG $22233556, $53662262, $CCCCCBB9, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCB45B, $CCCCCCCC
                LONG $01011010, $10101101, $26621011, $21112662, $65562211, $77536366, $27777777, $54577782
                LONG $65554675, $74557452, $86377A79, $82282222, $68222222, $66666677, $49454945, $44456666
                LONG $66666666, $65449456, $95366663, $54494549, $44455555, $55555555, $62222655, $66653536
                LONG $22265566, $66361126, $CCCCB943, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $BCB954CC, $CCBCCBCC
                LONG $10110101, $11110111, $11211111, $21111111, $55222116, $77774565, $67773867, $49477762
                LONG $65556264, $79977772, $22828677, $22822828, $82828282, $66666774, $54949945, $44456666
                LONG $66666664, $55444566, $53636636, $65545554, $55544555, $55599B95, $22265554, $26626666
                LONG $22265666, $33662122, $CCB99536, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CB95BCCC, $CCCCBCCB
                LONG $11111110, $11011010, $11111110, $21111111, $56622116, $77777777, $77A72287, $54667444
                LONG $55546265, $777AA774, $22222226, $22222222, $72222228, $56663777, $56549499, $45666594
                LONG $66666644, $66654456, $35335666, $55566666, $55555494, $55BB9945, $22655456, $26262622
                LONG $21265666, $63335362, $CBB43663, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $B44CCCCC, $CBCCCCBB
                LONG $10101011, $11111111, $11111111, $11111111, $77762111, $77227777, $77766277, $55625449
                LONG $77776265, $22867779, $82828282, $82828282, $78282822, $45456837, $44554549, $56665599
                LONG $53663445, $66655554, $66666656, $44545563, $35535999, $599B9455, $26654566, $66626222
                LONG $22265566, $36365552, $B9D36336, $CCCCCCCB, $CCCCCCCC, $CCCCCCCC, $4BCCCCCC, $CCCCCB44
                LONG $11111111, $11110110, $11111111, $11111111, $77777762, $78227777, $66544477, $46225555
                LONG $77A77377, $28222277, $28222282, $22222822, $22828228, $44556282, $44549455, $66649554
                LONG $54534455, $56666635, $56636366, $55545455, $53555494, $9BB95553, $55555663, $35662226
                LONG $22263366, $36366662, $47633363, $CCCCCCBB, $CCCCCCCC, $CCCCCCCC, $39CCCCCC, $CCCBCC46
                LONG $11011111, $10111111, $61111111, $72111116, $22777777, $76667776, $26454477, $77265556
                LONG $87777977, $22282822, $82282828, $28282282, $82282282, $94566822, $95659944, $6654B949
                LONG $63665556, $66666666, $55566666, $66535554, $35656565, $99955535, $54552654, $65662266
                LONG $222ED636, $33636666, $33333333, $CCCCCB9D, $CCCCCCCC, $CCCCCCCC, $6599BCCC, $CBCCCB46
                LONG $11111111, $11111110, $62111111, $77622226, $28777A77, $44477772, $26554623, $77477746
                LONG $22287777, $82822282, $22282222, $22828228, $28228228, $53356828, $94566549, $66655599
                LONG $66666666, $66666666, $55455566, $55666365, $55535655, $55555554, $55526494, $66626655
                LONG $62233365, $33366666, $36363355, $CCCBB476, $CCCCCCCC, $CCCCCCCC, $65B999BB, $CCCCCC93
                LONG $01111262, $11101111, $21112621, $7A762212, $677A777A, $45477736, $66555225, $77777777
                LONG $28222287, $22228228, $82822828, $82222822, $22822822, $36636282, $99946355, $66626544
                LONG $66666666, $36666666, $66554445, $55555666, $94453554, $5335659B, $56264994, $62265555
                LONG $66223666, $36366655, $33336333, $CCBBD363, $CCCCCCCC, $9BCCCCCC, $639BB999, $CCCBCCB5
                LONG $11111222, $11211111, $11112261, $97762121, $477777A7, $55522545, $47777225, $28777775
                LONG $82828282, $28282282, $28228282, $28282222, $88228222, $65366822, $54444436, $66666655
                LONG $66666666, $45366666, $63663555, $44956335, $99456599, $66666554, $66599466, $22665554
                LONG $66266622, $66366655, $33333333, $BB956333, $CCCCCCCC, $9999BCCC, $669BBBB9, $CCBCCCB5
                LONG $01111111, $11661111, $21111111, $A7721222, $46777A47, $55626554, $77A77625, $22228777
                LONG $22282228, $22828222, $22282228, $22822828, $22822828, $56666828, $45494566, $6666659B
                LONG $66666666, $55543366, $45656666, $44945599, $55555555, $66626656, $69949522, $66555552
                LONG $66226322, $36366666, $63333336, $BD333666, $CCCCCCCB, $B999499C, $664BB9BB, $CBCCBCB4
                LONG $11011111, $11221111, $22222111, $77321222, $668777A9, $77326555, $77777477, $82822286
                LONG $88228822, $82222882, $82828222, $82282282, $28282282, $66666828, $44945656, $62666649
                LONG $66666666, $66555453, $95366636, $33494999, $25544553, $59B31122, $59945222, $65445226
                LONG $66226522, $36666666, $66333333, $73663633, $4BCCCBB9, $9B999944, $665BBBBB, $CCCCCCC4
                LONG $11111111, $11111211, $11222211, $77811111, $62277997, $77726555, $22277777, $28282822
                LONG $22822228, $28282228, $22222828, $28228222, $82282822, $66666282, $59456656, $66666663
                LONG $43566666, $66666534, $94445666, $53554449, $25945449, $59966561, $94566222, $56556659
                LONG $26226663, $36666666, $36666666, $36338636, $4559BC93, $B9999999, $665B9B9B, $CCCBCBC9
                LONG $11111111, $21112211, $11111222, $77811211, $77779A77, $77777777, $82822277, $22822282
                LONG $28228282, $22828282, $82828282, $82282282, $22828282, $66666828, $54563566, $66666666
                LONG $65455662, $66666666, $59B95363, $94453335, $65566344, $45565952, $95222264, $56622599
                LONG $22226355, $66666666, $63666666, $63363363, $94454D33, $9B9B9999, $5669B9B9, $CCBCCCCB
                LONG $11111111, $22122111, $11001112, $88821111, $777A7777, $67777A77, $22228228, $28228228
                LONG $82282828, $28222822, $22222228, $22822822, $88222828, $66266828, $66655666, $66666666
                LONG $66665535, $63666666, $55545636, $4B994453, $B9555635, $54455665, $52222264, $52266594
                LONG $6222E366, $68622656, $36836666, $33333636, $44556333, $B9999999, $5669B9B9, $CBCCBCBB
                LONG $11111111, $12111211, $11100101, $77222111, $7A79A777, $2226777A, $88282822, $82822822
                LONG $28228222, $22828282, $82828222, $82282282, $82882222, $26662882, $66556666, $35566666
                LONG $66666635, $36666666, $53545656, $54459949, $944B9363, $65494554, $21222266, $22649956
                LONG $66626552, $66866226, $63666868, $33333363, $45455D33, $99B99994, $46249B99, $CCBCCCCB
                LONG $11111111, $12222621, $11001011, $87222101, $77777778, $82822263, $22222282, $22282828
                LONG $22822828, $82222228, $28222282, $28228222, $81228888, $66262222, $65666666, $65555666
                LONG $66666666, $66366366, $959B4363, $36355599, $35DCB444, $22544556, $22562112, $65264421
                LONG $22266532, $66686662, $36663666, $63333336, $455533E3, $99999445, $4665B999, $BCCBCBCC
                LONG $11111111, $11222211, $10111011, $77782110, $22288787, $22222222, $82882822, $82828222
                LONG $28228282, $22828282, $82828822, $22828282, $11112222, $66662112, $63666666, $66665655
                LONG $66666666, $33663666, $53555636, $44533335, $5634B4B9, $26654454, $25BB6232, $49543222
                LONG $62266663, $68666655, $33666333, $33333333, $555556D6, $66544454, $96259999, $CCBCCCCB
                LONG $11111112, $12221211, $11211111, $77777210, $82877877, $82828288, $22222282, $28222828
                LONG $82282222, $82282822, $22222282, $22288828, $11112111, $66662236, $66666626, $66666633
                LONG $66666666, $66336636, $66666366, $99994563, $95335544, $2665544B, $265565B5, $99432212
                LONG $55561265, $33686666, $66666633, $33333333, $55656633, $56656555, $B5669999, $CBCCBCBC
                LONG $11111116, $22221111, $22111016, $87777621, $62687787, $28222222, $88282822, $82882282
                LONG $28228882, $28228282, $28282228, $12222888, $11211121, $62622253, $66662666, $66666666
                LONG $66666666, $66663663, $33336363, $55499533, $B4545333, $633665D9, $22222595, $35621222
                LONG $26222596, $66666662, $66666666, $36333663, $665656E3, $55656666, $B5264999, $CCBCCBCC
                LONG $11111124, $62111111, $65201116, $77777612, $68687877, $82282828, $22822282, $82228222
                LONG $82822222, $22822222, $82828882, $11111188, $12536221, $66662222, $66666666, $66666666
                LONG $63636666, $63666366, $63663663, $53355563, $54495445, $53354363, $22222265, $46222222
                LONG $22265994, $66862626, $66666226, $33633336, $66665633, $44566662, $B4624994, $BCCBCCBC
                LONG $11111126, $11111011, $65211632, $77777111, $22678777, $22822228, $28282822, $28282828
                LONG $22282828, $28228288, $12222828, $21111188, $12663333, $66622212, $62666666, $66666626
                LONG $36666636, $36636666, $36333636, $94533333, $33535949, $3639B533, $12222266, $62221222
                LONG $22264994, $62628662, $36662262, $36633663, $22656663, $54556212, $C9215955, $CCBCCBCB
                LONG $11111112, $26611110, $11102545, $77876111, $68877778, $28228286, $82228228, $82822822
                LONG $82828282, $28282822, $11128288, $21111111, $21212265, $66662121, $66666666, $36355336
                LONG $66363666, $66363336, $33636363, $94563636, $45333354, $6635B449, $22222653, $21222222
                LONG $22222656, $66626862, $36636222, $33666666, $11666663, $55555561, $C9616555, $CBCCBCCC
                LONG $11111111, $64556211, $10101265, $77773111, $26687877, $82822228, $82822822, $28282282
                LONG $28282822, $28888282, $11112112, $11111111, $21111112, $66222221, $36662662, $63555555
                LONG $66666636, $36666366, $63363636, $53333333, $B9455335, $3568554B, $22263453, $22122121
                LONG $62222266, $63566622, $66336222, $56666666, $62666623, $66666655, $CB622555, $CCBCCBCB
                LONG $11011111, $65545556, $11121126, $77778111, $28877777, $22282828, $28288282, $82282822
                LONG $82828228, $22882822, $11111111, $11111111, $12212111, $26662222, $55533666, $65555555
                LONG $63666666, $63636663, $36636363, $33336363, $44499956, $99566355, $222633DD, $63322222
                LONG $26222222, $54533226, $86666666, $33668686, $55666663, $66666666, $CB522666, $BCCCBCCB
                LONG $10111111, $22222545, $11266126, $73776111, $86777777, $82822286, $82222822, $22828228
                LONG $28288282, $18112228, $11111111, $11111111, $21112111, $66622221, $55555336, $55555555
                LONG $66636663, $36663353, $63363636, $36363333, $63549B53, $DB944533, $12222633, $59433822
                LONG $35222226, $34462345, $66626655, $53666668, $26626668, $26222222, $CB422266, $CCBBCCBC
                LONG $11111115, $21111121, $11262212, $77772111, $77778777, $28228227, $22828228, $76222282
                LONG $22886755, $11111182, $11111111, $11111111, $18121211, $56666222, $55555555, $55555555
                LONG $66666635, $663659B4, $33336363, $33333636, $53365533, $64999945, $22222222, $35449522
                LONG $45222266, $46615999, $68666544, $D6366866, $6226266E, $22262226, $CB921262, $CBCCBCCB
                LONG $11101255, $11111111, $11111111, $77772662, $22877777, $22822822, $28282822, $44576828
                LONG $11279994, $11111111, $11111111, $81111111, $11212121, $55533282, $55555555, $55455555
                LONG $36366355, $36665445, $63363636, $33335333, $53333633, $26355499, $22826636, $22653222
                LONG $62622222, $52654944, $66633545, $76636668, $2222626E, $22222222, $BC962226, $CCBCCBCB
                LONG $11111645, $11111111, $56111111, $77576599, $22288773, $82282222, $82222228, $94944536
                LONG $112BB944, $11111111, $11111111, $11811111, $62218111, $55555536, $55555555, $45555555
                LONG $66633555, $63636666, $33363636, $63636363, $33333333, $66333345, $62222545, $22222236
                LONG $22262112, $239B9952, $68355553, $36666666, $26222226, $26222222, $BCB61266, $CBCBCCCC
                LONG $11111265, $11111111, $B4652111, $7773259B, $28227777, $22822828, $57628282, $49494944
                LONG $111BB999, $11111111, $11111111, $11111181, $53382212, $55555555, $45555555, $54554555
                LONG $36635545, $36366366, $33636363, $33333333, $33633333, $33333333, $35326355, $22626553
                LONG $11593112, $649B9522, $65334522, $36628286, $22222222, $66226222, $BCB42266, $BCCCCCBC
                LONG $11111112, $11111111, $B45B4226, $3776259B, $22287737, $28228222, $44553628, $94494949
                LONG $11134999, $11111111, $11111111, $12128211, $55553682, $55555555, $55455555, $55555545
                LONG $66355555, $36363636, $36333336, $33363636, $33336333, $63333533, $4B468663, $622235DD
                LONG $22652234, $94456222, $35455265, $36266663, $22222222, $66662222, $CCB41122, $CCCCCBCB
                LONG $16211111, $61111111, $434BB999, $77381165, $28223777, $62222282, $94445556, $99944494
                LONG $11266655, $11111111, $11111111, $68264622, $55555553, $55555555, $55555555, $45455555
                LONG $65545545, $63636363, $63636363, $33333363, $53333333, $33363333, $D9B44453, $22662653
                LONG $22226255, $54562622, $43562694, $63628635, $12222222, $22266221, $BCB96212, $CCCBCBCB
                LONG $26321111, $62546111, $126459BB, $77211111, $22227773, $76282228, $44444444, $65549944
                LONG $12565556, $11111111, $32281111, $35334649, $55555555, $55555555, $55555555, $55545554
                LONG $55555555, $36336636, $33363633, $36333333, $33353333, $45353333, $6549B994, $23536343
                LONG $22266622, $46222222, $45226599, $23666553, $22222222, $12266626, $BCBB5662, $CCCCBCBC
                LONG $56211111, $59BB9115, $11111254, $72111111, $22283737, $45622822, $44544444, $22655549
                LONG $14B56555, $18111111, $56456811, $55556354, $53535555, $55555555, $55555535, $55455555
                LONG $55454554, $63363633, $33633636, $33336336, $33333333, $45333333, $26655449, $63365946
                LONG $22262222, $66262232, $32254945, $23336545, $66622222, $62666666, $CBBB5666, $CCBBCBBB
                LONG $56621111, $6449B524, $11111112, $81111111, $22287777, $55388222, $44444445, $12555665
                LONG $29C52662, $11111111, $36556438, $35355333, $35555355, $63555555, $45556666, $54554555
                LONG $45555545, $63636334, $33336363, $33333363, $33333333, $53535353, $54363335, $6263DBB5
                LONG $66262265, $22262543, $26945446, $8D345336, $26662222, $66666662, $BBCB5126, $CBBCBBCC
                LONG $26666621, $11125465, $11111111, $21221011, $28287383, $99588828, $65549444, $11655622
                LONG $14B96622, $11111811, $33366493, $53533535, $55353555, $33555535, $55363533, $55455545
                LONG $45454554, $36363555, $63333336, $33333333, $33333333, $53535335, $44444353, $33362593
                LONG $44626495, $26626334, $25994522, $8D455322, $12266662, $22266662, $CBBB4222, $BCBBCBBB
                LONG $66666662, $11111666, $11111111, $12662221, $22227772, $B4628888, $55565559, $22226211
                LONG $12594562, $22432111, $35333366, $35535353, $55535555, $33555555, $55333333, $45554555
                LONG $54555455, $33633444, $36363633, $33333333, $35333333, $55535353, $3549B533, $4BA62663
                LONG $55626D4D, $22266265, $44456282, $23436223, $21122666, $22226566, $BBCB9622, $BBCBBBCB
                LONG $66666666, $11111662, $11111111, $11211661, $82287821, $66211122, $66562666, $56662111
                LONG $11166655, $63545332, $53353333, $35553353, $35355553, $33555555, $55533333, $54545555
                LONG $54544555, $63335555, $33363363, $33333333, $53333333, $53353533, $33354335, $DBC53532
                LONG $62636685, $61222222, $99952226, $23362264, $53611222, $22222266, $BCBBB622, $BCBBBCBB
                LONG $26666666, $11112666, $11111111, $11111111, $82276211, $66610128, $22212626, $65556612
                LONG $36212226, $33363554, $35333656, $53555555, $53555535, $55555535, $55555533, $45555554
                LONG $45455545, $63863554, $33333633, $33363333, $33533333, $55555555, $43333355, $6D954994
                LONG $63446636, $42128226, $93222264, $6662654B, $26556112, $22222211, $BBBBB522, $BBBCBBBC
                LONG $26666666, $11116626, $10111111, $21111111, $22288211, $66661111, $62112222, $66665566
                LONG $65522666, $33633663, $56535333, $55553555, $55335353, $55555553, $55555555, $55455555
                LONG $55545455, $36888833, $33633333, $33333363, $35333333, $55555335, $B4333535, $66634BCB
                LONG $64444996, $52246122, $92228265, $21223449, $21265535, $22222222, $BBCBB562, $CBBBBCBB
                LONG $62266666, $11266662, $11111111, $11165121, $11222111, $55561111, $56621122, $69622655
                LONG $33362545, $33333336, $53333333, $33355555, $55553553, $55555555, $55555555, $45545455
                LONG $83355545, $33888888, $63333333, $33333333, $53353333, $53555555, $43353553, $53369B94
                LONG $333DDBB5, $22543226, $62222222, $22662266, $22122662, $62222222, $BB9BB462, $BBCBBBBB
                LONG $62622666, $12611122, $21111111, $11136245, $11111112, $99511111, $65556655, $55666226
                LONG $63666665, $35363633, $55335565, $35355333, $55535555, $55553555, $55555555, $D5D55555
                LONG $88833555, $38888888, $33333333, $33333333, $55533333, $53535555, $33555555, $B9949453
                LONG $33633D9B, $22262635, $22682682, $22662822, $22222222, $22222222, $B9999436, $BCBBBBBB
                LONG $22266666, $12210111, $54561621, $11111654, $11111111, $99611111, $62265549, $62137265
                LONG $63636666, $55333363, $53355655, $55333353, $35355555, $55555555, $55555555, $55545D54
                LONG $88888833, $33888888, $33383638, $33333333, $55555333, $75553735, $55557553, $9499B933
                LONG $35453634, $868265D3, $28226822, $22632228, $22122222, $62222222, $94444443, $BCBCBBBB
                LONG $11122266, $21101010, $66466562, $11101122, $11111111, $52111111, $26526665, $66666265
                LONG $56363666, $55555356, $35553533, $55555535, $55555555, $35555555, $D5555555, $83355555
                LONG $E8888888, $3888E8E8, $33333333, $33333333, $33D35553, $55375533, $35555575, $35449555
                LONG $3BC45533, $22266683, $22282226, $22636222, $12222222, $45622222, $44444449, $BCBBBB99
                LONG $01011122, $62262101, $11266622, $11110101, $11121111, $12111101, $23466521, $66666661
                LONG $35666363, $56355635, $33533555, $55535553, $53555555, $55555555, $545D5555, $88883355
                LONG $8E8E8E88, $3383B95E, $33333338, $53333333, $D33333D5, $5D555355, $55755555, $53555555
                LONG $8FCB4945, $665D6863, $28222228, $22236282, $21212222, $99456222, $44444444, $CBBBB994
                LONG $01010101, $22644601, $01012122, $11011011, $11662111, $11111111, $66625512, $63666666
                LONG $55635656, $63556553, $55533363, $35555533, $55535333, $55535553, $35555555, $E8888883
                LONG $E8888888, $338DBBD8, $33333333, $33D33333, $3353D333, $553D35D3, $D55533D3, $B9535575
                LONG $66554999, $6D953343, $82228282, $22266222, $22222222, $99994562, $94444999, $BBB94444
                LONG $56110101, $12265202, $11111111, $11101100, $11262211, $11111111, $66662111, $36636666
                LONG $35353533, $33356335, $55535533, $53355555, $55553555, $55555535, $833555D5, $88E88888
                LONG $8DB4EE8E, $833EFFD8, $33333833, $D333D333, $53D33333, $35555D33, $53535335, $94357555
                LONG $33544444, $65DD49D3, $22822222, $22266222, $62222222, $94494944, $44999999, $BB944444
                LONG $44560010, $11122106, $02611221, $11110111, $11122211, $21111111, $66666662, $36663666
                LONG $65556635, $33633656, $55535555, $35333353, $55355355, $D5555555, $88883355, $E88E8E88
                LONG $E9CB3E88, $383EEE88, $D33E3333, $33D333D3, $35333D33, $333D335D, $55D54553, $44535555
                LONG $49455445, $8283DDC4, $22282882, $22226828, $44562222, $99949499, $94999999, $44494449
                LONG $65561166, $11221012, $11221262, $10111110, $11126611, $62211111, $36666666, $63366666
                LONG $36533555, $55556563, $33555355, $35555333, $35535553, $3555D555, $8E888883, $4D8E888E
                LONG $8FFF88E9, $333DEF88, $3333333E, $33333333, $3D3D333D, $5D333D35, $355D5D4D, $53553335
                LONG $99499455, $22828354, $28222226, $22266622, $99995222, $99994944, $99999999, $4944499B
                LONG $22210644, $12222101, $11101111, $11101011, $11122211, $66662111, $66636666, $36663633
                LONG $66566556, $56555535, $53335555, $55553535, $55353535, $88335555, $888E8E88, $B9E8E8E8
                LONG $8EEE88DB, $3D88EED8, $3E333D33, $3D333D3D, $D3333D33, $55555333, $33555554, $55D55555
                LONG $4449B955, $88686635, $82288282, $21266286, $94999956, $49449449, $BBB99999, $44499999
                LONG $22100265, $01112111, $10110100, $01111110, $21111111, $66666662, $53666666, $63656655
                LONG $33633366, $55555555, $33533335, $35533335, $55555355, $88883335, $E8E888E8, $AD88D9DD
                LONG $8EEE88EF, $38388888, $333E3333, $333E3333, $33D3D333, $D5D5D5D3, $3D33D575, $4454D5D5
                LONG $44444D4D, $26663354, $22822828, $43226622, $44999BB9, $94994499, $999BB949, $4449999B
                LONG $22110122, $00000111, $01011011, $10101011, $66221111, $66666666, $55556366, $56666355
                LONG $55536356, $33555555, $35355336, $33535355, $33555535, $E8E88883, $8E8E8E8E, $EE88FBB4
                LONG $88EE388E, $E3888888, $D3D3D3E3, $3D33D3E3, $33333D3E, $5D553D3D, $55533535, $5445445D
                LONG $44444544, $86633444, $22282626, $B9455628, $99499999, $49949449, $BB99B999, $49449999
                LONG $11111122, $01101001, $11101111, $01111101, $66666211, $66666666, $66655653, $63636666
                LONG $35555666, $33633553, $55353355, $53353353, $88333555, $8E8EEE88, $88D4D8E8, $EE88FAAE
                LONG $888888EF, $38888888, $33333333, $33E33333, $DDDDD33D, $353D3533, $4D5D3D3D, $4D5D5545
                LONG $44444454, $66354444, $28228282, $99BB9456, $44999999, $9B994944, $B9B999B9, $44994999
                LONG $00011121, $10010010, $10111010, $21101011, $66666666, $66666666, $66666656, $63636366
                LONG $65556665, $33356363, $33533553, $35533533, $E8888633, $EE8E888E, $8EBBBDE8, $EE88EEF8
                LONG $8888888E, $DE888888, $E3E3E3E3, $3D33EDE3, $533333DD, $9D5D5DDD, $55D5539B, $4544545D
                LONG $44444444, $33544444, $62286866, $99999993, $99999999, $9B949999, $9949B999, $49449999
                LONG $01100111, $10101101, $01010110, $66222101, $66666666, $55656666, $66366666, $66363663
                LONG $33363336, $35555336, $33335553, $33353333, $EEEE8886, $D5E8EEE8, $88FAA888, $8888EEE8
                LONG $88888888, $338E8888, $33333D3D, $D3DD333D, $DDDDDD33, $BD5D5735, $5D5D5D9B, $D445D5D4
                LONG $44444D44, $34444444, $66862683, $99994456, $99999999, $99999BB9, $449999B9, $99999999
                LONG $00010000, $01000010, $10101101, $66666221, $66666666, $56565666, $55535655, $65666365
                LONG $66366635, $55356355, $33333333, $88633353, $8E8EEEEE, $BB93E8EE, $88EFF88E, $8888EEFE
                LONG $E8888888, $ED888E88, $3EDE3E38, $D33DDD3E, $33333DD3, $5D33D5DD, $4545D5DD, $45D4445D
                LONG $44444445, $44444444, $83386635, $94944566, $99B99499, $B99BB49B, $94444949, $99994944
                LONG $11101010, $00101100, $22110101, $66666666, $56666666, $56665555, $55666555, $66635555
                LONG $33365636, $33333355, $35333333, $FEE88633, $EEEEEEEE, $ACA888D3, $88FEE88E, $888888E8
                LONG $88E8E88E, $38E888E8, $D33333ED, $3D3E33D3, $DDDDD33E, $D5DD3D53, $5D5D5D55, $44445D54
                LONG $44444444, $44444444, $38886354, $99453668, $99999994, $9B999B99, $49494944, $99999449
                LONG $10000100, $01001001, $66621110, $66666666, $55566666, $65666665, $66655555, $36555556
                LONG $63636366, $33333355, $63333333, $FFFAEE88, $48EEEEEF, $EFF88EBB, $88EFF88E, $88888888
                LONG $8E888888, $3EE88E88, $E3EDEE33, $3DD3DE3D, $3D3D3DDD, $5D3D3D3D, $D5D5D5DD, $4D45D445
                LONG $4444444D, $44944944, $88865444, $94536683, $94949944, $444444B9, $94949494, $99999994
                LONG $01010101, $10110110, $66666621, $66666666, $66666666, $66566666, $66555565, $65555666
                LONG $33636363, $33333636, $88633333, $FFFFFFEE, $988EEEEF, $EFE88FAC, $8888E88E, $88E88888
                LONG $888E8E88, $E33E888E, $3DD33D3E, $D3ED3DDD, $D3D3DD3D, $D5D5DD3D, $D4D54D5D, $444D44D4
                LONG $44444444, $44444444, $82234444, $43666838, $44994444, $944494B9, $99499494, $B9999999
                LONG $01010100, $62111001, $66666666, $66666666, $66666666, $66556666, $35556556, $55555666
                LONG $63363636, $33336333, $EEE88833, $FFFFFFFF, $F883B94E, $FFE88EFF, $88888888, $E8888E88
                LONG $E8E888E8, $DE38E888, $E3EDE3E3, $ED3DE3ED, $DDDD3DD3, $DDDD5DD3, $54DD5DD5, $4444D54D
                LONG $44444D44, $94949444, $88134444, $66683888, $44444444, $44944499, $99499494, $CB999999
                LONG $10101010, $66622110, $66666666, $66666666, $66666626, $66656666, $55655556, $65666656
                LONG $36636363, $83333336, $FFFFFEE8, $EEFFFFFF, $E88FAC98, $8EE88EEF, $88E88888, $888E888E
                LONG $8E88E888, $E3EE8E8E, $DDD3DEDD, $DDD3DDD3, $DD3DE3ED, $5D5DD7DD, $DD45DD5D, $D4D444D4
                LONG $44444444, $49444949, $11834494, $66838888, $99444453, $44494994, $99994999, $BCCB9949
                LONG $11101010, $66666622, $66666666, $62626666, $66666666, $66555666, $56555566, $36363666
                LONG $63636666, $E8866633, $FFFFFAAF, $879A7FFF, $E88EAAF8, $88888EFF, $88888888, $E8E88E88
                LONG $E88E88E8, $3EE8E888, $E3EDED3E, $3DEDD3ED, $3DDDDDDD, $DDDDDDDD, $44DDD5DD, $444D4D4D
                LONG $4444444D, $44494444, $88169449, $63388282, $44544566, $44449944, $49499944, $49BCCB99
                LONG $62110001, $66666666, $66666666, $22666666, $62666666, $66565666, $66665666, $66365636
                LONG $36363636, $AFE88666, $FFFFFAAA, $8FBB98EE, $E88EEEE1, $888888EE, $8E88E888, $8888E88E
                LONG $8EE8E88E, $DD8E8EE8, $DEDE3EDE, $EEDBBADD, $DDEDDDED, $D5DD5DDD, $DD44D4D5, $4D44D4D4
                LONG $4444D444, $49444449, $12884494, $38888828, $44453663, $94449444, $99494499, $44449BCB
                LONG $66622110, $66666666, $62626666, $22122662, $66666222, $66556666, $66566656, $36666365
                LONG $26636366, $FFFFE882, $7A7FFFFF, $8EAAA188, $888EAFE8, $88888888, $88E888E8, $EE8E8E88
                LONG $888E8E88, $EEE8E8EE, $DDDDEDED, $DDDABBDE, $DDDDDDDD, $DDDDDDDD, $4DDD4DDD, $44D44D4D
                LONG $44D944D4, $44944944, $82224444, $88882822, $44556633, $99944944, $BBB94444, $4449449C
                LONG $62666211, $66666666, $66662666, $22222266, $26666622, $65565666, $36666666, $66666666
                LONG $82266663, $FFFFEEE8, $CB977FFF, $8EFFE88D, $8888EFE8, $8E888888, $E8888888, $88E888E8
                LONG $EEE8E88E, $3DEE8E88, $DEDEDDDE, $DEDEDEDE, $DDDDEDDE, $DDDDDDDD, $D44DD4D4, $4D44D4D4
                LONG $4444444D, $94494444, $28284494, $88282828, $453663E8, $49499444, $9BCB9994, $B9444944
                LONG $66662210, $66666666, $66666666, $22222666, $66626222, $66656266, $66365666, $66366366
                LONG $EE882266, $FFFFEEEE, $AAA88E7A, $8EEFE18F, $88888888, $888E8888, $888E8E8E, $8E888E8E
                LONG $8E88E8E8, $DE8EEEEE, $EDEDDEED, $DDDDEDED, $DDDDDDED, $DDDDDDDD, $4DD4DDDD, $4D4D4D4D
                LONG $44444D44, $444449D9, $28285994, $88828282, $45663388, $94994944, $449BCB99, $B9B94444
                LONG $62622211, $66666662, $66262626, $21222666, $66666222, $66656662, $66666665, $26666666
                LONG $EE8E8822, $47EEEEEE, $DAF818BB, $8EEFE88E, $88888888, $8E888E88, $8E888888, $E8E8E888
                LONG $E8EE8E8E, $EDEE8E8E, $DEDEEDDE, $EDEDDEDE, $DDEDDDDD, $DDDDDDDD, $D4DDDDDD, $44D4D4D4
                LONG $D9D4D44D, $94444444, $22823BBB, $22828228, $44458888, $B9949494, $44449BCC, $BB9B9994
                LONG $62222111, $66262666, $66666666, $22222662, $62666662, $66666666, $66666666, $82226666
                LONG $8EE88888, $B688EEEE, $FFE18EAA, $8888881E, $88888888, $8888888E, $E8EEE8E8, $E8E8E8E8
                LONG $EE8EE8E8, $EDEEEEEE, $DEDEDEED, $FDFEDFDE, $DDDDEDED, $DDDDDDDD, $4D4D4DDD, $D44D4D4D
                LONG $94449D44, $9B944944, $828239B9, $88281812, $94994338, $BCB99449, $9444449B, $CCBBBBB9
                LONG $26621123, $26666226, $26262626, $66266666, $66626666, $66666666, $63666666, $88822266
                LONG $888E8E88, $A8184953, $FFE81EFA, $8888888E, $E8888888, $8E88E888, $8888E88E, $8E8E8E8E
                LONG $8EEE8EEE, $DEEEEEEE, $DFEDFEDE, $DEDFDEDF, $FDFDDFDF, $DDDDDDDD, $DDDDDD4D, $4DD4D4D4
                LONG $D44D49D4, $9C9B9444, $18288B94, $28182288, $49444445, $4BBCB994, $B9944444, $CCCCBBBB
                LONG $62211115, $66662662, $62666666, $66666226, $66266626, $66666666, $26666666, $88888812
                LONG $88E88888, $E81EACB5, $8E818EFE, $88888818, $888E8888, $8E8E88E8, $E8E88888, $EEE8E8E8
                LONG $EEE8EEE8, $FEEEEEEE, $FEDEFEFE, $FDFDEFFE, $DFDDFDFD, $DDDDDFDF, $DD9D4DDD, $D49D4D4D
                LONG $44444D44, $CBBCBDB9, $28286444, $45621822, $94494944, $4449BCBB, $BBB99454, $CCCBBCCB
                LONG $62211111, $26266626, $26262626, $26262666, $66662666, $66666262, $12226666, $88888888
                LONG $1884D388, $E11EFAA3, $82881EFF, $88888888, $88888888, $8E888888, $8E8E8E8E, $E8EE8E8E
                LONG $EEEEE8EE, $FEDEEEEE, $FFFFEFFD, $DFFEFDFD, $DFDFDFDF, $DDDFDDFD, $D4DDDDDA, $9DD4DDD9
                LONG $B9D4D44D, $449B9BBB, $81828544, $49533281, $BB444494, $454449BC, $CBBBB944, $CBBBBCCC
                LONG $22112110, $62666622, $62626662, $66666262, $62626266, $66666666, $88112226, $88888888
                LONG $11AB9488, $E81EEEF8, $8888188F, $88888888, $8E88E888, $88E8E8EE, $E8E8E8E8, $EE8EEEE8
                LONG $EEEEEEE8, $FFEFEEEE, $EDEFDEFE, $FDFDFFEF, $DFDFFFFF, $DADDFDDF, $DDDDADDD, $44D49DDD
                LONG $C99B9D9D, $44499C9D, $2828254D, $44535538, $BCBB9449, $44545559, $CCCBBB99, $BCCCCCCC
                LONG $01211011, $66266211, $26262626, $62622626, $66266262, $22266666, $88881211, $8D388888
                LONG $18FACD11, $811EFEE8, $88888882, $88888888, $88E888E8, $8E88888E, $8E8E8E8E, $EEEE8E8E
                LONG $EEEEEEEE, $EFFEFFEE, $FFFFFFFF, $FFFFFFFF, $FDFDFDFD, $DDDADFFD, $DD9DDDDF, $DD9DD9D9
                LONG $BD9BBB94, $55D4D9BB, $32888349, $53633494, $59BCC994, $99555555, $CCCBB99B, $BCCCCCCC
                LONG $21111111, $26662221, $26262626, $26266266, $26666626, $21222266, $88188882, $39943188
                LONG $18EFF811, $1818EFE2, $88888888, $88888888, $88888E88, $8E8E8E88, $E8E8E88E, $EEE8EEEE
                LONG $EEEEEEEE, $FEFFEFEF, $FEFFEFEF, $FDFFDFFF, $FFFFFFFF, $FDFDDADD, $D9FDDDAD, $D9DD9DDD
                LONG $99C9DBC9, $3D5544D9, $9D818395, $36388D99, $4499BCB5, $BB555555, $CCCBBB9B, $CCCCCCCC
                LONG $21211010, $62666222, $62626262, $62626622, $66262662, $18112222, $68881818, $EABB2188
                LONG $18FDE811, $88112882, $88888888, $88888888, $8888E888, $E888888E, $EE8E8EE8, $EEEEE8E8
                LONG $EFEEEEEE, $FFFEFFFF, $FFFFFFFF, $FFFFFFFF, $DFDFDFDF, $DADFFDFA, $DDDADADD, $4D9DDD9A
                LONG $D4BBBC9D, $844D5D4D, $FE882838, $583EEED4, $49999994, $BC956554, $CBBBB999, $CCCCCCCC
                LONG $11110111, $62266222, $26226262, $66262626, $22626266, $81821121, $95328881, $DFFF1115
                LONG $18EEF818, $88888221, $88188888, $88888888, $E888DD88, $8EE8EE88, $8EEE8E8E, $EEEEEEEE
                LONG $FFFFEEEE, $FFFFFFEF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FDFDAFDF, $9FDDDDAD, $9DBC9DDD
                LONG $4DD9B9DD, $85444555, $EEE18888, $9DEEEFFF, $9999B999, $9B956549, $BBBBBBB9, $CCCCCCCC
                LONG $11111111, $26622622, $62262626, $62626262, $11122662, $88181812, $B9411888, $EDDE118A
                LONG $8123E811, $88888881, $18881888, $88888818, $888D49BD, $E88E88E8, $EE8EE8E8, $EEEEEEEE
                LONG $EFEFFFEE, $EFFFFFFF, $FFFFFFFF, $FFFFFFFF, $ADFDFFFF, $ADAFDFFD, $DDADADDD, $D9ABCBDA
                LONG $444DDDD9, $834D4D4D, $EEE88282, $BDEEEEEE, $99999BBB, $44445499, $BBBBBBB9, $CCCCCCBB
                LONG $21101111, $62222211, $22622262, $26266626, $12121122, $61888118, $AA311255, $EFE8118F
                LONG $81821111, $88888888, $81811818, $BD388888, $E8EDDDD9, $8EE8EE88, $EEEE8EEE, $EEEEEEEE
                LONG $FFFFFEFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FDFDFFFF, $D9DADDFA, $DBBBCABC
                LONG $99DD4D9D, $83D444D9, $EFE5D388, $BAFEEEEE, $9999B99B, $55559999, $BBBBBCB9, $CCCCCBBB
                LONG $11111111, $26262112, $66262622, $22622262, $18111111, $43118181, $3D8113B9, $8EE8218D
                LONG $88888111, $88888818, $88188188, $D99E8881, $8EDDD3DD, $EE8EE8E8, $EEEEEE8E, $FFEEEEEE
                LONG $FFFEFFFE, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFAFFFFF, $DADAFDAF, $9ABDADAD, $9DACBDCC
                LONG $DE83DDDD, $6854DD49, $FFE45386, $BCAEEEEE, $999B9BBB, $55549B99, $BBCCCC94, $BBBBBBBB
                LONG $10221111, $52262112, $22626226, $11122262, $18181211, $B2118328, $EE101FAB, $112211EE
                LONG $18188881, $81888888, $88818181, $D3388888, $E3D4DD44, $EEEE8E8E, $EEEEEEEE, $EFFEFEEE
                LONG $FFFFFEFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $AFFFFFFF, $DFFDAFFD, $CCCDA9FA, $DDDDDBBC
                LONG $388D4DDD, $C4334999, $AFE3883B, $BCAEEEEF, $9999B9BB, $55559B99, $BBCCCBD5, $49BBBBBB
                LONG $11211111, $66221111, $26262225, $11111122, $21811811, $D1015943, $EE1115DF, $8211118E
                LONG $88881818, $88888888, $88888818, $38888888, $DDD3D9DD, $EEEEEE8E, $EEEEEEEE, $FEFFEFEE
                LONG $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $ADAFFFAF, $ABAFBCBD, $DDDDDFDD
                LONG $8E83DD4D, $BD3D99DE, $AFE888DC, $BBAFEEEE, $99999BBB, $45354BB9, $CCCCCAAD, $55499BBB
                LONG $12101111, $26211111, $22226262, $11111111, $21118118, $3118BB94, $68111DD5, $18181112
                LONG $18181818, $18188888, $81811188, $E8888888, $9DDDD3D5, $EEFFE8ED, $EEEEE8EE, $FFFEFEFE
                LONG $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $AFFAFFFF, $ACBADAFF, $DABDCCCA, $9DDDCBAB
                LONG $888ED4DD, $D37D9D88, $AFEE8DBB, $BCAFEEEE, $94454BBB, $D55449B9, $CCCAAACA, $26649BCC
                LONG $16511111, $62211211, $11122262, $18111111, $11145621, $3103DDB3, $11118EE3, $18181811
                LONG $88818188, $18181818, $88888818, $88888881, $9DDDDD3E, $EDDDEEDD, $EFEEEEEE, $EFEFFFEF
                LONG $FFFFFEFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $BCACBFFA, $CCCCCCCC, $DDADBCCB
                LONG $8E88DDDD, $38D4D888, $FFEE4BBD, $BBCFEEEE, $9952649B, $454949B9, $CCAACCAC, $222649BC
                LONG $02510111, $22111110, $11111122, $11181111, $00699951, $211344D3, $8111118E, $81818181
                LONG $81881818, $11818881, $81811818, $88888888, $ED99DD88, $EEDEDDDD, $EEFEEEEE, $FFFFEFEF
                LONG $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FAFFAFFF, $CBAACAFF, $CCCCCCCC, $DDDDABCC
                LONG $88ED4DDD, $8D45888E, $FAEEBBD3, $BBCFEEEE, $99526659, $999449B9, $CCACCCAA, $2226559C
                LONG $13101101, $12111011, $11111111, $12321181, $01DAB961, $1118D442, $11811111, $18181811
                LONG $18818188, $88112818, $88888181, $8E3DD888, $DDDDDEE8, $EEEEEEED, $FEFEFEEE, $FFFEFFFF
                LONG $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFAFFFFF, $BAFCCAFA, $CCCCCCCC, $DD9CCCCC
                LONG $888EB9DD, $5D388888, $EAEEAD83, $BBBAEEEE, $99462665, $944459BB, $BACCCAC9, $21655559
                LONG $66011011, $11111110, $11111111, $06945211, $1144DD10, $11128ED1, $18118121, $18818118
                LONG $11181818, $11818881, $88181818, $888885D8, $EDDDD88E, $EEEEEEEE, $EFEFEFED, $FEFFFFEF
                LONG $FFEFFFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $AFFFAFFA, $BAFBAFAF, $CCCCCCCC, $DDAABBCC
                LONG $E8DDDDDD, $D3888888, $FAEDE335, $5BCAEEEE, $B9462663, $449459B9, $AAAACA9D, $2226523D
                LONG $31110110, $11110002, $11111111, $14999611, $11544411, $11112211, $11181111, $81181181
                LONG $81881181, $81811111, $68888881, $D888335D, $EEE8E8E3, $EDEEEEEE, $FEFEFEEE, $FFFEFEFF
                LONG $FFFFFFEF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFAFFFFF, $AFFFFFFF, $CCCCCCCC, $DD9DACCC
                LONG $8DD8DD9A, $38888E8E, $EADDD3DD, $54BCEEEE, $B9453663, $444459BB, $BACCADDD, $22122264
                LONG $10101111, $11122116, $64621111, $69BDD110, $018D5510, $11111111, $81111818, $18118118
                LONG $11111111, $18188181, $D8818188, $888DDD33, $EEEE8E8D, $EEDAEFEE, $FFFFEFED, $EFFFFFFE
                LONG $FFFFFFFF, $FFFEFFFF, $FFFFFFFF, $AFFFFFFF, $FAFAFAFF, $CCAAFAFA, $CCCCCCCC, $9FFDDBBC
                LONG $E8E8EDDD, $88888883, $EFDD9D3E, $65BBFEEE, $B9955562, $544559BB, $BBAADFDD, $22262223
                LONG $01111101, $12116632, $49942111, $599B5001, $11228210, $18111111, $18181111, $26181181
                LONG $18181111, $38811811, $88888854, $88EAADD3, $FEE8E3DE, $FEDCAFAF, $FEEFEFEE, $FFEFFEFF
                LONG $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FFAFFFFF, $AFFFFFAF, $CCAFAFAF, $CBCCCCCA, $DDDDACAB
                LONG $38E8DDDA, $EE888E3E, $EFDDFEE8, $269BFEEE, $BB944436, $555564BB, $9BDDDDD5, $22562228
                LONG $10101011, $11100111, $D9941001, $24993106, $11111111, $11111111, $81111811, $23811811
                LONG $81111111, $45818181, $88881354, $88DDAD88, $FFEE83E8, $EEBCCAAA, $EFFFEFEF, $FFFFFFEF
                LONG $FFFFFFFF, $FFFFFFEF, $FFFFFEFF, $FFFFFFFF, $FAFAFAFF, $AFFFFFFA, $AACACAAC, $DADACBCB
                LONG $8E88ED9D, $E88883E8, $EDAFFE8E, $359CDEEE, $BBB94553, $365364BB, $4BDDEED3, $16552222
                LONG $11111110, $11061001, $94D30111, $12332105, $22222100, $11181111, $11818118, $11111118
                LONG $11811811, $36818181, $8881D955, $8E849D38, $FFE8E888, $EEFDABAA, $FFEFFEFE, $EFFFEFFF
                LONG $EFFFFFFF, $FFFFFFFF, $FFFFFFFE, $AFFFFFFF, $FFAFFFAF, $FFFEFAAF, $A9FFFACA, $A9FDBCCB
                LONG $E8EEDDDD, $8E8E3E88, $EEDEEEE8, $499BAEEE, $BBBB9944, $6666349B, $49DDE3E3, $22662222
                LONG $10101011, $10261110, $44420000, $11221115, $11226221, $81111111, $61111111, $11111115
                LONG $81111111, $68181338, $8888FAD5, $D99D8888, $AAEEE8E8, $EDEEEDD9, $FEFEFFEF, $FFFFFFFE
                LONG $FFEFEFFF, $FFEFFFFF, $FFAFEFFF, $FAFAFFFF, $FAFAFAFF, $FFFFFFFA, $CFEEEAAA, $DADEEDAC
                LONG $8E8DDDDA, $E883D38E, $EE8EE8EE, $99BBBEEE, $9B9BB999, $66635499, $4BD3E3D8, $22222222
                LONG $11111101, $11210001, $35511011, $21111012, $11122262, $18118111, $11111881, $11121112
                LONG $68688111, $22818623, $8883AFF3, $DD9D8388, $DDE3E888, $FEFBBAEE, $FFFFEEFE, $FFFFFEFF
                LONG $FFFFFFFF, $FFFFFEFF, $AAEFFFFF, $FFFFFFFF, $FFFFAFAA, $FFFFFFFF, $FFEEEFFF, $9DFEEEED
                LONG $E8DDDADD, $8E3E3E88, $EEEEEE8E, $9BBBBD8E, $99BBBB9B, $22665549, $49DD3E86, $22222222
                LONG $10110111, $01101110, $28611111, $62211111, $12122222, $11111121, $11111D31, $11111111
                LONG $62621181, $88833665, $882D4958, $3DD83883, $8E8E88DD, $EEDCABEE, $FEFEFFEF, $FFFFFFFE
                LONG $EFFFFFFE, $FFFFFFFF, $FFFFFFFF, $AFAFFFFA, $FFAFAFAF, $FFFEFFAF, $EEEEEFFF, $DFDEEEEE
                LONG $8DA9DDDA, $83E8888E, $E8EE8EEE, $BBBBCDEE, $49999BBB, $22126655, $59DDD366, $12222222
                LONG $01101101, $12011011, $12111010, $22111111, $11211222, $11111211, $11101331, $11111111
                LONG $18221811, $81DAD546, $3D4383D2, $DD388838, $E8E88EDD, $EAAD9EE8, $FFEFEFFE, $FEFEFFFF
                LONG $FFEFFEFF, $FFFFFFFF, $FFFFFFFF, $FAFFFFAA, $FAFFFFAF, $FFFFFAFA, $EEEEFFFF, $DADEEEEE
                LONG $DD4BBADD, $ED88E8E8, $EEE8EE88, $BBBBBF8E, $554999BB, $22221223, $594D5322, $62222221
                LONG $01110110, $11101101, $11111111, $11211111, $12121111, $01183111, $10111011, $81111111
                LONG $11811111, $639ADD31, $39DBD235, $D38D8388, $EE8E8E3D, $FDBAEEEE, $FFFFFEEF, $FFFFFEFE
                LONG $FFFEFFFE, $FFFFFFFF, $FFFFFFFF, $FFAFFAAC, $FFAFAFFA, $FFFFAFFA, $EEEFEFFF, $DDFEEEEE
                LONG $DDABA9BA, $D38888DD, $E8EEE883, $BBBBBDEE, $23354999, $22222212, $59445322, $95612222
                LONG $11011011, $32011010, $11111101, $11111111, $11121111, $11034211, $66111111, $11111111
                LONG $18118181, $D8349318, $D33486D9, $8E383E33, $8EE8E3E3, $FEFDEDEE, $EFFFEEFA, $FFFFFFFF
                LONG $FFFFAAFF, $FFFFFFFF, $AAAFFFFF, $AAFFAABC, $AFAFAFAF, $FAFFAFAF, $EEEEEFFF, $CBFEEEEE
                LONG $DDDBDDFB, $8E8E8EDD, $EEE8E8ED, $99B9C9EE, $12265549, $22222222, $59445636, $BB952221
                LONG $10111111, $36010101, $11011010, $21111111, $11211111, $10116111, $95561110, $11811115
                LONG $81182111, $59386281, $99388355, $E38E3883, $EE8E3E38, $FBFEEEED, $FFFEEEFB, $FFFEFFEF
                LONG $FFFACCCA, $FFFFFFFF, $ACCCAFFF, $AAAACCAC, $AAFAFAAA, $FAAAFFFF, $EEEFFFFF, $CBAFEEEE
                LONG $DDDDDBCB, $E3E3DD5D, $EE8E83D8, $999BBAE8, $22122655, $36222222, $49BB9536, $BBBB9566
                LONG $11101010, $50110110, $11110106, $11111111, $08D61112, $01111101, $92296261, $81111113
                LONG $28811811, $9BD28282, $3D383883, $38338DD3, $E3ED8E8E, $AFEDEEDE, $FFEEEEEF, $AAFFFEFF
                LONG $FFACCABC, $FFFFFFFF, $CAACFAFA, $FACCCAAA, $AFAAAAFA, $FFAFFFAF, $EEFEEEFF, $CABCAFEE
                LONG $DDDDFDAC, $3E4BC9DD, $E8E3DD8D, $34999BEE, $28281823, $36366228, $BBBBBB95, $BCBBBBBB
                LONG $01111111, $20101011, $01011015, $12111111, $08D61111, $31010110, $54496155, $11111111
                LONG $22281181, $B5268228, $D838664B, $E8E38943, $EE3E3E33, $FEEEEEED, $FEEEEEEF, $CCCAAFFF
                LONG $FACCCCAC, $AFFFFFFF, $CAAFAFFF, $AAAACAAA, $AAFAAAAA, $FFFEFFFF, $FEEEEEEE, $ADCCBCFE
                LONG $DDDDDDDD, $DDBBCCCB, $888838E8, $66349D3E, $22222181, $95333662, $BCBCBBBB, $BBCBCBBB
                LONG $10111011, $01010101, $11111156, $11111111, $11110111, $95111001, $12352264, $11811111
                LONG $28218118, $95882682, $D38385D3, $33383DA4, $DEE3E8EE, $EEEEDEEE, $FEEEEEFF, $BBBCACAF
                LONG $ACCBBBCB, $FAFAFFFF, $CAAFFAAA, $AAAAACCC, $FFAAAAAA, $EFEEEFAF, $AAFEEEEE, $DDABCAEF
                LONG $CBADDDAD, $DBCCCBCC, $88888E3D, $8283338E, $22828281, $BB956336, $BBBBBBBB, $CBBBBBBC
                LONG $11111101, $10101111, $11110650, $D6111111, $11111010, $95011611, $11111544, $21118111
                LONG $82222281, $D8666826, $83838495, $3E3E3833, $EE3E8D38, $EEEFEEDE, $EEEEEFFE, $CCAFFFFE
                LONG $CCBCABCA, $AFAAAFFA, $CAAAAAFF, $ACCAAAAA, $FAAAAAAA, $FEEFFFFF, $ACAEFFEE, $FADFAAFF
                LONG $CCCCBFDD, $BCBCBCCB, $8883E3DD, $81818888, $36222222, $BBBB9563, $BBBBBBCB, $BBBCBCBB
                LONG $11111111, $11011010, $11126421, $66111011, $11101101, $46611930, $11111125, $28111111
                LONG $22822222, $86686268, $3833D9B9, $E8383338, $DEEEDE8D, $EEEEEEEE, $EEEEFFEE, $BFFFFEFE
                LONG $CBCACBAC, $AAAAAACC, $AAAAAAAA, $CCCACAAA, $AAFAFAAA, $EFFFFAFA, $BCAEACAA, $DDFFEEAB
                LONG $CCBCCDFF, $CBCCCCBC, $88E3E3D9, $28288188, $63366828, $CBBBBB95, $BBCBBBBB, $BBCBBBBB
                LONG $11011101, $01101111, $32116201, $1110102D, $11101110, $11595961, $22111110, $21222221
                LONG $26228222, $68668686, $838659D3, $3E33E833, $E3E3E3D8, $EEEFEDEE, $EEEEEEEE, $FFFFEFFE
                LONG $CCCCACCB, $CCAAACCC, $AAACCCAA, $CCCCCCCA, $AAAAAACC, $AEFFFFAF, $FAFFCAAC, $FFDFFEEE
                LONG $BCCBBC9F, $CCBCBCCC, $83E8DEDB, $82828888, $95636662, $BBBBBBBB, $BBBBCBBB, $BBBBBBBC
                LONG $11101111, $10110111, $5B200010, $1001013D, $11631001, $22165421, $21121112, $22212282
                LONG $68222222, $66868662, $38338838, $36E83838, $EEDE3E3E, $EEEEEEED, $EEEEEEEE, $FFFFEFFE
                LONG $ACBCCCAA, $CAAAACAB, $AAAAAACC, $CCCCCAAA, $FAAACCCC, $AFFFFAFA, $EEEFCCAC, $DDFDFEEE
                LONG $CCCBCCCB, $CBCBCCBC, $8E3EDDA9, $68282888, $BB453336, $BBCBBBBB, $BBBBBBCB, $CBBCBBBB
                LONG $11111111, $11101111, $6D123111, $01111011, $11494521, $22662222, $11221222, $22122122
                LONG $62682222, $68686628, $83383363, $3E3333E3, $EEE3E3E3, $EEEEFEDE, $EEEEEEEE, $FFEEFFFF
                LONG $ABACBCAF, $CCAACAAA, $FAAAAAAA, $CCCAAAAA, $AAAACCCC, $FFFFAFFA, $EEEFFEAC, $FAFFFFFE
                LONG $BCDFBBAD, $BCCCCBCC, $E3E3DBDA, $366828ED, $BBBB4563, $CBBBCBBB, $BBBBBBBB, $BBBBBCBB
                LONG $11111111, $00111110, $10025011, $61001111, $21495456, $26555662, $12212222, $22222212
                LONG $28226282, $66666866, $38383868, $838E8383, $DE3EDE3E, $EEFADEEE, $EEEEEEEE, $FEFFEFEE
                LONG $AABACCAF, $ACCCAAAA, $AAAAAAAA, $CCCAAAAF, $AAAAAACC, $FFFFFAAF, $FEEEEFFA, $DDFDFFFF
                LONG $CDDDDFBC, $CBCBCCBC, $3E83DD99, $6336683D, $BBBBBB45, $BBBBBBBB, $CBBCBBBB, $BBCBBBBB
                LONG $11111111, $32100111, $11111114, $40620010, $21164935, $55494566, $21222226, $22221222
                LONG $66262222, $86868686, $33838383, $E3333838, $DDDE3E33, $EEFBADDE, $EEEEEEEF, $AAFEFFEE
                LONG $AAFADBCC, $AAAAAAAA, $AFAAACAA, $AACCAAAF, $AAAAAACC, $FFFAFFAA, $FFFFFFFF, $FADADFFF
                LONG $DDDDDDFD, $BCBCBCCB, $33E3EDD9, $43333688, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $55546111, $01010106, $96956311, $22111225, $99BB9456, $22222269, $22222221
                LONG $28628222, $38666668, $38383666, $38E3E333, $ABBDE3EE, $EEFBBCBD, $EEEEEFEE, $BCAAFFEE
                LONG $FAABAFAA, $ACAAAAFF, $AAACCCCC, $AAAAAAFA, $CCAAAAAC, $FFFFFFAC, $FFFFFFFF, $CBAFADAF
                LONG $DDDDFDDB, $BBCBCCBD, $8883DD9D, $B9433333, $BBBBBBBB, $BCBBBBBB, $BBBBBBCB, $BBBBCBBB
                LONG $11111111, $01254201, $11011110, $64964B95, $22211111, $BBBBB956, $21222259, $22222222
                LONG $66222628, $86686862, $83338386, $3E3383E8, $BDDDBDE3, $EEDCABAB, $EEEEEEFE, $AABCAFFE
                LONG $FFFFFBDB, $CACAAAAF, $AACCCCCC, $AAAAAFAA, $CCCAAAAA, $FFFFFFAC, $AFFFFFFF, $CCCADFFF
                LONG $DDDABCCC, $9CBCCBDD, $33383D9D, $BBB94363, $BBBBBBBB, $BBBBBBBB, $BCBBCBBB, $BBCBBBBC
                LONG $11111111, $11102111, $65310101, $1121594B, $62111111, $9BBBB956, $22222264, $22222222
                LONG $62686222, $63866668, $33833833, $D38E3383, $ABBD9DD4, $FFABBBBB, $EEEEEEEE, $DAAABCAF
                LONG $FFFFFFAF, $CCAAAFFF, $ACCCCCCC, $AAAFFAFA, $ACCACCCA, $FFFFAFAA, $DFAFFFFF, $CCCDFAFA
                LONG $9BCCCCCC, $BBCCBDDD, $333383D9, $BBBBB943, $BBBBBBBB, $BBBBBBCB, $BBBBBBBB, $BCBBCBBB
                LONG $10111111, $01111111, $26993021, $11111349, $62662111, $549BB945, $22222265, $22222122
                LONG $86262862, $68668666, $38383688, $BA338338, $BADED9CC, $EEFABBAB, $FEEEEFEE, $ADADAABA
                LONG $FFFFFFFA, $AAAAFFFF, $AACCCCCC, $AAAAAFAA, $FAAAACCC, $FFFFFFFF, $AFFAFAFF, $CCCFDFDF
                LONG $CCCCCBAB, $D9C9D49B, $43333833, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBCB, $BBBBBBBB
                LONG $11111111, $52111111, $25494699, $11111012, $65556661, $55455445, $12222649, $22222222
                LONG $68626222, $63866686, $83338333, $CD8DDD33, $9DEDDDDD, $FFDBABBA, $AAFFEEFF, $FDAFAFAA
                LONG $FFFEFEFF, $AAFFFFFF, $AACCCCCC, $ACAAAAAA, $AFAACCCA, $AFFFFFFF, $FFFFFFAF, $CCBFAFAF
                LONG $CCCCCBDB, $83DDDCCC, $B9D33333, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB, $CBBBBBBB, $BBBBBCBB
                LONG $11110111, $69501111, $123436B6, $61111111, $59466665, $44445661, $22222299, $82222222
                LONG $26268222, $68666866, $38383868, $D3D9BCD3, $DE8ED3E3, $EEABABAB, $ABAAFFFF, $FFFDFDAD
                LONG $FFEFEEFE, $AAAFFAAF, $AACCCACA, $CCCCAAAA, $FFFFAAAA, $FFAFFFFF, $FBCCAFFF, $CCADFDFD
                LONG $BA9BCCCC, $3D83DDBC, $B9B9D33E, $BBB9BBB9, $BBBBBBBB, $BCBBBBBB, $BBBCBBBB, $BCBBBBBB
                LONG $11111111, $29411111, $11011144, $22111111, $65553644, $69454622, $22122222, $22222221
                LONG $62622626, $66686668, $9D338363, $33DC9D49, $EEEEDD3E, $FFBABDBF, $DFABAAFF, $EFFFFFFA
                LONG $FFFEEFEE, $AAFFAFAF, $AAACCCAA, $ACCCCCAA, $FFAFFFAA, $FFFFAFFF, $CCCCCCAF, $CCDFFAFA
                LONG $DDDDBCCC, $333D3DDD, $9B9B99D3, $B9BBB9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $10111111, $49501111, $11126215, $53211111, $51164545, $25446225, $12621222, $22222222
                LONG $68626222, $68368662, $DB333838, $DBCCC4BD, $E88DCBDD, $FABBBADE, $ADFFABAB, $EEFEFFFF
                LONG $FFFFEEFE, $AFAFFAFA, $CCAAAAAA, $AAACAAAC, $FFFFAAFF, $AFFFFFAA, $CCCCCCCC, $CCFDFDAC
                LONG $DDDDDDBC, $D33E33DD, $9BB9B999, $BBB9BB9B, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBC
                LONG $11111111, $16211111, $11125520, $46111111, $B6553229, $2126215B, $22122221, $22222212
                LONG $66862862, $66666863, $BD386383, $CCCBCBDB, $EEEDDDDC, $BBABBBEE, $FFDAFDAF, $FEEEFEFF
                LONG $FFFFFFEE, $FAFAFFAF, $AACAAAAA, $FFFAAACC, $AAFFFFAA, $CCAAFAAC, $CCAFFCCC, $CCDFFFBC
                LONG $3EDDFDAC, $99D33E3E, $9B9999B9, $BBBB9B9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $10111111, $11112221, $62112111, $55994226, $12212165, $22212212, $22222222
                LONG $32622622, $86866634, $33833836, $BCBCCB53, $8E3E3E9C, $DFDBAEEE, $EFFFDADA, $EEFEEEFE
                LONG $FFAAFEFF, $AFAFFAFF, $ACCACCAA, $FFAFAAAC, $CACAFFFF, $CCCCFFAC, $CCBAFFFC, $C9FFDFBC
                LONG $DDDEDDBC, $B994D3E3, $B99B9B99, $9B9BB9B9, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $10111111, $61111111, $054954B4, $21121211, $21221212, $22222221
                LONG $68268222, $36668355, $38386368, $DACCB9D8, $EED3E3ED, $ADBFDEE8, $FEEFFFFD, $EFEEFEEE
                LONG $AAAAFAFE, $AAAFAFFA, $ACCACCCC, $FAFFAFAA, $ACCAFAAF, $CCCAFFFF, $CCCCBAFB, $BFF9AABC
                LONG $3EDDDEDA, $999B9DDD, $9B999999, $BBB9B9B9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11011110, $11111111, $11111111, $41111111, $11663459, $22121111, $22122121, $22222122
                LONG $86222262, $66836543, $83338836, $EDDD9B9D, $8DDD8E38, $DADFEEEE, $EEEEEEEF, $EEFEFEFE
                LONG $AFFFFFFF, $AAFAFAFF, $AAAAAACC, $AFFFFAFA, $FFACACAF, $CCBFFFFF, $CCCCCCCC, $DDCCCCCC
                LONG $D3EDEDDE, $DBD99B9D, $99B9B99B, $9B9B9B9B, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB, $BBBCBBBB
                LONG $11111111, $11111111, $11111111, $21111111, $21111115, $11111212, $21221212, $22222221
                LONG $22828228, $86663536, $3D933683, $3D38D943, $DDDEE3E3, $EDFDEEEE, $EFEEEEEE, $FFEFEFEE
                LONG $FEEEEFFF, $AFFFFFAA, $FFFAAAAC, $AAAFFFFF, $AAFFACAA, $CBFFFFDA, $CBBACCCC, $FFDFBCCC
                LONG $9DDEDDED, $B9B9A999, $9B999999, $B9B9B9B9, $BBBB9BBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $11111112, $21212111, $22212121, $22222222
                LONG $66222222, $6833D568, $83949566, $D8E38883, $DDE33E33, $EEEEEEED, $FEFEEEEE, $EEEEFEFE
                LONG $EEEEEEEE, $AAFAFACA, $AAAFFAAA, $CCAAAFFF, $CCCCBFAC, $AFFFFACC, $FFFFBCCC, $EDFDFADF
                LONG $9B9DDDED, $9999999A, $B9999999, $9B9B9B99, $BBB9B9B9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111101, $11111111, $11111111, $11111111, $11111111, $11111121, $12212121, $22222222
                LONG $28268222, $633D5686, $33349D86, $33383338, $DE8E83E8, $EFFEEE3D, $FEEEEEEE, $EFEEEFEE
                LONG $AFEEEFEE, $FFFFFAAA, $FFFAFFAF, $ABCAAAAF, $CCCCCCAF, $FFFACCCC, $FFFCCCCA, $EDEFDFFF
                LONG $D9A99DDD, $9DBDBDB9, $999999A9, $B9B9B99B, $BB9BBB9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $11111111, $21211111, $22221212, $22222222
                LONG $66222282, $4D543262, $DB9D6668, $8E338838, $E3E3E83E, $EEEEEEDD, $EEFEFEEE, $EEEEEEEE
                LONG $CAAFEEEE, $FFFFFFAC, $AAFFAFFA, $AFABCCCA, $AFABCCCC, $FAFACCCC, $FFACCCBF, $DFDEFDFF
                LONG $9D9ADBDD, $999999DB, $9999B999, $B99B9999, $9BBB9BB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $12111111, $21121211, $22222211, $82222222
                LONG $26862222, $44536868, $6D958866, $383E3333, $DE3E3E83, $EEEEDD9D, $EEEEEFEE, $EEEEEEEE
                LONG $CAFAAFEE, $FFFFFFFA, $AAAAFFFF, $CCAFFCAC, $FFFFFACC, $FFFCCCCF, $FDFFBCFF, $ADFDEEFD
                LONG $A999A9AB, $9999A999, $9B999999, $9B99B9B9, $BB9BB99B, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $11111111, $11211112, $22222212, $22222222
                LONG $62622822, $D4568262, $38335383, $83383838, $DEE3E33E, $EEEFDABD, $EEEEFEEF, $FFEEEEEE
                LONG $CFFACAEF, $FFFFFFEF, $CACCAAFF, $CCCFFFAA, $FFFFFDAC, $FFACCCAF, $FFFFFFFA, $D9ADFDFE
                LONG $99AD9DBD, $9999999D, $99999999, $B99B9999, $B9BB9BB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111101, $11111111, $11111111, $11111111, $11111111, $12112111, $22222222, $22222222
                LONG $82228262, $55686686, $83599933, $3E338833, $E3DE3E83, $DFDDBADD, $EEEEEEEE, $AAFFEEEE
                LONG $EEEFCAFF, $FFFFFFEF, $FFACCFFF, $CCFFFCAA, $FFFFFACC, $FFACCCBF, $FFFFFFFF, $BFDADFDF
                LONG $9D99DBDA, $A9D9D9DB, $99999999, $9B999B99, $BBB9B9B9, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $11211111, $22211111, $22222222, $22222222
                LONG $68666222, $43862622, $383D4955, $83E88883, $DE33E3E3, $FCCCD9DE, $EEEEEEEE, $AFFAFEEE
                LONG $EEFEAFEA, $FFFEFEEE, $AFFAFFFF, $CFFFFCCC, $FFFFACCC, $FFFCCCCC, $FEFFFFFF, $DBA9A9FD
                LONG $99D9DA9D, $9999999D, $99999999, $99B99999, $9B9B9B9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $11111111, $22222212, $22122222, $62222222
                LONG $25494562, $36668682, $838633D4, $E3388838, $E3EE3333, $BCCCCDD3, $EEEEEEEF, $AEEFAFFE
                LONG $EFEFFEFC, $FFFFFFEF, $CFFFFFFF, $FFFFFACC, $CBAACCCC, $FFFFACCC, $DFFFEFFF, $ADDDADAA
                LONG $9D9D99D9, $99999D99, $99999999, $B999999B, $BBBB9B99, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $11111111, $21212212, $21222222, $22222222
                LONG $59B9B932, $68682626, $66683345, $33838688, $DD333E3E, $BCBBBBBD, $EEEEEEEA, $AAFAAFAE
                LONG $FFFEEEEF, $FFFFFFFE, $CFFFFFFF, $FFFFFACC, $CCCCCCCF, $FFFFFABC, $AADFFFFF, $D9ABDADD
                LONG $D9BD9D9A, $99D9D999, $9999999D, $999B9999, $9B99B99B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $21111111, $12222122, $22212122, $33222222
                LONG $49559B56, $66666863, $88633458, $38D83833, $9DDDE383, $BBBBBBBB, $FFEEEEEB, $EFACFEAF
                LONG $EFEFEEEE, $FFFFFFEF, $CFFAACAA, $FFFFFACC, $CCCCCCFF, $FFFFFFFF, $DDAFFFFF, $DADDADBF
                LONG $9DD9DBD9, $999999D9, $99999999, $9B999999, $B9BB9B99, $BBBB9B9B, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $22211111, $21212221, $22222222, $99432222
                LONG $9D639949, $86866264, $3883D436, $33333383, $B994DD38, $ABABBDBB, $FAEEEEEB, $EEEFEEAF
                LONG $FEFEFEEE, $CFFFFFFE, $CCCCCCCC, $FFFFFFCC, $FFCCCAFF, $FFFFFFFF, $ADADAFFF, $BDBD9FDA
                LONG $999D9D9D, $49D9D99D, $99999999, $99999999, $B99B99B9, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $11111111, $21222221, $22122112, $12221221, $44946322
                LONG $56349499, $36668663, $36334386, $E3838338, $BBBB99D5, $BBBBBBB9, $BCBACEDD, $EEEEEEAA
                LONG $FEFEFEFE, $AAFFFFFF, $CCCCCCCC, $FFFFFFCC, $FFFFFFFF, $FFFFFFFF, $DADAFDAF, $DDDADBDD
                LONG $D9D9D9AD, $99999D99, $999999D9, $9B999999, $BBB99B9B, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $21111111, $21212121, $21221222, $23562212, $23994956
                LONG $66344D95, $68686668, $63833366, $4D333383, $B9B9B9B9, $BBBDBBBB, $FFFFBEEB, $EEEEEEEF
                LONG $FFEFEEEE, $FFFAAAFF, $CBCAAFFF, $FFFFFCCC, $FFFFFFFF, $FFFFFFFF, $ADDFDAFF, $BD9DDDAD
                LONG $9D9D9D9D, $9D94999D, $99999999, $99999999, $B999B999, $BBB9BBB9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $21211111, $12121212, $12222121, $59952222, $54945993
                LONG $22636594, $86666866, $83668686, $994D3833, $BB9B99B9, $BBBBB9B9, $EEEDBEDB, $EEEEEEEE
                LONG $FFFEFEEE, $FFFCCCAF, $AFFFFFFF, $FFFFCCCC, $FFFFFFFF, $FAFFFFFF, $DADAFFDA, $DDD9DADD
                LONG $99D9D9D9, $4999D494, $99999499, $9999B999, $9BBB9B9B, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $12122111, $21212212, $22122212, $9B946222, $99735945
                LONG $66222649, $66666626, $86836636, $9B994433, $9B99B99B, $BDBBB9BB, $EEEEEEFB, $EEEEEEEE
                LONG $FEFFEFEE, $FFCCCCAF, $FFFFFFFF, $FFFACCCA, $FFFFFFFF, $FFFFFFFF, $DDADDAFD, $99DD9DDA
                LONG $D99D9D9D, $944994D9, $99999949, $99B99999, $9999B999, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $12221211, $12121222, $44612121, $94995264, $33659435
                LONG $28626223, $68686866, $33668686, $B99B9945, $B9BB99B9, $BBBBDB99, $EEEEEEDD, $EEEEFEEE
                LONG $FFFFFFFF, $FCCCCAFF, $FFFFFFFF, $FFFFACCF, $FFFFFFFF, $DFFAFFFF, $ADDADFDA, $DD9DD9DD
                LONG $9D49D9DD, $99994994, $99994994, $B9999999, $BBBB9999, $BB9B9BB9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $22121222, $21212121, $9B932121, $4349564B, $22659449
                LONG $66262222, $86666668, $95336666, $99B99B99, $B9B99B9B, $BBB9B99B, $EEEEEEEB, $EFEFFFFE
                LONG $FFFFFAFF, $CCCCCFFF, $FFFFFFFF, $FFFFFFFF, $FFFFFFFF, $FADFFAFF, $DDFDFDFF, $D9DDDDDD
                LONG $449D44D9, $949D49D4, $99999999, $99999999, $B9999BB9, $9BBBBB99, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $21211222, $12121221, $44B52212, $33943599, $22235494
                LONG $26262622, $66866266, $99443386, $9B99B999, $9B9BB9B9, $BDDDD9D9, $EEEE8EE9, $FEFADFDF
                LONG $FEFFACCA, $CCCCCCAF, $FFFFFFFB, $FFFFFFFF, $FFFFFFFF, $FDFFFFFF, $DDDDDADD, $DDD9DDD9
                LONG $D9D9D9D4, $49499499, $99949949, $B9999999, $9B9B9999, $BB9B9BBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $12122822, $21222122, $35942212, $259439B4, $62222333
                LONG $62622262, $86626862, $54944456, $99B99995, $99B9B99B, $DB9DDD4D, $DDEEE8E9, $FABCBADF
                LONG $AAFACCCA, $CCAFBCCC, $FFFFFFAC, $FFFFFFFF, $FFFFFFFF, $FFDFDFFF, $DADFDDFD, $94DD9DDD
                LONG $44444D4D, $949449D9, $99999994, $99999999, $9B99B9B9, $BBBBB99B, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $22112222, $12211222, $45946221, $25944999, $22622222
                LONG $26266222, $56862626, $35444944, $B99BB995, $D49B9B99, $99DD33DD, $DEDDDED9, $FBBCBCDF
                LONG $CCBCCBCD, $CCFFFFBC, $FFFFFACC, $FFFFEFFF, $FFFFFFFF, $DDFDFFDF, $DDDDFDDF, $DD9DD9DD
                LONG $49D4D444, $99949444, $99994949, $B9999999, $B9B9B999, $9B9B9BB9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $21228221, $22122121, $44446112, $26354435, $26222622
                LONG $62622626, $44536262, $56555449, $99B99945, $3D49B999, $D9D38E33, $DDEDEDDB, $BBCBBBCD
                LONG $ABCCBCFF, $BFFFFFFF, $EFFEFFAC, $FFFFFFEF, $FDFFFFFF, $DFDFFDFF, $DDDDDDFD, $D4DDDDDD
                LONG $4D4449D4, $44494494, $99999999, $99B99999, $9B9B99B9, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $12282111, $12121222, $66493222, $22265622, $22262226
                LONG $22222222, $44445362, $33355555, $99945445, $33549999, $9B9D3333, $DEDDDDD9, $BCBBBABF
                LONG $DCCBCFDB, $FFEFEFFF, $FEFFFFEF, $FFFFFFFF, $DFFDFFFF, $ADDFDFDF, $9DDDDDDD, $4D44D4DD
                LONG $4944DD4D, $99444944, $99949494, $99999999, $9B99B999, $B9B9B9BB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $21282211, $21212121, $26495222, $22222222, $62226222
                LONG $62626222, $54449453, $55555555, $99455545, $3D554999, $D99D38E3, $DDEEDDD9, $BBBCBCBD
                LONG $CCCCFDAC, $EFFFFEFD, $FFEFEFFF, $FFFFFFFF, $FDFFFDFF, $DFDDFDFD, $DDDDDDDD, $D4DD4D4D
                LONG $44D944D4, $49949D94, $99999999, $9B999999, $B9BB9B9B, $BBBB9B9B, $BBBBBBB9, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $22822111, $22222221, $22594612, $62222222, $22222222
                LONG $53622222, $55554494, $44555555, $94555994, $33354499, $BD4DD338, $ADD3EDD9, $BCBBBBCB
                LONG $CBCAFDCB, $FFEEFEFC, $FFFFEFEF, $FDFFFFFF, $DFFDFFDF, $DDFDDFDF, $DDDDDDDD, $4D44DDDD
                LONG $4944D444, $94494944, $99999494, $99999999, $9B99B9B9, $BB9BBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $28222211, $21222222, $22645622, $22222222, $22222222
                LONG $44556222, $65555544, $94444555, $44549BBB, $38835444, $B94D5DD3, $99DDDDD9, $CBABBBBB
                LONG $BCADDCCB, $EEFFEEFB, $FDFFFEFE, $DFFDFDFD, $DFDDFDFF, $DDDDDDDD, $4D4DDDDD, $4D4DD44D
                LONG $4444444D, $94994449, $99949999, $9B9B9999, $B9BB9B99, $BBBBB9B9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $11111111, $28226662, $22222212, $22226222, $22222222, $22222222
                LONG $44444566, $55555555, $99994444, $449CCBBB, $38385449, $99445453, $BBBD4D99, $BBCBCABA
                LONG $FADDCBBB, $FEFEDFEE, $EFDEFFDE, $FDFFFFFF, $FDDFDFDF, $DDDDFDFD, $DDDDDDDD, $44D44DD4
                LONG $4444D4D4, $49449494, $99999949, $B9999999, $9B99B9B9, $9BB9BBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $56111111, $82226659, $22222222, $22222222, $22222222, $66222222
                LONG $55444445, $44555555, $99999999, $9BCCCCBB, $55354444, $BB94D44D, $BABBD999, $ACBBBCBB
                LONG $EEDCBBBC, $EDEEEDED, $FDFDDFFD, $FDFDDFDD, $DFDDFDFD, $DDDDDDDD, $4DDD4DDD, $4D4D44DD
                LONG $4444444D, $94944944, $99999999, $9B99B999, $BBB9B99B, $BBBBB9B9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $99461111, $2223669B, $22222128, $22222222, $22222222, $45362222
                LONG $55555444, $99444555, $99999999, $BCCCCBB9, $44494544, $99999444, $BBBBBBD9, $CBBBBBBB
                LONG $DEDDACBB, $DEFDEEDE, $DFDFFEDF, $DFDFFDFF, $DDFDDFDD, $DDDDDDDD, $DD4DDDDD, $4444DD44
                LONG $94444444, $99499444, $99999994, $99B99999, $9B9B9BB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $11111111, $23B41656, $2226659B, $22222228, $21212221, $22222222, $44453622
                LONG $55555554, $99494444, $99B99999, $CCCCBB99, $449944D9, $99999944, $BBBBBBD4, $BBBBBCBB
                LONG $EDEDD9AB, $DFDDEDEE, $DFDFDDFE, $FDDDDDFD, $DDDDDDFD, $DDDDDDDD, $D4DDD4DD, $D4D4D44D
                LONG $44444D44, $94994949, $99999999, $9B999B99, $BB9BB99B, $BBB9B9B9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $46111111, $459924B9, $2211124B, $22222282, $22222222, $22222222, $55444556
                LONG $44555565, $B9944444, $9BB9999B, $CCCCBB94, $9449DABC, $99999999, $BBBBBDDD, $BBBBCBBA
                LONG $DEEDEEDD, $FDEFDEDE, $FDEDFDDD, $DDFDFDED, $DDDDDDDD, $DDDDDDDD, $4D4D4DD4, $44444DD4
                LONG $44444444, $99494494, $99999994, $B99B9999, $B9BB9BB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $9B411111, $49B41545, $21111116, $22212862, $22222221, $56222121, $55554445
                LONG $44445555, $BB994444, $4994999B, $CCCBB995, $999BBBCC, $3D999999, $BBBBD333, $DDBBBABB
                LONG $EDEEDEDE, $DDDDEDDD, $DDFDDEDE, $FDDDDDDD, $DDDDDDDD, $DDDDDDDD, $4D4DD4DD, $4D4D444D
                LONG $9444444D, $99949444, $99999999, $B9B99B99, $BB9BB9B9, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $29B21111, $26566991, $11111111, $12128682, $12122222, $44566222, $55555544
                LONG $44444455, $CBB99445, $5535449C, $CCBBB945, $99CACCCC, $335D4449, $BBB9DE88, $EEDDBBBB
                LONG $DED3E3E3, $EDEDDDED, $EDDDFDDD, $DDDDFDFD, $DDDDDDDD, $D4DDDDDD, $4D5D4DD4, $44444D45
                LONG $49444444, $94994994, $99999999, $9B9B999B, $BBBB9B9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $5BB21111, $111124B4, $11101111, $22222822, $22222212, $54445662, $55565655
                LONG $94454444, $CBB99944, $536654BC, $CCBBB945, $49CCCCCC, $88833544, $BBBD363E, $E33EDD9B
                LONG $EDDED3E3, $DDDDDEDD, $DDDEDDED, $DDDDDDDD, $DDDDDDDD, $5DDDDDDD, $D4D45D4D, $444D44D4
                LONG $94444444, $99949944, $99999999, $9B99B9B9, $B9B9B9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $46111111, $BB911116, $11111154, $11111111, $11222221, $62112122, $55544455, $44555565
                LONG $99994544, $CBB99999, $555559CC, $CBBB9945, $4CCACCCC, $88888335, $9BD83838, $3E838DDD
                LONG $DDD3DE3E, $EDDEDDDE, $DDDDDDDD, $DDDDDDDD, $DDDDDDDD, $DD4D4DDD, $4D4DD4D4, $44444D44
                LONG $44944444, $99494494, $B9999999, $9B9B9999, $BBBBBB9B, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB
                LONG $49462111, $64611116, $21111111, $11111111, $21122111, $55622121, $66555444, $44445555
                LONG $99999455, $BB999B99, $55554BCC, $BB9B9944, $DCCCBCCB, $E8888883, $3D83E383, $D3E3383E
                LONG $3EDE3DD3, $DDDDDEDD, $DDDDEDDD, $DDDDDDDD, $DDDDDDDD, $4DD5DD5D, $44444D4D, $4444444D
                LONG $94444444, $99999949, $99999999, $B9B9B9B9, $BBB9BBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $69BB5121, $11111112, $12221111, $11111111, $12211212, $44456222, $55665555, $55444455
                LONG $499B9945, $B999BB94, $94449BBB, $99999999, $DBADD499, $88882886, $38383838, $3D33E383
                LONG $DD3DD3DE, $DDEDDD3D, $DDDDDDDD, $DDDDDDDD, $D5DDDDDD, $5D4D5DDD, $54D4D54D, $44444444
                LONG $44949444, $99994994, $B9B99999, $BB9B9B99, $BBBB9B9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $49BB5693, $11111125, $21111111, $12111111, $21121212, $55544466, $55555655, $55544444
                LONG $44BBB945, $94449455, $99999BBB, $44999999, $5D386335, $88888828, $83338338, $3E3E33E3
                LONG $D3ED3E3D, $D3D3D3DD, $DDDDDDD3, $DDDDDDDD, $5DD5D5DD, $4D4D4D4D, $4D4544D4, $444444D4
                LONG $99444444, $99999949, $99999999, $B9B9B99B, $BB9BBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $499B55B9, $11111116, $11111111, $21212112, $56211111, $65555545, $44555566, $54444944
                LONG $54BCCB44, $95555365, $99999999, $35549999, $38181826, $68281888, $38383838, $3D33D333
                LONG $3DD3DD3D, $DDDDDD3E, $D3D3D3DD, $DD3DDD3D, $DD5DDDD5, $D4D5D4D5, $444D4D45, $44444444
                LONG $49494944, $99994994, $9B999999, $9B9B9BB9, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $164925BB, $11111111, $21111111, $11121211, $55562211, $66666554, $44455555, $44999999
                LONG $59CCBB94, $55656666, $999B9B99, $16555449, $68181181, $88888182, $E3383383, $D3D33D33
                LONG $D3D3E3D3, $3D3D3DD3, $DDDDDD3D, $5DD5DDDD, $4DDD5DDD, $4D4D4D5D, $444444D4, $44444444
                LONG $94949449, $99999999, $99B99999, $BBB9B99B, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $111215B9, $11111121, $11111111, $21211212, $55455621, $56666665, $94445555, $99999999
                LONG $5BCCB944, $45565666, $49999999, $12655554, $68818118, $66828828, $33333838, $333D33D3
                LONG $3D3D3D3E, $DDD3DD3D, $3D3D3DD3, $D5DD5D3D, $D545DD5D, $54D454D4, $4444544D, $49444444
                LONG $99494944, $99999994, $B999B999, $9B9B9BB9, $BBB9BBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $21111262, $11121122, $12111121, $22112121, $65544456, $55562226, $99454555, $99999999
                LONG $9BBB9944, $45555555, $54449999, $16566265, $32181181, $38682816, $D3333336, $D3D33333
                LONG $D3D3D333, $D33DD3D3, $DDDDDD3D, $5DD5D5DD, $54DDD5DD, $445D4D4D, $444D4444, $94444444
                LONG $94949494, $99999999, $9B9B9999, $BBBB9B9B, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $21111111, $11112212, $11121211, $56621212, $26549444, $54546222, $B9995555, $9955499B
                LONG $99BB9444, $99445554, $65554499, $81652112, $38118111, $36668282, $33333338, $3333D333
                LONG $3D3D3D3D, $D3D33D3D, $5D35D3D3, $D55DDD5D, $DD544D54, $4D445454, $444444D4, $44944444
                LONG $99994949, $99999999, $B9999B99, $B9B9B9B9, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $21111111, $21112222, $11112221, $44566211, $65444999, $55494612, $BB944545, $D36554BB
                LONG $99994554, $49999449, $12655554, $11212111, $36811818, $33836626, $33D33333, $D3D333D3
                LONG $D333D333, $DD3DD3D3, $5DD3D5D3, $DD555D5D, $44D5D4D5, $444D4D45, $44444444, $94494444
                LONG $49499949, $99999999, $9BB99999, $BB9BB9B9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $12112122, $11128222, $99445662, $55499999, $56549566, $B9945554, $366669CC
                LONG $99445555, $54449999, $12656555, $11111111, $36818111, $33366666, $33333333, $33333333
                LONG $3D3D33D3, $53D33D3D, $D55D5D3D, $55DDD5D5, $D54D455D, $454444D4, $44444444, $94944944
                LONG $99994994, $99999999, $B99B9B9B, $9BB9BB9B, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $12122221, $62122821, $99994456, $54999999, $55655555, $B9955554, $666659BC
                LONG $99955656, $55544999, $12566266, $18111811, $33111181, $33333666, $33333333, $D3D33D33
                LONG $33D33D33, $D5D5D3D3, $D5D5D5D3, $4D4555D5, $44D454D5, $44445454, $44444444, $94949449
                LONG $99999949, $99999999, $99B9B999, $BB9BB9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $21212221, $55628222, $99999944, $99499999, $55626654, $B9455554, $56654BBB
                LONG $99945566, $66555449, $12266111, $11181111, $33281811, $33333336, $33333333, $33333333
                LONG $D333D333, $3D35D353, $5D5D535D, $D55D4D5D, $D545D454, $44444444, $44444444, $49494944
                LONG $99999999, $9B999999, $BB9B99B9, $B9BB9B99, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $62111111, $21222266, $44442222, $99999999, $99999999, $43622654, $99945444, $65559BB9
                LONG $44994455, $11655554, $11111111, $11111122, $33211118, $33333333, $33333333, $D333D33D
                LONG $3D5333D3, $55D35D3D, $55D5D5D5, $4D4555D5, $44444D45, $4444D445, $94944444, $99949494
                LONG $99999949, $99999999, $9B99B99B, $BBB9BBBB, $BBBBB9B9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $95111111, $12122226, $9999D222, $99999999, $99999999, $D5221659, $9449944A, $55549999
                LONG $54449444, $12656655, $21111111, $18111666, $35818111, $33333333, $33333333, $3D333333
                LONG $53D3D533, $D35D3553, $5D555555, $5554D45D, $44D454D4, $44444444, $94444444, $49499494
                LONG $99999999, $9B9B9999, $B9BB9B99, $BBBBB9B9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $55111111, $21122212, $99BB4622, $99999994, $99999999, $A3226549, $4494499C, $49444994
                LONG $55544499, $12666226, $21111111, $11111653, $33611181, $33333333, $33333333, $33353333
                LONG $D5353335, $5555D3D5, $455D5D5D, $D44D55D5, $45454445, $44444444, $94949444, $99994994
                LONG $99999999, $B9999999, $9B99B9B9, $BB9B9B9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $51111111, $11222226, $99994322, $99999999, $99999999, $C3226559, $444499BA, $99999944
                LONG $26555444, $11266111, $11111111, $11111266, $63321111, $33333633, $33333333, $53533533
                LONG $353535D3, $5D555553, $5D555555, $45545454, $44444544, $44444444, $94944494, $99499494
                LONG $99999999, $99B99B99, $B9BB9B99, $B9BBBBB9, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BCBBBBBB
                LONG $61111111, $21211225, $99994322, $99999994, $99999999, $A8266554, $99444BCC, $44499999
                LONG $11265554, $11111110, $11111122, $81111112, $33362111, $33333333, $33333333, $35333335
                LONG $55D3D335, $55D35D55, $55D5D5D5, $54454545, $44444454, $44444444, $49499444, $99999999
                LONG $99999999, $9B9B9999, $BB9B99BB, $BBB9B9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $21222666, $49994612, $99999999, $99994444, $D2666559, $99999AAC, $54444494
                LONG $11656665, $21111111, $11111266, $11111111, $33333281, $33333333, $53333333, $53353533
                LONG $35353553, $55555355, $4454555D, $4454545D, $44445444, $49444444, $94944949, $99999949
                LONG $99999999, $B999999B, $B9B9BB99, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $21226621, $99994612, $55449994, $99455566, $32266549, $999BCCCA, $65544499
                LONG $11656626, $21111110, $11111255, $11111111, $36335311, $33333333, $33335333, $35535333
                LONG $55555535, $555D555D, $55555555, $45444454, $44444445, $94444444, $99499494, $99999999
                LONG $99999999, $99B9B9B9, $9BB9B99B, $BB9B9B9B, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BCBCBBBB
                LONG $11111111, $12266661, $99945212, $66654499, $99455226, $66666549, $9BBBCCAD, $12655549
                LONG $11166201, $21110111, $11111166, $11111111, $33333362, $33333336, $53533333, $53353353
                LONG $55533553, $5D555553, $45445455, $44454545, $44444444, $44949444, $99949494, $99999994
                LONG $9B999999, $BB999999, $B99B9BB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $12266621, $49995122, $26665549, $94456212, $66665499, $9BCCCCA3, $11166554
                LONG $11111111, $11111112, $11111111, $22111111, $33333336, $33333333, $33333533, $35355335
                LONG $55555355, $55555555, $45545545, $45445445, $44444444, $94444444, $94999949, $99999999
                LONG $99999999, $99BB9B9B, $BBB9B9B9, $BBB9B9B9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BCBBBBBB
                LONG $11111111, $12666611, $99946222, $22666554, $99445212, $66655499, $9BCCCCD3, $11266666
                LONG $21011010, $11011266, $11111111, $36221111, $33333333, $33333333, $35353335, $55533553
                LONG $55555555, $54555555, $54454554, $44444544, $44444444, $49994949, $99949499, $99999999
                LONG $9B9B9999, $BB99B999, $B9BB9B9B, $B9BBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBCBB
                LONG $11111111, $22666111, $44952256, $22666655, $99445221, $65555999, $69BBC945, $11265622
                LONG $61111111, $11110265, $11111111, $33332211, $33336333, $35333333, $53533533, $35355533
                LONG $55555535, $55555555, $44554545, $44454445, $44444444, $99449444, $99999994, $99999999
                LONG $B9999999, $999B99B9, $9BB9B9BB, $BBBB9B9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BCBCBBBB
                LONG $11111111, $22662111, $49456256, $22222265, $99456662, $55564999, $264B9945, $11126211
                LONG $21010010, $11111166, $11111110, $33335662, $33333333, $33353333, $35353335, $55535355
                LONG $55555555, $54545555, $45445454, $44444444, $94944444, $49494994, $99999499, $99999999
                LONG $99B9B9B9, $BBB9BB9B, $BB9BBB99, $BBBBBBBB, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $11111111, $26551111, $44952232, $22112126, $99456556, $55654999, $12654944, $11111110
                LONG $11101112, $11011111, $61111111, $33363533, $33333363, $53333533, $53535533, $35553535
                LONG $55555555, $45554555, $44544545, $44444454, $44444444, $99949449, $99999999, $99999999
                LONG $B9999999, $99B999B9, $9BB9B9BB, $BBB9BB9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $CBCBBCBB
                LONG $11111111, $16521111, $54461621, $62221125, $99945665, $56659999, $11264495, $21101011
                LONG $10111166, $11110111, $56621111, $36356333, $56565653, $35353333, $35353353, $55355553
                LONG $55555555, $45455545, $54454545, $44444444, $94949444, $94994994, $99999999, $99999999
                LONG $99B9B99B, $BB9BBB99, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBCBBBC
                LONG $11111111, $25611111, $44462211, $56622665, $99995565, $22654999, $10264446, $61011110
                LONG $11110265, $11111110, $33566211, $33563336, $33353336, $53356535, $53553553, $55555535
                LONG $55555555, $54545455, $44544454, $44444444, $49444444, $99999949, $99999999, $9B999999
                LONG $BB9B9999, $B9BB999B, $9B9B9B9B, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BCBBBBBB
                LONG $11111111, $65211111, $54562211, $65666555, $99999456, $11165499, $11164952, $61110101
                LONG $10101126, $10101111, $33653662, $33653335, $35363335, $35335333, $55535335, $55555355
                LONG $54555555, $45455554, $44445445, $44444444, $94949944, $49949494, $99999999, $99999999
                LONG $99B99B9B, $B9B9BBB9, $BBBBBBB9, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $CBBCBCBB
                LONG $11111111, $46111111, $49522116, $55655554, $99999944, $11126554, $10164561, $11011110
                LONG $11110112, $62111110, $65335536, $35363363, $65353563, $53533535, $35355553, $55555555
                LONG $55555555, $54544545, $44444544, $94444444, $49494494, $99999949, $99999999, $99B99999
                LONG $9B9B99B9, $BB9B99BB, $BB9B9B9B, $BB9BBBBB, $BBBBBBBB, $BBBBBBBB, $BCBBBBBB, $BCBBBBCB
                LONG $11111111, $51111111, $94562125, $44555444, $54499999, $11111265, $01264561, $11110011
                LONG $11011011, $56621111, $56363635, $56535353, $56533356, $35355335, $55535355, $55555555
                LONG $54545555, $45455454, $45444454, $44444444, $94949494, $99994999, $99999999, $B9999999
                LONG $9B9B9B99, $9BB9BB9B, $9BBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBCBCBBB
                LONG $11111111, $61111111, $94622165, $99949499, $65544999, $11111112, $11254511, $10101110
                LONG $10110111, $65566210, $63533533, $33356363, $65533565, $55556535, $55555553, $55555555
                LONG $45555555, $54454455, $44444444, $94944444, $49494944, $99999949, $99999999, $9B9B9B99
                LONG $B9B9B99B, $BB9BB9B9, $BBB9B9B9, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB, $CBBCBBBB, $BCBBBBCB
                LONG $11212111, $11111111, $95621255, $99999999, $12665449, $10100111, $01645611, $11011011
                LONG $11111110, $33355662, $53656333, $35365353, $53655653, $53355533, $55555355, $55555555
                LONG $55454555, $44545544, $44445444, $44444444, $99949494, $99999999, $99999999, $9999999B
                LONG $B9B9B9B9, $B9B9B9B9, $BB9BBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $CBBCBCBB
                LONG $66622222, $11111122, $45611652, $44999949, $11126654, $11011011, $12545211, $01101101
                LONG $62110111, $56565556, $65335656, $65356565, $55565535, $55533553, $55555553, $55555555
                LONG $45455454, $54444455, $44444444, $94949444, $49999499, $99999999, $99999999, $B9B9B999
                LONG $9B9B9B9B, $BBBB9B9B, $9BBB9B9B, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBCBBBBB, $BCBBBBBC
                LONG $65666666, $11112665, $45666561, $54449499, $11111265, $10110110, $15445110, $10011010
                LONG $53622010, $33353655, $53333535, $53335333, $33553353, $53555535, $55555555, $45555555
                LONG $54554555, $44545444, $44444444, $49494494, $99494994, $99999999, $B9999999, $9B9B99B9
                LONG $B9B9B9B9, $B9B9BBBB, $BBBBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $CBBBBCBB, $CBBCBCBB
                LONG $55544445, $11126666, $46265521, $66544494, $10011111, $11101101, $25456211, $01110101
                LONG $35536611, $33533533, $33535656, $65353353, $55535565, $55535356, $55555555, $55455555
                LONG $45445545, $44444445, $44444444, $94944944, $99999949, $99999999, $99B99999, $B9B99B99
                LONG $9B9BB9B9, $BBBB9B9B, $BBB9B9BB, $BBBBBB9B, $BBBBBBBB, $BBBBBBBB, $BBCBBBBB, $BBCBCBBB
                LONG $44949999, $12666655, $52126611, $12665444, $01101111, $11111111, $64452222, $11011011
                LONG $53555666, $53335353, $35656535, $53565335, $56535655, $55555555, $55555555, $55555555
                LONG $44554454, $44444544, $94444444, $99499494, $99994994, $99999999, $B9999999, $9B9B99B9
                LONG $BBB99B9B, $B9B9BBB9, $B9BBBBB9, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $CBBBCBBB, $CBBBBBCB
                LONG $54555549, $26665554, $51111111, $11126655, $11110101, $22111111, $55556622, $62211116
                LONG $35355555, $53533535, $35335336, $53556533, $55555533, $55555355, $55555555, $45454555
                LONG $54445545, $44444454, $49444444, $49949449, $99999999, $99999999, $999B9B99, $B9B9B99B
                LONG $9B9BB9B9, $BBBB9BBB, $BBBBB9BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBCBBBBB, $BCCBCBBB


Sprite_Size     BYTE $20
Sprite_Data     WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020



                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'}


{
' Balloon
Palette_Data    LONG $9EBDBCBB, $5B5AAB03, $02045D07, $02020202

Image_Data      LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $32111000, $44401222, $44444444
                LONG $00055555, $55555555, $55555555, $55555555, $55555555, $55555555, $00505555, $00000050
                LONG $00000000, $00000000, $00500000, $00000050, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $11111000, $44501111, $44444444
                LONG $50555555, $55555555, $55555555, $55555550, $55555555, $55555555, $60555555, $05060500
                LONG $00000000, $00000000, $06000000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $11111100, $75551111, $44747474
                LONG $50055555, $55555555, $05555555, $55555555, $55555555, $55555555, $00055550, $00000600
                LONG $00000000, $00000000, $00600000, $00000600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $11111100, $55551111, $44444444
                LONG $55005555, $55555555, $50555055, $55555555, $55555555, $50555555, $06005555, $06060006
                LONG $00000000, $00000000, $00000000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $11111110, $55551111, $74444444
                LONG $55655554, $55055555, $55000005, $55555555, $55055555, $55555555, $00600050, $00000600
                LONG $00000000, $00000000, $00060000, $00006000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $11111111, $05551111, $44474460
                LONG $55000554, $50555555, $55560555, $55555555, $55555555, $05555555, $00000600, $00600006
                LONG $00000000, $00000000, $60000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $11111111, $55555111, $44744500
                LONG $55555544, $00555555, $55550050, $55555555, $55505555, $55555555, $06006006, $06006000
                LONG $00000000, $00000000, $06000000, $00060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $10000000, $11100011, $55551111, $44444005
                LONG $55505556, $55055555, $55555065, $55555555, $55555555, $55555555, $00600005, $00000600
                LONG $00000000, $00000000, $00600000, $00006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $11000000, $11100001, $05550111, $44640060
                LONG $55500544, $05555555, $55555500, $55555555, $55055555, $55555555, $60006055, $60060060
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $11000000, $11000001, $55551111, $46500000
                LONG $55555554, $60005555, $55555550, $55555555, $55555555, $55555555, $00000055, $00600000
                LONG $00000000, $00000000, $06000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $11100000, $11100001, $50555111, $40006060
                LONG $55550556, $50605555, $55555550, $55555555, $55055555, $55555555, $60606555, $00000600
                LONG $00000000, $00000000, $00060000, $00000606, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $01110000, $11110000, $65551111, $00600000
                LONG $55550555, $00050555, $55555550, $55555555, $55505555, $55555555, $00000555, $06060060
                LONG $00000000, $00000000, $00000000, $00500000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $01110000, $33333111, $05501883, $60000060
                LONG $55555550, $06005505, $55555556, $55555555, $55555555, $55555555, $60555555, $00000000
                LONG $00000000, $00000000, $06000000, $00006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $33111000, $88888883, $05552888, $00060600
                LONG $05555500, $00060555, $55555555, $55555555, $55555555, $55555555, $00555555, $50600600
                LONG $00000000, $00000000, $00060000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $33321100, $88838388, $65502838, $00600000
                LONG $55555006, $50000055, $55555555, $55555555, $55555555, $55555555, $05555555, $00006006
                LONG $00000000, $00000000, $00000000, $00006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $33331100, $83888838, $05513883, $60000600
                LONG $00555500, $55606005, $55555555, $55555555, $55555555, $55555555, $55555555, $06000000
                LONG $00000000, $00000000, $00600000, $00600000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $83332110, $88883888, $05413888, $00060006
                LONG $55555060, $55000060, $55555555, $55555555, $55555555, $55555555, $05555555, $00556060
                LONG $00000000, $00000000, $06000000, $00006060, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $33333111, $83838883, $00518883, $06000600
                LONG $05555000, $55506005, $55555555, $55555555, $55555555, $55555555, $45555555, $06000000
                LONG $00000000, $00000000, $60000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $10000000, $33333311, $88888333, $04028388, $50060006
                LONG $55555060, $55500060, $55555555, $55555555, $55555555, $55555555, $55555555, $00060555
                LONG $00000000, $00000000, $00600000, $00006000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $10000000, $10000000, $33333321, $38383333, $55538838, $06000000
                LONG $55555000, $55555005, $55555555, $50555555, $55555555, $55555555, $55555555, $06005555
                LONG $00000000, $00000000, $00000000, $00000606, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $10000000, $11000000, $33333331, $88833333, $55128888, $00060605
                LONG $55555006, $55550605, $55555555, $55505555, $55555555, $55555555, $05555555, $00005555
                LONG $00000000, $00000000, $06000000, $00060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $11100000, $33333333, $83323333, $55138383, $06000005
                LONG $55555060, $55555000, $55555555, $55555555, $55555555, $55555555, $55555555, $00655555
                LONG $00000000, $00000000, $00000000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $21100000, $33333333, $33333333, $55188888, $00060655
                LONG $55550500, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55055550
                LONG $00000000, $00000000, $06000000, $00006000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $31110000, $33333333, $33333333, $55288883, $50000055
                LONG $55550060, $55555505, $55555555, $55550555, $55555555, $55555555, $55555555, $55555505
                LONG $00000000, $00000000, $00000000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $33111000, $33333333, $92233333, $50233333, $60605555
                LONG $55556000, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55550550
                LONG $00000000, $00000000, $06000000, $00060600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $33311000, $AAA9A123, $74777AAA, $57999997, $00055555
                LONG $55550060, $55555505, $55555555, $55555505, $55555555, $55555555, $55555555, $55555050
                LONG $00000000, $00000000, $00000000, $60000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00010000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $7A111100, $44444477, $44444444, $57999977, $60455555
                LONG $55500000, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55550555
                LONG $00000000, $00000000, $06000000, $00060006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00011000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $44477110, $44444444, $44444444, $57999A74, $05555555
                LONG $55506000, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00000000, $00000000, $00000000, $00006000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00121000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $44444110, $44444444, $44447444, $5A99A744, $05555555
                LONG $55500606, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555050
                LONG $00000000, $00000000, $06000000, $00600000, $00000006, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $01331000, $10000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00110000, $10000000, $44444411, $47474744, $47444447, $799AA744, $55555555
                LONG $55500005, $55555555, $55555555, $55555555, $55555555, $55555555, $50555555, $55500555
                LONG $00000000, $00000000, $00000000, $00006000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $13321000, $10000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00110000, $11000000, $44744471, $44444447, $44444444, $59AAA744, $55555555
                LONG $55550655, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00000000, $00000000, $00000000, $00600006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $11221100, $00000001, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $01111000, $11000000, $44444447, $44444444, $44747444, $799AA744, $55555555
                LONG $55050055, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00000002, $00000000, $60000000, $00000600, $00000060, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $11111110, $00000000, $00111000, $00000000, $00000000, $00000000
                LONG $00000000, $11111100, $51100000, $44744444, $47447447, $74444447, $99AAA744, $55555555
                LONG $55055555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00000013, $00000000, $00000000, $60060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $22211110, $00001112, $11000000, $00000001, $00000000, $00000000
                LONG $00000000, $11212310, $70100001, $44444447, $44444444, $44444444, $9AAAA744, $5555555A
                LONG $55600555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00000023, $00000000, $00600000, $00000600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $11131110, $00000001, $00000000, $00000010, $00000000, $00000000
                LONG $00000000, $33333320, $50111011, $44446444, $44474444, $44744744, $99AAA744, $5555555A
                LONG $50004555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00000133, $00000000, $00000000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $33333211, $40011333, $44444444, $47444444, $74444444, $99AAA744, $555555A9
                LONG $50555055, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00000333, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $33333110, $40133233, $44444444, $44444444, $44474444, $9AAAA744, $55555799
                LONG $50000555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00002333, $00000000, $00000000, $06060600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111000, $11133331, $77777777, $44477777, $44444664, $99AAAA74, $55555799
                LONG $00605555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00013333, $00000000, $60600000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $11112310, $11111111, $11111111, $11111111, $29111111, $55555122
                LONG $06005055, $55555550, $55555555, $55555555, $65555555, $55555554, $55555555, $55555555
                LONG $00023333, $00000000, $00000000, $00060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $11111100, $11111111, $11111111, $11111111, $33211111, $55551233
                LONG $00060550, $55555555, $55555555, $55555555, $66555555, $55555556, $55555555, $55555555
                LONG $00033333, $00000000, $06000000, $00000060, $00000000, $00000000, $00000000, $00500000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $10000000, $11111111, $11111111, $11111111, $33331111, $55551333
                LONG $06005555, $55555550, $55555555, $55555555, $66455555, $55555546, $55555555, $55555555
                LONG $00023333, $00000000, $00000000, $00000600, $00000006, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $10000000, $11111111, $11111111, $11111111, $33321111, $55551333
                LONG $00055555, $55555556, $55555555, $55555555, $66655555, $55555566, $55555555, $05555555
                LONG $00013333, $00000000, $06000000, $00606006, $00000000, $00000000, $00000000, $00060000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10000000, $10110050, $11111111, $11111111, $11111111, $33211111, $55513333
                LONG $06005555, $55555550, $55555555, $55555555, $66665555, $55555666, $55555555, $05555555
                LONG $00001331, $00000000, $00600000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11000000, $11011550, $11111110, $11111111, $11111111, $33331111, $55523333
                LONG $00555555, $55555500, $55555555, $55555555, $66664555, $55556666, $55555555, $55555555
                LONG $00001331, $00000000, $00000000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11110000, $11111551, $11111111, $11111110, $11111111, $33321111, $50133333
                LONG $55555555, $55555506, $55555555, $55555555, $66666555, $55546666, $55555555, $55555555
                LONG $00002320, $00000000, $00000000, $60600600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111000, $00111550, $11001100, $11111111, $11111111, $33331111, $51233333
                LONG $55555555, $55555555, $55555555, $55555555, $66666555, $55466666, $55555555, $55555555
                LONG $00003310, $00000000, $00000000, $00006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111100, $10001550, $11001101, $11111111, $11111111, $33311111, $51333333
                LONG $55555555, $55555555, $55555555, $55555555, $66666645, $54666666, $55555555, $55555555
                LONG $00003310, $00000000, $06000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111110, $00011001, $11111100, $11011111, $11111111, $33321111, $12333333
                LONG $55555555, $55555555, $55555555, $55555555, $66666665, $56666666, $55555555, $55555555
                LONG $00023100, $00000000, $00000000, $00000606, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111110, $11111001, $00010011, $11100000, $11111111, $33211111, $13333333
                LONG $55555555, $55555555, $55555555, $55555555, $66666664, $66666666, $55555555, $55555555
                LONG $00032000, $00000000, $06000000, $00006000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111110, $11111111, $11111111, $11100011, $11111111, $33321111, $13333333
                LONG $55555555, $55555555, $55555555, $55555555, $66666665, $66666666, $55555556, $55555555
                LONG $00010006, $00000000, $00000000, $60060006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111110, $11111111, $11111111, $11111111, $11111111, $33211111, $33333333
                LONG $55555551, $55555555, $55555555, $55555555, $66666666, $66666666, $55555566, $55555555
                LONG $00100000, $00000000, $06000000, $06000000, $00000000, $00000000, $00000000, $00000050
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111111, $11111101, $11111111, $11111111, $11111111, $33211111, $33333333
                LONG $55555512, $55555555, $55555555, $65555555, $66666666, $66666666, $55555666, $55555555
                LONG $00100600, $00000000, $00000000, $00006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111111, $11111111, $11111111, $11111111, $11111111, $33333211, $33333333
                LONG $55555123, $55555555, $55555555, $65555555, $66666666, $66666666, $55556666, $55555555
                LONG $00150000, $00000000, $06000000, $00000000, $00000000, $00000000, $00000000, $00000600
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111111, $11111111, $11111111, $11111111, $11111111, $33333111, $33333333
                LONG $55555133, $55555555, $55555555, $64555555, $66666666, $66666666, $55466666, $55555555
                LONG $00000600, $00000000, $00000000, $60600606, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111111, $11111011, $11111111, $11111111, $11111111, $33332111, $33333333
                LONG $55550233, $55555555, $55555555, $66455555, $66666666, $66666666, $54666666, $55555555
                LONG $00000005, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111111, $11111011, $11111111, $11111111, $11111111, $33333211, $33333333
                LONG $55551333, $55555555, $55555555, $66655555, $66666666, $66666666, $56666666, $55555555
                LONG $00060000, $00000000, $06000000, $00006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000100, $11111110, $11111011, $11111111, $11111111, $11111111, $33332111, $33333333
                LONG $55513333, $55555555, $55555555, $66645555, $66666666, $66666666, $66666666, $55555555
                LONG $00000060, $00000000, $00000000, $00060060, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111111, $11100111, $11111111, $11111111, $11111111, $33333311, $33333333
                LONG $55513333, $55555555, $55555555, $66655555, $66666666, $66666666, $66666666, $55555546
                LONG $00600600, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111110, $10000011, $11111100, $11111111, $11111111, $33323111, $33333333
                LONG $55133333, $55555555, $55555555, $66665555, $66666666, $66666666, $66666666, $55555666
                LONG $00060000, $00000000, $06000000, $00006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11111111, $00000000, $11110000, $11111111, $11111111, $33323211, $33333333
                LONG $55133333, $55555555, $55555555, $66665555, $66666666, $66666666, $66666666, $55556666
                LONG $00000000, $00000000, $00000000, $00000600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00111110, $00000000, $11000000, $11111111, $11111111, $33332111, $33333333
                LONG $51333333, $55555555, $55555555, $66664555, $66666666, $66666666, $66666666, $55556666
                LONG $00600060, $00000000, $60000000, $60060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $11111110, $11111111, $33331111, $33333333
                LONG $01233333, $55555555, $55555555, $66665555, $66666666, $66666666, $66666666, $55466666
                LONG $06006000, $00000000, $00000000, $00600060, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000001, $00000000, $11000000, $11111111, $32121111, $33333333
                LONG $01113333, $55555555, $55555555, $66666555, $66666666, $66666666, $66666666, $66666666
                LONG $00000060, $00000000, $60000000, $00006000, $00000000, $00000000, $00600000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000001, $00000000, $00000000, $11111110, $33112111, $23333333
                LONG $00111212, $55555550, $55555555, $66666455, $66666666, $66B66666, $66666666, $66666666
                LONG $06060000, $00000006, $00000000, $00000060, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $11111110, $11111111
                LONG $00111011, $55555550, $55555555, $66666555, $66666666, $66666666, $66666666, $66666666
                LONG $00000060, $00000000, $06000000, $00060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00011000
                LONG $00010000, $55555500, $55555555, $66666555, $66666666, $66666666, $66666666, $66666666
                LONG $60000000, $00000000, $00000000, $06000606, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10000000, $00000001, $00000000, $00000000, $00000000, $00000000, $00011000
                LONG $00001000, $55555500, $55555555, $66666455, $66666666, $66666666, $66666666, $66666666
                LONG $06000600, $00000000, $00000000, $00000000, $00000000, $00000000, $00060000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10000000, $00000001, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00001000, $55555000, $55555555, $66666655, $66666666, $66666666, $66666666, $66666666
                LONG $00000060, $00000006, $06000000, $06006006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10000000, $00000001, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $55550000, $55555555, $66666655, $66666666, $66666666, $66666666, $66666666
                LONG $06000000, $00000000, $00000000, $00000060, $00600000, $00000000, $00006000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $55550000, $55555555, $66666645, $66666666, $66666666, $66666666, $66666666
                LONG $00060060, $00000000, $06000000, $00060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $55500000, $55555555, $66666655, $66666666, $66666666, $66666666, $66666666
                LONG $60000000, $00000060, $00000000, $06000606, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $54500000, $55555555, $66666645, $66666666, $66666666, $66666666, $66666666
                LONG $00006060, $00000000, $00000000, $00000000, $00000000, $00000000, $00006000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $11000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $64000000, $55555554, $66666655, $66666666, $66666666, $66666666, $66666666
                LONG $60000000, $00000060, $60000000, $00000600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $45000000, $55555546, $66666655, $66666666, $66666666, $66666666, $66666666
                LONG $00006006, $00000000, $00000000, $00006006, $00050000, $00000000, $00000605, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $50000000, $55555466, $66666645, $66666666, $66666666, $666666BB, $66666666
                LONG $06000600, $00000060, $06000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10000000, $55546667, $66666655, $66666666, $66666666, $B6666666, $66666666
                LONG $00060000, $00000006, $00000000, $00000606, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $31000000, $55666649, $66666645, $66666666, $66666666, $66666666, $66666666
                LONG $00000060, $00000600, $00000000, $00606000, $00000000, $00000000, $00000060, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $82000000, $56666678, $66666665, $66666666, $66666666, $666666B6, $66666666
                LONG $00006000, $00000006, $06000000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $88200000, $66666498, $66666666, $66666666, $66666666, $6B6666B6, $66666666
                LONG $00000060, $00000600, $00000000, $00000600, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $88810000, $66666738, $66666666, $66666666, $66666666, $6666BB66, $66666666
                LONG $00600000, $00000000, $60000000, $00600000, $00000000, $00000000, $00000006, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $88830000, $66666988, $66666666, $66666666, $B6666666, $6666BB66, $66666666
                LONG $00006060, $00000060, $00000000, $00006060, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00060000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $11100000
                LONG $00000000, $88883100, $66667388, $66666666, $66666666, $6B666666, $BBBB6666, $666666BB
                LONG $00600000, $00006000, $60000000, $00000000, $00000000, $60000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10000000, $00000011, $00000000, $00000000, $00000000, $00000000, $33310000
                LONG $00000001, $88888310, $6666A888, $66666666, $66666666, $B6666666, $BBB66BB6, $6666666B
                LONG $00006000, $00000060, $00600000, $00000060, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00606000, $00000000, $00000000
                LONG $00000000, $21000000, $00000111, $00000000, $00000000, $00000000, $00000000, $33331000
                LONG $00000013, $88888810, $66642888, $66666666, $66666666, $66666666, $B66B666B, $66666666
                LONG $00000060, $00006000, $60000000, $00060000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000060, $00000000, $00000000
                LONG $00000000, $33310000, $00111111, $00000000, $00000000, $00000000, $00000000, $33333100
                LONG $00001233, $88888881, $664A8888, $66666666, $66666666, $6B666666, $BB6BB666, $6666B66B
                LONG $00606000, $00060000, $00000000, $06000600, $00000000, $06000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $50000000, $00006000, $00000000, $00000000
                LONG $00000000, $33332100, $11111111, $00000011, $00000000, $00000000, $00000000, $33333321
                LONG $10012333, $88888883, $66738888, $66666666, $66666666, $66666666, $666B6666, $6666B6B6
                LONG $06000060, $00000600, $00600000, $00000006, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00600000, $00000060, $00000000, $00000000
                LONG $00000000, $13333310, $11111111, $00111111, $00000000, $00000000, $21000000, $33333333
                LONG $30123333, $88888888, $64988888, $66666666, $66666666, $66666666, $B6B66666, $66666B66
                LONG $00006000, $00000000, $60000000, $00000000, $00000000, $00600000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $60000000, $00060000, $00000000, $00000000
                LONG $11000000, $13333332, $11111111, $12111111, $00001111, $00000000, $33211000, $33333333
                LONG $83333333, $88888888, $64388888, $66666666, $66666666, $66666666, $BBB66666, $B6BB66B6
                LONG $60600000, $00060600, $00000000, $00000060, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00060000, $00000060, $00000000, $00000000
                LONG $33310000, $13333333, $11111111, $33311111, $12333333, $32111111, $88833333, $33333338
                LONG $88833333, $88888888, $6A888888, $66666666, $66666666, $66666666, $BBBB6666, $6BBBB6BB
                LONG $00006060, $00000000, $06000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $60000000, $00060600, $00000000, $00000000
                LONG $33331111, $13333333, $11111111, $33321111, $33333333, $33333333, $33888333, $33333333
                LONG $88883233, $88888888, $42888888, $66666666, $66666666, $66666666, $6BBB6666, $66B666B6
                LONG $60000000, $00606000, $60000000, $00500000, $00000000, $00600000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00606000, $00000006, $00000000, $11100000
                LONG $33333211, $13333333, $11111111, $33332111, $33333333, $33333333, $38833833, $33333333
                LONG $88883333, $88888888, $78888888, $66666666, $66666666, $66666B66, $BBBB6666, $666B6B6B
                LONG $06060060, $00000000, $00000000, $00060000, $00000000, $00005000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $11000000, $00000111, $05055000, $11100000, $11111111
                LONG $33332111, $13333333, $11111111, $33333111, $33333333, $33333333, $33888833, $33333333
                LONG $88888332, $88888888, $98888888, $66666666, $66666666, $666666B6, $BBBBB66B, $666B66BB
                LONG $00006000, $00600600, $60000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $11100000, $11111111, $11111111, $11111111, $11111111
                LONG $33331111, $11333333, $11111111, $33333111, $33333333, $33333333, $32388333, $33323333
                LONG $88888833, $88888888, $88888888, $66666667, $66666666, $66666666, $BBBBBB66, $B6B6BBBB
                LONG $06000060, $00000006, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $11110000, $11111111, $11111111, $11111111, $11111111
                LONG $32211111, $11333223, $11111111, $33333111, $33333333, $33333333, $A7938883, $32333333
                LONG $88888883, $88888888, $88888888, $66666669, $66666666, $66666666, $BBBBBB66, $B666BBBB
                LONG $00060000, $00606000, $60000000, $00000000, $00000000, $00000000, $00000000, $00010000
                LONG $00000000, $00000000, $00000000, $11111000, $11111111, $11111111, $11111111, $11111111
                LONG $33111111, $51233323, $11111111, $33332111, $33332333, $33333333, $44723888, $33333331
                LONG $88888888, $88888888, $88888888, $66666678, $66666666, $666666B6, $BBBBBBB6, $6BB66BBB
                LONG $00006000, $00000006, $06000000, $00000000, $00000000, $00000010, $00000110, $00001100
                LONG $00000000, $00001000, $00000000, $11111100, $11111111, $11111111, $11111111, $11111111
                LONG $33311111, $664A2333, $11111546, $33321111, $33333333, $33333333, $4447A988, $833332A4
                LONG $88888888, $88888888, $88888888, $66666738, $6666B666, $66666666, $BBBBBBBB, $66BBBBBB
                LONG $60600060, $00606000, $00000000, $00011000, $00000000, $00000001, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $11111110, $11111111, $11111111, $11111111, $11111111
                LONG $33321111, $B6466792, $11746666, $33311111, $32333333, $83323333, $44464793, $88333A44
                LONG $88888888, $88888888, $88888888, $66666988, $66666B66, $66666666, $BBBBBBBB, $6B6BBBBB
                LONG $00006000, $05000000, $00011000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $50000000, $11111111, $11111111, $11111111, $11111111, $11111111
                LONG $11211111, $BB666667, $44666666, $32321117, $33333333, $92333333, $474447A9, $8332A444
                LONG $88888888, $88888888, $88888888, $66664288, $6666B666, $B6666666, $BBBBBBBB, $66BBBBBB
                LONG $05000060, $00000005, $00500010, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $B6000000, $11111756, $11111111, $11111111, $11111111, $11111111
                LONG $64511111, $66666666, $66666666, $91557666, $22222912, $AA922222, $4644467A, $8883A444
                LONG $88888888, $88888888, $88888888, $66669888, $666B6666, $B66666B6, $BBBBBBBB, $BBBBBBBB
                LONG $05550005, $00000000, $60000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $BBB00000, $1174466B, $11111111, $11111111, $11111111, $11111111
                LONG $66664711, $6B666666, $66666666, $44444666, $44447444, $AAA77444, $4666666A, $83389744
                LONG $88888888, $88888888, $88888888, $66673888, $66BB6666, $BB6B6666, $BBBBBBBB, $BBBBBBBB
                LONG $00000000, $00600605, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $BBBB0000, $7666B6BB, $111A1117, $11111111, $11111111, $71111111
                LONG $66666664, $66B66664, $66666666, $44744666, $44444444, $AAAAA744, $66666664, $88888A46
                LONG $88888888, $88888888, $88888888, $66698888, $66B66666, $BBBBBBB6, $BBBBBBBB, $BBBBBBBB
                LONG $60606060, $00006000, $06000000, $00000000, $00000000, $00000005, $00000000, $00000000
                LONG $00006000, $00000000, $BBBB6600, $BBB6BBBB, $55744666, $55717155, $55557155, $66664755
                LONG $66666666, $66B66666, $66666666, $44444466, $44744444, $7AAAA747, $66666666, $88383274
                LONG $88888888, $88888888, $88888888, $66788888, $66B66666, $BBBBBB66, $BBBBBBBB, $BBBBBBBB
                LONG $00000000, $00600060, $00000000, $00000600, $00000000, $00000000, $00000000, $00000000
                LONG $00060060, $60600000, $BBBBB600, $BB66BBBB, $6BBBBBBB, $6666BB6B, $66666666, $66666666
                LONG $66666666, $6BB66666, $66666666, $74444466, $44444744, $67AAA744, $46666666, $88333339
                LONG $88888888, $88888888, $88888888, $64988888, $6B666666, $BBBB6666, $BBBBBBBB, $BBBBBBBB
                LONG $60060600, $00000000, $60000000, $00000000, $00000000, $00000000, $00060000, $00000000
                LONG $06000000, $00060606, $BBBBBBB5, $666BBBBB, $BBB6BBBB, $BBBBBBBB, $B6666BBB, $66666666
                LONG $66666666, $BBB66666, $66666666, $44744466, $44474444, $667AAA74, $A6666666, $83338888
                LONG $88888888, $88888888, $88888888, $49888888, $BBB66666, $BBB66666, $BBBBBBBB, $BBBBBBBB
                LONG $06000060, $00606060, $00000000, $00000000, $00000000, $00000000, $00000600, $00000000
                LONG $00000006, $60000000, $BBBBBBB6, $BBB6BBBB, $BBBBBBBB, $BBBBBBBB, $66666BBB, $66666666
                LONG $66666666, $6BB66666, $66666666, $44444446, $47444474, $667AAA74, $3A666666, $88383388
                LONG $88888888, $88888888, $88888888, $A8888888, $BB666666, $BBBB666B, $BBBBBBBB, $BBBBBBBB
                LONG $00060000, $00000000, $05000000, $00000600, $50000000, $00000000, $00050006, $00000000
                LONG $60060060, $66000600, $BBBBBBBB, $BB6BBBBB, $BBBBBBBB, $BBBBBBBB, $6666BBBB, $66666666
                LONG $66666666, $6BBB6666, $66666666, $47444744, $44444444, $6557AAA7, $83A66666, $88883338
                LONG $88888888, $88888888, $88888888, $38888888, $B66666B4, $BBBB666B, $BBBBBBBB, $BBBBBBBB
                LONG $60000600, $06000600, $00000000, $00000000, $00000000, $00000006, $00000600, $60600000
                LONG $00600000, $BB606006, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $B66BBBBB, $66666666
                LONG $66666666, $6B645666, $66666666, $44474466, $44444744, $5557A7A7, $88376666, $88833238
                LONG $88888888, $88888888, $88888888, $88888888, $B666BB6A, $BBBBB66B, $BBBBBBBB, $BBBBBBBB
                LONG $00605000, $00600060, $06000000, $00060000, $05000000, $60000000, $00000000, $00000000
                LONG $00006000, $BBB60000, $BBBB66BB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $666BBBBB, $666666B6
                LONG $66666666, $44555546, $66666664, $74444446, $74644444, $555011A7, $88824645, $88882133
                LONG $88888888, $88888888, $88888888, $88888888, $BBBBBB43, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $60005000, $60006000, $00000000, $00000606, $00000060, $00000006, $00050060, $00600000
                LONG $06000606, $BBBB6060, $BBB66BBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $6BBBBBBB, $6666666B
                LONG $46666666, $55551115, $66665555, $44444466, $A7744444, $55555111, $33883155, $88883212
                LONG $88888888, $88888888, $88888888, $88888888, $BBBBB498, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $00060000, $00600000, $00000000, $06060000, $06000600, $60600000, $00000500, $00006000
                LONG $60060000, $BBBBB600, $BBBBBBBB, $B666BBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $66666B6B
                LONG $55546666, $55555105, $15111115, $777AA515, $11117777, $55555511, $38332155, $8888881A
                LONG $88888888, $88888888, $88888888, $88888888, $BBBB6288, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $60000000, $60006060, $60000060, $00000600, $00000006, $00000000, $00000000, $00600000
                LONG $00000060, $BBBBB660, $BBBBBBBB, $54666BBB, $BBBBBBB6, $BBBBBBBB, $BBBBBBBB, $6B6666BB
                LONG $55555466, $55555105, $11111115, $11111111, $11111111, $55555551, $32123115, $88888333
                LONG $88888888, $88888888, $88888888, $88888888, $BBBB9888, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $06060600, $00600000, $00000000, $00060060, $06000000, $06006000, $00000006, $60000600
                LONG $00606000, $BBBBBBB6, $BBBBBBBB, $5566666B, $BBB66455, $BBBBBBBB, $BBBBBBBB, $5546666B
                LONG $55555555, $55555015, $11111100, $11111111, $11111111, $15555551, $11111122, $88888833
                LONG $88888888, $88888888, $88888888, $88888888, $BBBA8888, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $00000060, $60000060, $06050600, $06000000, $00000606, $00600600, $00000000, $00060060
                LONG $B0000060, $BBBBBBBB, $666BBBBB, $56666666, $45455555, $6B66B666, $545666B6, $55555554
                LONG $55555555, $00555515, $11111110, $11111111, $11111111, $21555555, $2AA12233, $88888322
                LONG $88888888, $88888888, $88888888, $88888888, $BB788888, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $60060000, $00006000, $00060006, $00000606, $00050000, $00000000, $50000000, $06000000
                LONG $BB006000, $BBBBBBBB, $666666BB, $66666666, $55555555, $64555555, $55555544, $55555555
                LONG $55555555, $00555115, $11111111, $11111111, $51111111, $25555555, $12333333, $88888821
                LONG $88888888, $88888888, $88888888, $88888888, $49388888, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $00600600, $06060060, $06000600, $00060000, $00605006, $06006000, $00000000, $00060060
                LONG $B6400060, $66B6B6B6, $66666666, $66666666, $55555555, $65555555, $55555566, $55555555
                LONG $55555555, $00555555, $11111110, $11111111, $51111111, $15455555, $33333333, $88888882
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $BBBBBB49, $BBBBBBBB, $BBBBBBBB
                LONG $60000060, $00000000, $00550006, $00000600, $00000000, $50000600, $06000005, $06000006
                LONG $55555006, $66666444, $66644454, $66666666, $55555556, $65455555, $55555546, $55555555
                LONG $55555555, $00055550, $11111111, $11111111, $55511111, $25665555, $33333333, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $BBBBB628, $BBBBBBBB, $BBBBBBBB
                LONG $00006000, $06006060, $60000600, $00000060, $00000060, $06060000, $00000000, $00000600
                LONG $55665500, $66544555, $66666456, $66666666, $55555556, $65555555, $55555556, $55555555
                LONG $55555555, $11055500, $11111111, $11111111, $55511111, $25664555, $33333333, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $BBBBB738, $BBBBBBBB, $BBBBBBBB
                LONG $06060000, $00060000, $00000000, $60606000, $00600000, $00000060, $00600000, $00606006
                LONG $54666650, $66665555, $66666664, $66666666, $55555566, $65555555, $55555556, $55555555
                LONG $55555555, $11000550, $11111111, $11111111, $11111111, $14664551, $83133333, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $BBB6A388, $BBBBBBBB, $BBBBBBBB
                LONG $50000606, $60000060, $06556060, $00000060, $00000050, $00506000, $06006000, $60000000
                LONG $46646660, $64545555, $66546656, $66666666, $55555466, $55655555, $55555566, $55555555
                LONG $10555555, $11000001, $11111111, $11111111, $11111111, $56666401, $88222331, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $BB428888, $BBBBBBBB, $BBBBBBBB
                LONG $60060000, $06006000, $00500000, $00000006, $00000050, $00500060, $00000000, $05506006
                LONG $66666BBB, $65555556, $66664566, $66666666, $55555666, $65555555, $55555646, $55555555
                LONG $11105555, $11111101, $11111111, $11111111, $11111111, $BB666511, $88311235, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $BB498838, $BBBBBBBB, $BBBBBBBB
                LONG $06000600, $00600060, $05500060, $00000600, $00000000, $50550000, $55055655, $B5500555
                LONG $66666BBB, $46555466, $66666664, $66666666, $55555566, $64555555, $55555566, $55555555
                LONG $11110005, $11111110, $11111111, $11111111, $31111111, $647A9238, $88831554, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $BB428888, $BBBBBBBB, $BBBBBBBB
                LONG $50060000, $60006005, $05060600, $11110055, $65333221, $B6511555, $B6555BBB, $B555A56B
                LONG $66BBBB6B, $66555566, $54664646, $44666666, $05005554, $66545555, $55555556, $00005050
                LONG $11111010, $11111111, $11111111, $11111111, $88211111, $33888888, $88832199, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $B6513888, $BBBBBBBB, $BBBBBBBB
                LONG $00000606, $00600000, $11000050, $88888832, $69888833, $BB011006, $BB4056BB, $66652156
                LONG $6BB6BBBB, $65500056, $66666666, $55554666, $00055555, $55555000, $00000555, $00000000
                LONG $11111000, $11111111, $11111111, $11111111, $83311111, $88838883, $88888888, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $64553838, $BBBBBBBB, $BBBBBBBB
                LONG $06000000, $2A005606, $21233332, $88383888, $13888333, $6BB01506, $6B6555BB, $B6511145
                LONG $BBBB6B6B, $55000056, $64646464, $55550555, $00555555, $55000000, $00000555, $00000000
                LONG $11110000, $11111111, $11111111, $11111111, $83332111, $83883888, $83838388, $88888888
                LONG $23233888, $83239333, $88888888, $88888888, $88888888, $6555A883, $BBBBBBBB, $BBBBBBBB
                LONG $50556050, $38310050, $32388888, $33888888, $33233322, $BBB60001, $05B5000B, $BB651155
                LONG $BB6BBBBB, $0000066B, $00000000, $55505555, $00555555, $50000000, $00000505, $00000000
                LONG $11110000, $11111111, $11111111, $11111111, $88883111, $88888383, $88888883, $23888883
                LONG $99999999, $999A9A99, $88388299, $88888888, $38888888, $66455288, $BBBBBBB6, $BBBBBBBB
                LONG $06000005, $88310115, $11338888, $119A9932, $11111111, $06650100, $00050000, $6BB67100
                LONG $B6BB6BBB, $00056BBB, $00000000, $55550055, $05555555, $55500000, $00005555, $00000000
                LONG $11110000, $11111111, $11111111, $11111111, $83333311, $83838888, $88383838, $99923838
                LONG $99999999, $AAAAAAA9, $8329A999, $38838383, $88838888, $64465288, $BBBBBB6B, $BBBBBBBB
                LONG $00060500, $83101100, $01133838, $01157A71, $01110111, $00001110, $11000011, $BBBBB511
                LONG $BBBBBBB6, $5546BBB6, $00000000, $55555555, $05555555, $55550000, $00005555, $00000000
                LONG $11110000, $11111111, $11111111, $11111111, $83333332, $38888383, $38888888, $98888888
                LONG $A9999999, $AAAAAAAA, $9999999A, $88388882, $38388383, $4A755188, $BBBB6BB6, $BBBBBBBB
                LONG $50000000, $21000110, $11111222, $10000000, $00001111, $11101111, $11111111, $6BBBB511
                LONG $6BBBB6BB, $54BBB6BB, $50000555, $55555555, $55555555, $55505000, $00005555, $00000000
                LONG $11110000, $11111111, $11111111, $91111111, $33333332, $88388888, $83838388, $88388388
                LONG $AA999999, $AAAAAAAA, $999999AA, $88833999, $88888888, $47576138, $BBBBBBB6, $BBBBBBBB
                LONG $00000000, $00010010, $00000000, $11100001, $11111111, $11111111, $11111111, $BBBBB611
                LONG $BBBBBBBB, $6BB6BBBB, $50000555, $55555555, $55555555, $55550050, $00055555, $00000000
                LONG $10000000, $11111111, $11111111, $99000001, $33333329, $38883832, $88888838, $88838883
                LONG $AA999938, $AAAAAAAA, $99AAAAAA, $83899999, $83838383, $56644728, $BBBB6BB4, $BBBBBBBB
                LONG $00000000, $00110000, $00000000, $11111111, $11111111, $11111111, $11111111, $BBBBB611
                LONG $BBBBBBBB, $BBBBBBBB, $0005554B, $55555555, $55555555, $55555555, $00055555, $00000000
                LONG $00000000, $11110000, $00000001, $99900000, $32323999, $88388833, $83838388, $83888838
                LONG $AA998888, $9AAAAAAA, $99AAAA99, $39999999, $88888888, $56644598, $BBBBBB65, $BBBBBBBB
                LONG $00000000, $11000000, $00000111, $11111100, $11111111, $11111111, $11111111, $BBBBBB51
                LONG $BBBBBBBB, $B6BBBB6B, $055556BB, $55555555, $55555555, $55555555, $00555555, $00000000
                LONG $00000000, $00000000, $00000000, $99991000, $32999999, $83833323, $38888883, $88838388
                LONG $AA988883, $99AAAAAA, $999AAA99, $999999A9, $38383839, $6BB55618, $BBBBB647, $BBBBBBBB
                LONG $00000000, $00000000, $00000000, $11111100, $11111111, $11111111, $11111111, $BBBBB651
                LONG $BBBBBBBB, $BBBBBBBB, $5554BBBB, $55555555, $05555555, $55505050, $50055050, $00000000
                LONG $00000000, $00000000, $00000000, $9999A000, $29999999, $88833333, $88383888, $83888888
                LONG $A9838888, $999AAAAA, $9999AA99, $999999AA, $88883999, $6BB65698, $6666BB45, $66666664
                LONG $00000000, $00000000, $00010000, $11111100, $11111111, $11111111, $11111111, $BBBBBB41
                LONG $BBBBBBBB, $BBBBBBBB, $06BBBBBB, $50555550, $00000000, $00000006, $05000000, $00000050
                LONG $00000000, $00000000, $00000000, $99999910, $99999999, $38833332, $38888838, $38883838
                LONG $38332393, $AAAAAAA2, $999AAAAA, $99999999, $83889999, $6BBB6573, $66B00664, $BBBBBBBB
                LONG $00000000, $00000000, $00010000, $11111000, $11111101, $11111111, $11111111, $BBBBBB61
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $00000000, $06060506, $06060600, $60060606, $00000000
                LONG $00000000, $00000000, $00000000, $99999990, $22223399, $83223332, $88838388, $99938888
                LONG $29999999, $AAAAAA93, $9999AAA9, $99999999, $88399999, $6BB66679, $BBB6BB64, $BBBBB6B6
                LONG $00000000, $00000000, $00011000, $11111000, $11111111, $11111111, $11111111, $BBBBBB61
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $060000BB, $00000060, $00000006, $00600000, $00000506
                LONG $00000000, $00000000, $10000000, $99999999, $33299999, $33333333, $38388883, $99999838
                LONG $999999AA, $99AA9929, $A9AAAAA9, $99999999, $82999999, $6BB44653, $B66BB647, $BBBB6BBB
                LONG $00000000, $00000000, $00011000, $11111000, $11111111, $11111111, $11111111, $BBBBBBB5
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $00066BBB, $00606000, $60060000, $00000600, $00005000
                LONG $00000000, $00000000, $91000000, $99999999, $39999999, $33883333, $88838388, $999AA288
                LONG $9999A779, $99999999, $A999AAA9, $999999AA, $39999999, $6B644B49, $6B666B45, $B6BBBB6B
                LONG $00000000, $00000000, $00000000, $11111000, $11111111, $11111111, $11111111, $BBBBBBB5
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $006BBBBB, $60000006, $06006060, $60060060, $00005060
                LONG $00000000, $00000000, $9A700000, $99999999, $29999999, $88323333, $83888838, $99AAAA93
                LONG $999A7A99, $99999999, $999999A9, $99999999, $99999999, $76646B5A, $BBB6BB67, $BB6B6BBB
                LONG $00000000, $00000000, $00011000, $11110000, $11111111, $11111111, $11111111, $BBBBBBB4
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $06BBBBBB, $00600600, $00000000, $00600000, $00005500
                LONG $00000000, $00000000, $9AAA7000, $99999999, $99999999, $33333333, $28883888, $9AAAAA7A
                LONG $9A74A999, $A9999999, $999999AA, $99999999, $83999999, $7756BB61, $BB6BBBB4, $6BBBBBB6
                LONG $00000000, $00000000, $00000000, $11111100, $11111111, $11111111, $11111111, $BBBBBBB6
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $6BBBBBBB, $60006006, $66600600, $00000600, $00600506
                LONG $00000000, $00000000, $AAAAA700, $99999999, $99999999, $23333333, $93888833, $AA77444A
                LONG $444999AA, $9999999A, $99AAAAA9, $99999999, $88399999, $5776B662, $B6BBB6B6, $BB6B6B6B
                LONG $00000000, $00000000, $00000000, $11111110, $11111111, $11111111, $11111111, $BBBBBB6A
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $00000066, $00000000, $060B6060, $60006000
                LONG $00500000, $55011100, $AAAAAA10, $9999999A, $99999999, $33333329, $A9838233, $466647AA
                LONG $7A9AA7A7, $923339A4, $99999999, $99999999, $88399999, $B666BB5A, $BBB666BB, $BBBBBBBB
                LONG $00000000, $00000000, $00000000, $11111110, $11111111, $11111111, $11111111, $BBBBB67A
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $6BBBBBBB, $6BB6B666, $BB6BB6BB, $50605BBB, $00050555
                LONG $56000000, $66400100, $AAAAAAA0, $9999999A, $99999999, $33333229, $92833333, $647AAAAA
                LONG $9AAA7446, $3333339A, $99999922, $99999999, $88829999, $BBB66661, $B6BB4546, $B6B6B6B6
                LONG $00000000, $00000000, $00000000, $11111100, $11111111, $11111111, $11111111, $BBB679AA
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $46BBBBBB, $6BBB6B66, $BBBB6666, $00060606, $00600600
                LONG $00000000, $01110050, $AAAAAA75, $999999AA, $99999999, $33333999, $A9333333, $AAAAAAAA
                LONG $9A77647A, $83322133, $99999933, $99999999, $88899999, $6BB66B61, $BBBB6555, $BBBBBBBB
                LONG $00000000, $00000000, $00000000, $11111000, $11111111, $11111111, $A1111111, $BB6799AA
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $66BBBBBB, $BB6BB666, $B6BBBBBB, $066BBBBB, $00000006
                LONG $00000000, $01212560, $AAAAAA44, $9999AAAA, $99999999, $33322999, $4A333333, $AAAAA774
                LONG $2A77AAAA, $33211333, $99999333, $99999999, $38329999, $546BBB61, $B6B6B645, $B6B6B6B6
                LONG $00000000, $00000000, $00000000, $11100000, $11111111, $11111111, $A1111111, $B4799AAA
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $666BBBBB, $66BBB664, $BB6BB6BB, $066BBBBB, $06060600
                LONG $00000000, $40545B60, $77744444, $AAAAAA7A, $99999999, $32299999, $47933333, $AAAA7444
                LONG $329AAAAA, $21133321, $AAA23332, $99999AAA, $38899999, $5566B661, $BBBBBBB4, $BBBBBBBB
                LONG $00000000, $00000000, $00000000, $11111000, $11111111, $11111111, $41111111, $7AAAAAA7
                LONG $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $446BBBBB, $66466664, $6BB6BBBB, $06BB6B6B, $00000000
                LONG $00000006, $450BBBB0, $77777774, $99A9AA77, $99999999, $32299999, $47233333, $74774444
                LONG $339AAA77, $71233123, $AA923297, $9999999A, $38329999, $656BBB41, $B6B6B6BB, $B6B6B6B6
                LONG $00000000, $00000000, $00000000, $11111000, $11111111, $11111111, $47111111, $AA77AAA4
                LONG $BBBBBB67, $BBBBBBBB, $BBBBBBBB, $4446BBBB, $A7444444, $BBBBB64A, $66BBBBBB, $0006006B
                LONG $00000000, $4456B060, $AAAAAAA7, $999999AA, $99999999, $33339999, $47933333, $44466646
                LONG $11A74444, $64A11233, $999A7446, $99999999, $88899999, $BBBBBB6A, $BBBBBB6B, $BBBBBBBB
                LONG $00000000, $00000000, $00000000, $01111000, $11111111, $11111112, $45111111, $7A77AAA7
                LONG $BBBBBB64, $BBBBBBBB, $BBBBBBBB, $666B6B6B, $16666666, $B6BB6A29, $BBB6B6B6, $00600BBB
                LONG $00000000, $44400000, $AAAA7444, $9999AAAA, $99999999, $33329999, $47233333, $66644744
                LONG $11766666, $66411111, $99974466, $9999999A, $83299999, $BB6B6B51, $B6B64446, $B6B6B6B6
                LONG $00000000, $00000000, $00000000, $01111000, $99999211, $11111111, $44A11111, $477AAAA7
                LONG $BBBBBB64, $BBBBBBBB, $BBBBBBBB, $66BBBBBB, $06666B66, $BBB64999, $6BBBBBBB, $000066BB
                LONG $00000000, $44440006, $A7744447, $9AAAAAAA, $99999999, $29999999, $47992329, $44744444
                LONG $11144444, $44451111, $79974666, $77447474, $8399AAA7, $BBBBBB62, $BB655444, $BBBBBBBB
                LONG $05000000, $00000050, $00000000, $11111000, $12211210, $11111111, $74711111, $4477AAAA
                LONG $BBBBBBB4, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $1566BB66, $BB6667A7, $BB66B6B6, $0600006B
                LONG $06000000, $44446600, $77444444, $AAAAAAAA, $99999999, $99999999, $47999999, $44444474
                LONG $11A46444, $66471111, $47A76B66, $A7774466, $99AAAAAA, $6BB6BB61, $B6566645, $B6B6B6B6
                LONG $00000000, $00000500, $00000000, $11100000, $11111110, $11111111, $44A33221, $54444AAA
                LONG $BBBB6BB4, $BBBBBBBB, $BBBBBBBB, $66BBBBBB, $A006BB66, $66664777, $B6BBBBBB, $600B6BBB
                LONG $0006066B, $47446006, $44474474, $AAAAAA74, $99999A9A, $99999999, $4A999999, $44447464
                LONG $11766664, $464A1111, $44746446, $AAAAA774, $747AAAAA, $BB6BBB64, $656BBB66, $BBBBBBBB
                LONG $00000000, $00000000, $00000000, $11100000, $11111111, $33333311, $447A3838, $074447AA
                LONG $BBB6BB65, $BBBBBBBB, $BBBBBBBB, $66BBBBBB, $45056BB6, $46644477, $BBB66B64, $BBBBBBB6
                LONG $00006B6B, $44444660, $44444444, $AAAAAA74, $99999AAA, $9AA99999, $6A999999, $66666666
                LONG $11766666, $47711111, $46666444, $7AAAAA74, $44444444, $BBBB6B64, $65BBB666, $B6B6BB6B
                LONG $00000005, $00000000, $00000000, $21111110, $33332222, $AA292323, $7444799A, $174444AA
                LONG $BBB66B60, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $440006BB, $44664464, $6BB66647, $6BBB6BBB
                LONG $06000000, $44744645, $44447444, $AAA77744, $AAAAAAAA, $AAAAAAAA, $67AAAAAA, $66666666
                LONG $17466666, $4A111111, $66644664, $44477744, $74444474, $B6BBBB67, $66BB666B, $BBBB6BB6
                LONG $00000555, $00000000, $11111111, $33232332, $A9999112, $46666647, $A746A9A7, $1174444A
                LONG $B6666B50, $BBBBBBBB, $BBBBBBBB, $B6BBBBBB, $464005BB, $46444464, $BBB64754, $0BBBBB6B
                LONG $60060060, $74446666, $44744474, $74744447, $A77A77A7, $AAAAAAAA, $667AAAAA, $66666666
                LONG $74444466, $A1111119, $64AAA774, $44729A74, $44447444, $BBB6BB64, $56BBB6BB, $6BB66666
                LONG $11105505, $22191111, $22223229, $55411122, $75555566, $74666666, $A4444AAA, $11544447
                LONG $600BBB60, $BBBBBBBB, $BBBBBBBB, $66BBBBBB, $44640046, $46444464, $B6644444, $0066B6BB
                LONG $60600060, $66666666, $44444444, $44444444, $74774444, $77AAA7AA, $64777777, $466666BB
                LONG $44474446, $11111197, $7A1111AA, $4A2332A7, $44444444, $6B6BBB67, $5BB666BB, $BBB66455
                LONG $999AAAAA, $1AAAA999, $477511A1, $55466664, $474556B4, $44666666, $77447997, $11564447
                LONG $00BB6B51, $BBBBBB60, $BBBBBBBB, $4466666B, $44464004, $44644446, $66644447, $BBBBBB66
                LONG $66666666, $66666666, $47447446, $44447444, $44444444, $44774447, $4A97A7A7, $66666666
                LONG $44666666, $1111AA56, $11111117, $6A233111, $4746B44B, $BBBB6B47, $6666BBB6, $BBBBB645
                LONG $45471111, $BBBBB666, $BBBB66B6, $55546666, $464546B6, $74666666, $44444AAA, $115B6544
                LONG $006BBB51, $55505000, $7441A555, $AA9999A7, $4477A711, $44664444, $66664655, $66666666
                LONG $66666666, $66666666, $44444444, $44744474, $44444444, $AA747444, $99999AAA, $66666647
                LONG $66666666, $71117466, $11111115, $B4123111, $4446B66B, $B6BBB664, $BB66B6BB, $66BBBBB6
                LONG $B6664111, $BB66646B, $BB6BBB6B, $66666456, $66666BB6, $A4666666, $77744AAA, $10665444
                LONG $006BBB41, $00055506, $99A77745, $17AAA999, $44466649, $54666447, $66666655, $66666666
                LONG $66666666, $66666666, $44444666, $74444444, $47444474, $AAAAAA77, $AAAAAAAA, $666664AA
                LONG $B6666666, $4774446B, $11111175, $BB111111, $746BBBB6, $BB6BB644, $6BBBBBB6, $66B6BB66
                LONG $B6666511, $BB6774BB, $6B66666B, $66666656, $67466B66, $A4666666, $44447AAA, $55644666
                LONG $6666664A, $46444654, $AAAAAA75, $1A9A9AAA, $44466641, $54666644, $66666645, $6666B666
                LONG $66666666, $66666666, $44444466, $44474744, $77477444, $AAAAAAAA, $AAAAAAAA, $666664AA
                LONG $66666666, $66666666, $111A4466, $44711111, $666BB666, $B6BBBB66, $74BBB6BB, $6B666647
                LONG $B6B66671, $B67994BB, $BB66446B, $B6666666, $6466BB66, $A4666666, $647447AA, $46666666
                LONG $BB666666, $6BB6666B, $A9A7AAA7, $7AA99A99, $44444641, $56666644, $66666664, $6666B666
                LONG $66666666, $66666666, $64755646, $44446466, $74774444, $AA7AAAA7, $AAAAAAAA, $66664AAA
                LONG $66666666, $BB666666, $74666666, $4444A1A7, $BBBBB664, $BBB6B6BB, $994BBBB6, $BBB47A99
                LONG $B6BB6664, $64A9A7BB, $66666BBB, $666666BB, $6666BB66, $77666666, $44447777, $B6444464
                LONG $6644666B, $BB664466, $9A457A95, $5A7A9A7A, $47444671, $66666666, $66666666, $46BBB666
                LONG $66666666, $66666666, $66556BB6, $44444466, $44444444, $47447444, $A77A7744, $66667AAA
                LONG $66666666, $6BB66666, $666666BB, $44B66666, $6BB66666, $B6BBBBB6, $A746BBBB, $BBB6477A
                LONG $B66B6666, $7A97446B, $B4466477, $6B6666BB, $666BB666, $66444466, $6B6B6746, $664A4666
                LONG $64466666, $B6666466, $7A156776, $5777A777, $66744451, $6B6BB666, $66666666, $64666666
                LONG $66666666, $66666666, $4646BB66, $47444474, $44444747, $44444444, $77444444, $66644444
                LONG $66666666, $BBB66666, $66666BBB, $66B66BBB, $6B664666, $BBBBBBBB, $9A4BB6BB, $BB6B64A9
                LONG $BB666666, $229AA746, $644AA938, $BBB66666, $B6BBB66B, $B6BBBBBB, $B6666666, $6B446B6B
                LONG $B666BB66, $BBBBBBBB, $6755466B, $5557A666, $66644455, $6BBBBBBB, $666B6666, $66666666
                LONG $66666646, $66666666, $44446666, $66664664, $44474466, $44444444, $44444444, $66474444
                LONG $66666666, $BBB66666, $B6BBBBBB, $666BBBBB, $66644BBB, $6BBBBBB6, $9A6BBBBB, $BBBB6499
                LONG $B6666444, $933A4446, $BB447382, $666444BB, $BBBBBBBB, $BBBBBBBB, $B6666446, $6B66BB6B
                LONG $B776B644, $BBBBBB6B, $66646446, $55664466, $B6666645, $BBBBBBBB, $BB6B6666, $66666BBB
                LONG $66666666, $66666666, $92333974, $66666667, $44666666, $74446666, $44447474, $44777444
                LONG $BBBBBBB6, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $6666466B, $BBBBBBBB, $996BB6B6, $BB66B6A9
                LONG $B6744477, $7389777B, $BB66A316, $67A7546B, $BBBBBBBB, $6BBBBBBB, $66666446, $B66666BB
                LONG $B47466BB, $666BB64B, $B654B676, $64664466, $BBBB6666, $BBBBBBBB, $BBB66BBB, $BBBBBBBB
                LONG $66BBBBBB, $46666666, $8888833A, $66666649, $66666666, $44664746, $44744444, $46644747
                LONG $BBBBBBB6, $BBBBBBBB, $BBBBBBBB, $BB66BBBB, $66444466, $BBBBBBB6, $774BBBBB, $BB666667
                LONG $B4444444, $77AAA746, $BBB44AAA, $B7A45546, $BBBBBBBB, $6BBBBBBB, $B66BB676, $B644446B
                LONG $BB664BBB, $B666B66B, $4656666B, $66666666, $BBBBBB66, $BBBBBBBB, $BBBBBBBB, $6BBBBBBB
                LONG $6666BB66, $46666666, $8333882A, $66666B73, $66666666, $46BB79A6, $46666666, $66666664
                LONG $BBBBBB6B, $BBBBBBBB, $66BB6BBB, $6BB66666, $66666666, $BBBBBBBB, $BBBBB6BB, $66666BBB
                LONG $B6747444, $64447746, $6B644644, $6B646555, $BBBBBBBB, $BBBBBBBB, $BBBBB676, $B66446BB
                LONG $BBB66BBB, $BBBBB666, $56666BBB, $666BBB40, $BBB6B666, $BBBBBBBB, $BB6B6BBB, $66BB6666
                LONG $BB6BB666, $4666666B, $33333999, $B6BBB678, $66BB66B6, $B6446466, $B6B6B6B6, $BBBBBBBB
                LONG $B6B6B6BB, $45444666, $46455454, $66655554, $BBBBBB64, $BBBBBBBB, $BBBBBBBB, $66B6B6BB
                LONG $B4444474, $BB444446, $BBB6BBBB, $BB646645, $BBB66BBB, $BBBBBBBB, $BB6BB676, $B66666BB
                LONG $BBBBBBBB, $BB66B446, $5554BBBB, $66BBB665, $66664466, $B6666666, $6B666BBB, $66BBB666
                LONG $B66B6666, $4BBBBBBB, $88839A71, $BBBBBB42, $4BBBBBBB, $B1299A74, $6BBBBBBB, $66666666
                LONG $55555444, $55555555, $55555555, $66666466, $BBBBBB66, $BBBBBBBB, $BBBBBBBB, $66BBBBBB
                LONG $B4744444, $BB674746, $6BBB6466, $BB66B6BB, $BBBBB6BB, $BBBBBBBB, $BBBBBB66, $BBBB6BBB
                LONG $BBBBBBBB, $BBBBBBBB, $05556BBB, $6BBB5566, $B66666B6, $6B6B6666, $6B666BB6, $66BBBB66
                LONG $A76BB664, $6BBBBBB4, $38832976, $BBBBBBBA, $4B6BBBBB, $5177229A, $54444444, $55555555
                LONG $44455555, $54554545, $66666664, $66666666, $BBBBBB66, $BBBBBBBB, $BBBBBBBB, $66BB6BB6
                LONG $BB644AA4, $6B6466BB, $46BB6555, $BBB7A744, $B65156BB, $BBBBBBBB, $BBBBBB66, $BBBBBBBB
                LONG $BBBBBBBB, $BBBBBBBB, $455546BB, $BBB60121, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $4B666666
                LONG $9A666B65, $BB66BB6A, $388839A6, $46666667, $76645556, $56B629AA, $55555555, $46646455
                LONG $66664646, $B6666666, $BB666666, $BBBBBBB6, $BB66BBBB, $BBBBBBBB, $66BBBBBB, $B666BB64
                LONG $BBB64199, $56476BBB, $76BB6555, $BB67AA44, $B61156BB, $66BBBBBB, $BBBBBBB6, $BBBBBBBB
                LONG $BB66BBBB, $B66BBB6B, $51154666, $BBB40282, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $447A474B
                LONG $97BBBBB6, $466B66A9, $88882974, $55555555, $44555555, $66B6A274, $66666666, $66666666
                LONG $6BB66666, $B666B666, $BB6BB66B, $BBBBBBBB, $BBBB666B, $BBBBBBBB, $666BBBBB, $666B6BB6
                LONG $6666554A, $5522A466, $6BBBB455, $664A9744, $B6156BBB, $454BBB65, $B6666B64, $BBBBBBBB
                LONG $BBB6BBBB, $BBBBBBBB, $1115564B, $BBB41282, $BBBBBBBB, $BBBBBBBB, $B66BBB6B, $66677566
                LONG $A66BBBB6, $55554722, $33339A44, $4555545A, $46664464, $664A32AA, $66666666, $66666666
                LONG $B6666666, $B6B66BBB, $BB6B6BB6, $6B6B6BBB, $6666BBBB, $6B6666B6, $666666B6, $66B6BB65
                LONG $64445466, $03888394, $BBBBB655, $544A7774, $6656BB65, $4556BB65, $BB664647, $6BBBBBBB
                LONG $BBBBB666, $BBBBBBBB, $32129A56, $B6651233, $666666BB, $46466666, $BBBBB666, $666BB6BB
                LONG $946BB6BB, $55555932, $33297464, $6666666A, $76666666, $667239AA, $66666666, $66666666
                LONG $66666666, $66666666, $666B6666, $66666666, $66666666, $66666666, $6B666666, $6B66B664
                LONG $4544466B, $38888824, $BBBB6A32, $BB66BB66, $66B6BB6B, $A75544AA, $BBBBBBB4, $6BBBB666
                LONG $6666BBB6, $6BBB66BB, $111A1214, $44550131, $55544445, $55555555, $666BBBB4, $45546666
                LONG $A71A7774, $666B7999, $99994466, $66666649, $46666666, $66A32564, $66666666, $66666666
                LONG $66666666, $66666666, $66666666, $66666666, $66666666, $66666666, $66666666, $66BBBBB6
                LONG $5566666B, $38338315, $BB647339, $BBB6BBBB, $4BBBBBBB, $99229797, $7754664A, $666BB655
                LONG $666666BB, $66666BB6, $6B6B45A7, $5555512A, $55550555, $65555555, $666BBBBB, $A5666666
                LONG $6A332999, $66664AA4, $999A4444, $6666664A, $66666666, $64131664, $66666666, $66666666
                LONG $66666666, $66666666, $66666666, $66666666, $66666666, $55466666, $56655544, $BBBBBBB6
                LONG $55666666, $33399105, $B6567399, $BB6BBBBB, $76BBBBBB, $99299AA7, $AAAAAA99, $46666655
                LONG $44464445, $66666666, $A176B656, $45551129, $44444546, $66666664, $66BBBBB6, $1546B666
                LONG $6A329933, $6666655B, $999A4444, $66666669, $66666666, $67145466, $66666666, $66666666
                LONG $66666666, $66666666, $44446646, $44555444, $44455444, $55555555, $56655555, $BBBB6665
                LONG $5464AA46, $A299AA10, $B66B6932, $BBB6BBBB, $116BBB6B, $A9229911, $5575A99A, $66455405
                LONG $55554666, $46666666, $11A66665, $66651111, $66666666, $46666666, $B6BBBB64, $1566666B
                LONG $4229AA23, $6666654B, $779A7444, $6666666A, $46666666, $50465555, $66666666, $46466666
                LONG $55455455, $55554554, $55555555, $55555555, $55555555, $55555555, $74655555, $66666655
                LONG $564999A6, $29999975, $BBBBB711, $BBBBBBBB, $110546BB, $A4647A11, $5555A944, $BBBB6445
                LONG $B666B6B6, $66666666, $A4466666, $66657445, $66666666, $66666666, $666BBB66, $75466666
                LONG $79997733, $66666556, $77999744, $6666664A, $56666666, $11541991, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $56655555, $66555546, $64546466, $55455556, $55555555
                LONG $6B67AA74, $33299915, $64565528, $66BBBB66, $15A11066, $A74A4411, $6455A974, $66BB6666
                LONG $6BBBBBBB, $B6666666, $BBBBBB66, $66574BBB, $66666666, $66666666, $666666B6, $556666B6
                LONG $A9999932, $46664566, $A4799A44, $54546679, $05555454, $15555A9A, $55555551, $55555555
                LONG $54555555, $55555555, $55455455, $54555555, $55545555, $55555555, $55555555, $55555555
                LONG $76B64774, $33299975, $11105528, $55550111, $51511555, $47AA4455, $BBB6AA66, $BBB66BBB
                LONG $666BBBBB, $B666B6B6, $BBB6BBB6, $64756BBB, $66666666, $66666666, $BB666B66, $7566B666
                LONG $5A9A9329, $44645556, $74479974, $555555A9, $55555555, $55555555, $55554555, $55555555
                LONG $45666545, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $7BB647A7, $32999A55, $111555A3, $55555751, $11111550, $44664511, $BBB676BB, $BBBBBBBB
                LONG $BBBBBBBB, $66BBBBBB, $BBBBBB66, $65A66BBB, $66666666, $66666666, $66666666, $55564666
                LONG $55555755, $55555555, $A44599A4, $55555513, $55554465, $55555555, $54565655, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555554, $55555555, $55554555, $55545555
                LONG $BBB44777, $999AA054, $55555519, $11111055, $11111111, $66444511, $BBB66BBB, $BBBBBBBB
                LONG $B6BBBBBB, $BBBBBBBB, $BBBBBBB6, $65566BBB, $66666666, $66666666, $64554666, $55555545
                LONG $55555555, $45555555, $9467A9A4, $55666493, $55555666, $55555555, $55557555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $54555545
                LONG $BB644774, $66BB6556, $55554665, $51111111, $71155515, $BB644455, $BBB4BBB6, $BBBBBBBB
                LONG $66666BBB, $BBBBB666, $BBBBBBB6, $546666BB, $55644446, $55555444, $45555555, $54455666
                LONG $45554455, $44666466, $94655AA4, $55544592, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555755, $55555555, $55555555, $55554555, $66454455, $54445554
                LONG $BB6444A9, $BBBBB6BB, $11055546, $55051551, $75455555, $BB64455A, $BB6BBB66, $666BBBBB
                LONG $66666BBB, $BB6B66B6, $BBBBBBB6, $555546BB, $55555555, $45555555, $66456554, $45664566
                LONG $55555555, $45555555, $94555AA4, $55555593, $55555555, $55555555, $55555555, $55575555
                LONG $55555555, $45555555, $55555555, $55555555, $65554555, $46666666, $44445545, $55544554
                LONG $BBB647A9, $BBBBBBBB, $5555566B, $55555555, $10505555, $6B644711, $66666666, $666BB666
                LONG $666666BB, $6666666B, $B6BBBBB6, $645546BB, $54566654, $55546664, $55546454, $55555555
                LONG $55555555, $45555555, $94555AA4, $55555592, $55555555, $55555555, $5555A575, $55555555
                LONG $54555555, $45555555, $45555555, $45546456, $66556556, $55466666, $45555554, $45555554
                LONG $BBB4447A, $BBBBBBBB, $555566BB, $50555555, $11110511, $6B667110, $56466666, $54546645
                LONG $B4454454, $666666BB, $64BBBB64, $554566BB, $55554555, $55555555, $55555555, $55555555
                LONG $55555555, $45555555, $974559A7, $55555513, $55555555, $55555555, $55555455, $55555555
                LONG $55545555, $55555545, $64465555, $46455666, $54545555, $46666645, $45554546, $55555455
                LONG $BBBB6666, $BBBBBBBB, $55556B6B, $10511155, $11111111, $5BB4A111, $45555555, $66555555
                LONG $46466556, $56664546, $456BBBB4, $55555BBB, $55555555, $55555555, $75555555, $55555555
                LONG $55555555, $45555555, $9A455AA4, $55555559, $55455555, $45555555, $55555545, $55555555
                LONG $54555555, $46654664, $66666645, $66664666, $66466555, $66664444, $66664566, $46555644
                LONG $BBBBBBBB, $BBBBBBBB, $5555466B, $15511150, $11105111, $66644555, $66654464, $45555556
                LONG $55555555, $55555555, $655BBB65, $55556BBB, $55555555, $55555555, $55555555, $55555555
                LONG $56465555, $46555465, $75545574, $55546657, $55555444, $55555555, $55555555, $54555565
                LONG $66665545, $66666666, $66666666, $64566666, $66656446, $66666466, $66666666, $64466666
                LONG $BBBBBBBB, $BBBBBBBB, $555546BB, $55111151, $55555511, $64446555, $55555556, $55555555
                LONG $55555555, $55555555, $4556BBB4, $55556BB6, $55555555, $55555555, $55555555, $55555555
                LONG $66666555, $55556666, $54666555, $55544555, $55554466, $64555444, $56455555, $66666664
                LONG $66666666, $66666666, $66666666, $66556456, $66556666, $66666666, $66666666, $66666666
                LONG $BBBBBBBB, $BBBBBBBB, $555566BB, $55555115, $55555555, $55464111, $55555555, $55555555
                LONG $55555555, $55555555, $5546BBB5, $55554BB6, $55555555, $55555555, $65566455, $54555546
                LONG $65466455, $44446454, $66646666, $66645666, $66665566, $66665666, $46666666, $66666666
                LONG $66666666, $66666666, $66666466, $66645645, $65466464, $66666666, $66666666, $64666646
                LONG $BBBBBBBB, $BBBBBBBB, $55556BBB, $55555555, $11115555, $5564A111, $55555555, $55555555
                LONG $55555555, $45555555, $456BBBB6, $45456BBB, $55466666, $66645465, $44666666, $66666666
                LONG $66666666, $66666666, $66665666, $66666666, $66666556, $55466666, $66666666, $66646666
                LONG $66666666, $66666666, $45666666, $56645555, $66664565, $66666666, $65466666, $65666664
                LONG $BBBBBBBB, $BBBBBBBB, $55566BBB, $55551555, $55151115, $54667111, $55555555, $55555555
                LONG $55555555, $66666554, $6666BBB6, $66666BBB, $66666666, $66466666, $66666666, $66646666
                LONG $46666666, $66666666, $66656666, $46666666, $46645464, $45566454, $66466666, $66666666
                LONG $66666666, $66664664, $66646646, $55555566, $66666665, $46456666, $66656645, $66445666
                LONG $BBBBBB6B, $BBBBBBBB, $550466BB, $50111115, $51151111, $54664111, $64655555, $64655454
                LONG $66646464, $66666666, $6666BBB6, $6666BBB6, $66666666, $64555466, $46666446, $64666646
                LONG $45666666, $55666546, $65545455, $64664546, $66555666, $45555555, $55655665, $46666645
                LONG $46646465, $45666664, $46645546, $55555555, $45456666, $66666666, $55464566, $46646666


Sprite_Size     BYTE $20
Sprite_Data     WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020



                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000

 }



{
 ' Boat 
Palette_Data    LONG $2D4C2C3B, $DA5B8D7D, $8E4D2A4A, $02020207

Image_Data      LONG $10000000, $00010100, $01010121, $01101001, $12111011, $11111112, $11111111, $11313111
                LONG $31313111, $31313131, $31313131, $31313131, $31313131, $31313131, $31113131, $11311311
                LONG $33313313, $45555541, $55555555, $55555555, $55555555, $55555555, $55555555, $55455555
                LONG $87656455, $A3BAA988, $5555BAAA, $55555555, $55555555, $0BBB5555, $13131133, $31131313
                LONG $00001002, $10000000, $01001000, $10100100, $10000100, $11011101, $10110110, $11111111
                LONG $11111111, $11111111, $11111111, $11111111, $11111111, $11111111, $11311111, $13113113
                LONG $31131131, $56454441, $55555545, $55555555, $55555555, $55555555, $55555555, $56555555
                LONG $87564455, $AA3AA888, $55555BA3, $55555555, $55555555, $ACBB5B5B, $11111131, $31111111
                LONG $00000000, $00000000, $10000000, $01011101, $11111111, $01100110, $11011101, $11111110
                LONG $11111111, $11111111, $11111111, $11111111, $11111111, $11111111, $13331111, $11311311
                LONG $13333113, $55555542, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $86475455, $A3AAA888, $55555A3A, $55555555, $55555555, $BB555555, $11113312, $31111111
                LONG $00100010, $01001001, $00110010, $01000100, $11010101, $11111101, $11111111, $10110111
                LONG $11111101, $11111111, $11111111, $11111111, $11111111, $31111111, $13113313, $31131131
                LONG $11311311, $55575542, $55555555, $55555555, $55555555, $55555555, $55555555, $65555555
                LONG $87575755, $AAA3388D, $5555BAA3, $55555555, $55555555, $BBB5B555, $1111311A, $31111131
                LONG $00000002, $00000000, $01001001, $01111011, $10110110, $11011011, $01111110, $11111111
                LONG $11111111, $11111111, $11111111, $11111111, $11111111, $33113113, $11131331, $13313313
                LONG $31333331, $54556552, $55545575, $55555555, $55555555, $55555555, $55555555, $45555555
                LONG $84574555, $333A2888, $55554AAA, $55555555, $55555555, $BB555555, $3113312C, $31113311
                LONG $00000000, $01001001, $10100010, $11000100, $11101101, $11110110, $11111111, $11111111
                LONG $11111111, $11111111, $31111111, $11111311, $31111111, $11311311, $13133113, $31131331
                LONG $13331313, $55545544, $55554555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $65756456, $3A3A9D8D, $5555BA3A, $55555555, $55555555, $B55B5555, $131331BB, $31131313
                LONG $00100000, $00100001, $10010100, $00111010, $10111010, $11011101, $11111111, $11111111
                LONG $11111111, $11111111, $33111111, $11111333, $11131111, $31131111, $31311131, $31313113
                LONG $13131133, $65556554, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $74475556, $333A9888, $555543A3, $55555555, $55555555, $5B555555, $11331ABB, $31111111
                LONG $00001001, $10010100, $01001001, $11001011, $11101111, $11111111, $11111110, $11111111
                LONG $11111111, $11111111, $11131111, $11113333, $33311311, $13113113, $31331333, $13313331
                LONG $21333131, $55755654, $55565555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $74547545, $33339888, $5555A333, $55555555, $55555555, $555B5555, $33332BBB, $33111331
                LONG $00100000, $01000001, $10101010, $01110100, $11011011, $11111110, $11111111, $11111111
                LONG $11111111, $13111111, $33333331, $11333333, $13313113, $31331331, $33113113, $31331133
                LONG $13333333, $75545554, $55555455, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $47765755, $33A388D8, $5555A3A3, $55555555, $55555555, $5B555555, $1311ABB5, $31331133
                LONG $00000001, $10010010, $10001100, $11011101, $10110110, $10111011, $11111111, $11111111
                LONG $11111111, $33331111, $33333333, $13333333, $31331313, $13133113, $31333131, $33333313
                LONG $23333333, $55555554, $55555545, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $65556445, $33328888, $5555A333, $55555555, $55555555, $55555555, $3113BB55, $33131331
                LONG $01010001, $00100100, $01100001, $10110010, $11101111, $11101111, $11111111, $11111111
                LONG $11111111, $33333111, $33333333, $33333333, $33333111, $31331333, $13333331, $33313333
                LONG $41331333, $55655555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $44775755, $33A08D88, $5555AA33, $55555555, $55555555, $55B55555, $312ABB5B, $31331331
                LONG $00000100, $01001001, $00010110, $00111101, $11011011, $11111101, $11111111, $11111111
                LONG $11111111, $33333111, $3A333333, $33333A3A, $33333133, $13333333, $13333313, $33331333
                LONG $41333333, $55575565, $55455455, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $75575555, $33398887, $55553333, $55555555, $55555555, $B5555555, $31ABBB55, $33133133
                LONG $00100001, $10010010, $11101001, $11101011, $10110100, $11111011, $11111110, $11111111
                LONG $11111111, $33331111, $A33A3A33, $33333333, $33333313, $33333333, $33333333, $33331331
                LONG $41333333, $55554555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $45757655, $3336D887, $55B4A333, $55555555, $55555555, $55555555, $32BB5555, $33333313
                LONG $10011010, $00110100, $00101010, $11010110, $11111111, $11101111, $11111111, $11111111
                LONG $11111111, $33333311, $33A3A333, $33333A3A, $33313333, $33333333, $33333333, $13333333
                LONG $41333333, $55555445, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $56456555, $333988D6, $55543333, $55555555, $55555555, $55555555, $2ABBB55B, $31133333
                LONG $00100100, $01001001, $01011011, $10111111, $01101101, $11111111, $11111111, $11111111
                LONG $33331111, $33333331, $A3333333, $333A33A3, $33333333, $33333333, $33333333, $33333333
                LONG $42333333, $75565555, $55555455, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $47574655, $33188886, $555AA333, $55555555, $55555555, $5B555555, $2BBB5555, $33133333
                LONG $10000001, $00110110, $10101101, $11011001, $11110110, $11011011, $11111111, $33331111
                LONG $33333133, $33333333, $3A3A3A33, $33A3333A, $33333333, $33333333, $33333333, $33333333
                LONG $4A133333, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $45755655, $3328D875, $555A3333, $55555555, $55555555, $55555555, $ABB5555B, $33133333
                LONG $10010000, $10000010, $11011010, $01110110, $11111111, $11111111, $11111111, $33333311
                LONG $33333333, $33333333, $33333333, $333A3A33, $33333333, $33333333, $33333333, $33333333
                LONG $BAAA3133, $5555555B, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $76576555, $33988D74, $55B23333, $55555555, $55555555, $55555555, $BB55B555, $3133333A
                LONG $00101003, $10101101, $01110101, $11011111, $11101111, $11111110, $11111111, $33333311
                LONG $33333333, $33333333, $3333A333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $4BBC3133, $5B5B5555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55754455, $33988886, $555A3333, $55555555, $55555555, $55555555, $B55B5555, $3333332B
                LONG $01000012, $10011010, $11101010, $11111011, $01111111, $11111111, $11111111, $33333311
                LONG $33333333, $33333333, $3A333333, $33333333, $33333333, $33A3A333, $33333333, $33333333
                LONG $4A313333, $45444444, $55555555, $55555B55, $555B5555, $55555555, $55555555, $55555555
                LONG $47547555, $33988875, $55B63331, $55555555, $55555555, $55555555, $BB555555, $333332AB
                LONG $11010101, $11010101, $10011010, $01101110, $10111011, $11111111, $11111111, $33333331
                LONG $33333333, $3A333333, $3333A33A, $33333333, $33333333, $3AAA3333, $33333333, $33333333
                LONG $44213333, $55555545, $54545445, $55555545, $B555555B, $5BBBBBB5, $55555B5B, $55555555
                LONG $75756455, $3388D865, $5BBBAA33, $5555B5B5, $5B55B55B, $5555B555, $55555555, $333333BB
                LONG $10011002, $10110011, $11101110, $11111010, $11111111, $11111111, $11111111, $33333311
                LONG $A3333333, $A3AAA3A3, $333A3AA3, $33333333, $3A3A3A33, $3A3AAA3A, $33333333, $33333333
                LONG $B4333333, $44555B54, $B5BBB5B5, $5BBB555B, $55555BBB, $B5555555, $BBBBBB5B, $5555B5BB
                LONG $45756455, $A3888867, $BBA33333, $BBBBBBBB, $BBBBBBBB, $BBBBBCBC, $BBB55BBB, $33330BB5
                LONG $00100011, $01011011, $11111011, $11011101, $11111111, $11111111, $13113111, $33333111
                LONG $3A333300, $AA33AA3A, $A3AAAA3A, $33A3A3A3, $AAAA333A, $A3AA3AAA, $33333333, $33333333
                LONG $54233333, $D2B55545, $66666688, $6665BBB5, $BBBBB466, $BBBBBBBB, $BBBB5B55, $BBBBB5B5
                LONG $56554555, $32888875, $BBA23333, $BBBBBBBB, $BCBCBBBB, $CBCBCBBB, $BBBBCBCB, $33314BBB
                LONG $11111001, $11101110, $10010010, $01101111, $11111111, $11111111, $11311131, $13331313
                LONG $33333308, $3AAA33AA, $3AA3A3AA, $AA3A3333, $3A33A3A3, $33A3AA3A, $3333333A, $33333333
                LONG $44A33333, $86BB4555, $8D8D8D88, $8D886BBB, $BBB56888, $B566865B, $BB5BBBBB, $5BBBBBBB
                LONG $6555545B, $308D8846, $BBA33333, $BBBBBBBB, $BBBBBBBB, $BBBCBCBC, $BCBCBBBC, $3333BCBB
                LONG $01001113, $11110111, $11101101, $11110111, $11111111, $11111111, $33113113, $03333131
                LONG $A3A33309, $B3AAAA3A, $AAAAAB3A, $33A3A3A3, $33333A3A, $33AAAAAA, $3333A333, $33333333
                LONG $54A23333, $D85B5545, $588888D8, $488884BB, $BBB6D86B, $48886884, $686BBBBB, $BBBB5BB5
                LONG $5555445B, $3988D654, $BBA33333, $BBB5B5B5, $BCBBBBBB, $CCBBBBCB, $BBCBCBCB, $333BCBBC
                LONG $11100103, $01111011, $11110111, $11111110, $11111111, $31111111, $31333333, $23333331
                LONG $3A333309, $3AA3AAA3, $AA3AA3A3, $AA3A3A3A, $3A3A3A3A, $A33A3A3A, $33333A33, $33333333
                LONG $55A23333, $884B5454, $B5446445, $B68D885B, $4B488DBB, $8886B6D8, $86884BB5, $45BBBB48
                LONG $565555B5, $3888D655, $BBA33333, $BBBBBBBB, $BBCBBBBB, $BBBCBCBB, $BCBBBCBB, $3A3BBBCB
                LONG $10101001, $11101100, $11011111, $11111111, $11111111, $31111111, $11131333, $03333333
                LONG $A3333318, $AACAAA3A, $3A3CA3CA, $A3A3A3AA, $A3AAAAAA, $AAAAAA33, $3A3AA33A, $33333333
                LONG $454A3333, $6D455455, $55BBBBBB, $B6D8885B, $6C4888BB, $88DAB6D8, $6B6D8BC4, $884BBB6D
                LONG $45545456, $38888645, $BB233333, $BB5BBBBB, $CBBBBBBB, $CBCBCBBC, $CBCBCBCB, $33BBCCBB
                LONG $11111003, $11010111, $11111111, $11111111, $11111111, $31311111, $33113333, $03333333
                LONG $A3333339, $3A33A3A3, $AAB3AB3A, $AAAA3AA3, $AAA33CA3, $3A3A3AAA, $A3A33AAA, $33333333
                LONG $544A3333, $686B4554, $5BB44662, $8888D8BB, $4BB8D864, $8886B488, $4B6884B4, $48D6B488
                LONG $555545BB, $188D8455, $BB233333, $BBBB5B5B, $BBBBBBBB, $BCBBBBBB, $BCBBBBBC, $3ACCBBCB
                LONG $00101103, $11111011, $11111011, $11111101, $11111111, $31111111, $31333333, $93333333
                LONG $33333319, $AAAA3A33, $A33A33AA, $3A3AAAAA, $3A3AAAAA, $AAAAAA3A, $333AAA3A, $33333333
                LONG $4444A333, $D885B544, $5B68D8D8, $888884B5, $BBBB6888, $8886BBBB, $4B6886C5, $B686C688
                LONG $5555445B, $08888455, $BB233333, $BBBBBBBB, $BBBBBBBB, $BBCBCBBB, $BBCCBCBB, $3BBBCBBC
                LONG $11100011, $11111101, $11111111, $11111110, $11111111, $11113311, $33313111, $03333333
                LONG $33333339, $3A3A33A3, $AAAAAAAA, $AAAAA3A3, $AAACAA3A, $3A3A3AAA, $33A33AAA, $33333333
                LONG $5544A333, $48845455, $B6D88864, $88D89BB5, $BBBB48D8, $4888944B, $BB6D86BB, $B568B488
                LONG $545555BB, $08888555, $BA333333, $BB5BBB5B, $BBBBBBBB, $BCBBBBCB, $BCBBBCBC, $ABCBBCBC
                LONG $01011102, $10111111, $11101101, $11110111, $11111111, $11313311, $11333313, $93333333
                LONG $3A333330, $A3333333, $A3A3A3A3, $3A3A3AAA, $A3A33AAA, $AAAAAAA3, $A333A3A3, $33333333
                LONG $44442333, $B5555455, $4888D95B, $D888885B, $BBB48888, $B688DD85, $4C6884BB, $B4484488
                LONG $455544BB, $9D8D6545, $B433333A, $5BBB5BB5, $BBBBBBBB, $CBBCBBBB, $BBCBCBCB, $CCBBCBCB
                LONG $10110011, $01101001, $10111111, $11111111, $11111111, $31133111, $33313131, $91333333
                LONG $A3333339, $33333333, $3A333333, $AA3A3A33, $AAAAAA3A, $33A33A3A, $3333333A, $33333333
                LONG $5544A333, $5B4B5544, $488886B5, $86488D9B, $BB568888, $6888644B, $6B6D65CB, $88868B68
                LONG $55454455, $98886555, $BAA333A3, $BBBBBBBB, $BBBBB5BB, $BCBBCBBB, $BCBBCBBB, $BBCBBBBC
                LONG $11001003, $11110110, $11111101, $11111111, $11111111, $31313311, $33333331, $91333333
                LONG $AA3A3330, $33333333, $33333333, $33333333, $33333333, $A33A3333, $3333A3A3, $33333333
                LONG $4454A333, $56884B45, $48D8D65B, $65C4D884, $BBB888D8, $88D6BCBB, $88884CB4, $D4688466
                LONG $555545B6, $88884545, $BB3A33A3, $BB5BB5B5, $BBBBBBB5, $BBCBBBBB, $CBCBBCBC, $CBBCCBCB
                LONG $10101113, $11011010, $11101111, $11111111, $11111111, $31331311, $33333333, $91333333
                LONG $AA333310, $A33333AA, $33333A33, $33333333, $33333333, $33333333, $33A3A333, $3333A333
                LONG $54444A33, $488D6454, $588888BB, $BBB488DA, $4BB8D886, $8886BBB4, $644BBBB6, $8B688568
                LONG $454555B8, $88D84555, $BA333A32, $BBBBBB5B, $BBBB5BBB, $CBBBBBBB, $BBBBCBCB, $BBCBBCBB
                LONG $11110113, $01110101, $10111111, $11111111, $11111111, $33333111, $33333333, $92333333
                LONG $33333310, $3A3AA333, $3333333A, $33333333, $33333333, $33333333, $3333A33A, $333333A3
                LONG $44544A3A, $B8D86B54, $B6D8886B, $CBB68884, $D4B688D6, $88D4BB68, $8BCBBBB4, $6C6D6B98
                LONG $55554458, $88864555, $5A3A3AA2, $5BB5BB5B, $BBBBBBBB, $BBCBBBBB, $CBCCBBBB, $BCBBCBBC
                LONG $11001133, $11011110, $11111110, $11111111, $11111111, $33311311, $33333333, $90333333
                LONG $33A33331, $A3A33101, $33333AAA, $33333333, $33333333, $A3333333, $A33A33A3, $333A3333
                LONG $45444A33, $68894B45, $BB6D8D86, $65488D6B, $D6B58888, $8886BC48, $84BBB5B4, $6C686B68
                LONG $55544548, $D8D64554, $BA3A3A32, $BB5BBB5B, $BBBB5B5B, $CBBBBBBB, $BCBBCBCB, $CBCBBCBB
                LONG $11111333, $11111111, $11111111, $11111111, $11111111, $33333133, $33333333, $90333333
                LONG $03333311, $AAA33101, $3A3AA3A3, $3333A33A, $33333333, $3A333333, $A3A3A333, $33333333
                LONG $44544A33, $886B5445, $55B48888, $8DD884B5, $84BB46D8, $88884B6D, $864BBBBB, $6B484C56
                LONG $54555454, $88865455, $5AA33AA9, $BBBBBB5B, $BBBBBBBB, $BBBBBBBB, $CBCBBCBB, $BBBCBBCB
                LONG $10101133, $11111101, $11111111, $11111111, $13111111, $33313311, $33333333, $99333333
                LONG $10013111, $AA331101, $AAA3AAAA, $33A33AAA, $33333333, $33A33333, $3A333A3A, $3333A3A3
                LONG $54454AA3, $45444554, $B555B545, $6444BB55, $6BBBBB44, $568D8D89, $888684BC, $8B46BBB5
                LONG $55454456, $D8865554, $BA3A3A36, $5BB5BB5B, $BB5BBBBB, $BBBBBBBB, $BBBBCBCB, $CBCBBCBC
                LONG $01101333, $10111110, $11111111, $11111111, $33131133, $33331333, $33333333, $09333333
                LONG $01001111, $33331011, $A3AAAA3A, $3333A3A3, $33333333, $3A333333, $3A3AA3A3, $33A33A3A
                LONG $545444A3, $44464444, $4B545544, $5555B55B, $BB555BBB, $BB44444B, $566884BB, $D884BBBB
                LONG $545544B6, $88D45555, $A3AA3AA8, $BBBBBB5B, $BBBB5B5B, $BBBBBBBB, $CBCCBBBB, $BCBBCBBB
                LONG $33331213, $11111113, $11111111, $31111111, $11311313, $33333313, $33333333, $99333333
                LONG $10000900, $A3310111, $AAA3A3AA, $3AA3AAAA, $33333333, $33333333, $A33AAA3A, $A333A3A3
                LONG $444444A3, $24444454, $554B4444, $444444B5, $BBBBB544, $BBB5BBBB, $BBBBBBBB, $664BBBBB
                LONG $45554454, $88845545, $4A3AACA8, $B5BBB55B, $5BBBBBBB, $BBBBBBBB, $BCBBCBBB, $CBCBCBCB
                LONG $33333333, $11101333, $11111111, $31111111, $31113133, $33333331, $33333333, $99333333
                LONG $11000009, $A3301111, $A3AAAAA3, $3AAAA3A3, $3333333A, $A3A33333, $3A333A3A, $33A333A3
                LONG $45454443, $45555445, $B5B44444, $55444445, $5B5B4454, $B4545544, $BB55BBBB, $BCCBBBBB
                LONG $55445555, $8D645555, $AAAAAB38, $BBB5B5BB, $BB5BBBB5, $BBBBBB5B, $CBCBBCBB, $BBBCBBCB
                LONG $3A3A3333, $1111333A, $11111111, $11111111, $31111311, $33333311, $33333333, $99333333
                LONG $01000099, $A3111110, $AAA3A3AA, $A3A3AAAA, $33333A33, $33333333, $AAAAA3A3, $A333A33A
                LONG $44544443, $44445544, $B5544444, $BB444444, $55B55444, $5B5BB555, $B554BBB5, $BBBBBBBB
                LONG $55554455, $88645454, $A33C3B2D, $55BBBB5B, $BBBBB5BB, $BBBBBBBB, $BBBBCBBB, $CCBBCBBC
                LONG $A3A33333, $11333AAA, $11111111, $31111111, $13311311, $33333333, $33333333, $98233333
                LONG $10009009, $31001101, $A3AAAAAA, $3AAAA3A3, $3333333A, $A3333333, $3A333A33, $333A3A3A
                LONG $454444A3, $55554445, $5B444444, $5B544544, $45B44545, $B45BB445, $BB5BBBBB, $BB5BBB5B
                LONG $5444545B, $8D655555, $3AA3BA68, $BB5BB55B, $5BB5BBBB, $BBBBB5BB, $BCBCBBBB, $BBBCBBCB
                LONG $3A333323, $233AAA3A, $11111111, $11111111, $33133133, $33333333, $33333333, $99033333
                LONG $10000099, $30900011, $AA3AA3A3, $A33AAAAA, $333333A3, $33333333, $333A3333, $A3333AA3
                LONG $4445444A, $46444454, $B4444464, $BB445444, $4BB54544, $55B5B54B, $BB5BBBBB, $BBBBBBBB
                LONG $4555545B, $88654545, $A3AB3B9D, $5BBB5B5B, $BBBBB5BB, $BB5BBBBB, $BCBBBBBB, $CBCBCBBC
                LONG $3AAAA333, $33333AAA, $11111123, $11111111, $33313311, $33333333, $33333333, $99933333
                LONG $01001009, $18898990, $3AA3A333, $3A333A3A, $33333333, $33333333, $A3A3A333, $A3A3333A
                LONG $4544544A, $44646445, $55464646, $5B544444, $45B54444, $BBBBB454, $5BBBB5BB, $BBBBBBBB
                LONG $5555445B, $88455555, $AA33A388, $B5B55B54, $BB5BBB5B, $BBBB5BB5, $CBCBBBBB, $BCBBBCBB
                LONG $AA3A3A33, $3A3AAA3A, $11111133, $31111111, $33331331, $33313333, $33333333, $98033333
                LONG $99009009, $08989899, $A3333AA3, $33A3A3A3, $33333333, $33333333, $33333333, $A33A3333
                LONG $44544444, $54464644, $B5644444, $4B444442, $BB554545, $BB5B4445, $54BBBB5B, $BBBBBBBB
                LONG $4454545B, $88445455, $3AAABA88, $BB5B55BA, $5BBB55B5, $5BBBBBBB, $BBBBBBBB, $CBBCBCBC
                LONG $A3A333A3, $333AA3AA, $11111133, $31111111, $33333333, $33331331, $33333333, $09933333
                LONG $89899099, $89889898, $33A33333, $333A33A3, $33333333, $33333333, $33333333, $A3333333
                LONG $45444444, $54464664, $B4464464, $44444444, $4BBB5444, $BB445545, $4BBB5BBB, $BBB5B5B5
                LONG $555544BB, $D6455554, $A3B3A288, $B5BB5BB5, $BBB5B5BB, $BBB5BB5B, $BCBBBBBB, $BCBCBBCB
                LONG $AA33333A, $3A3333A3, $33331333, $33311113, $33333333, $33333133, $33333333, $99033333
                LONG $88889899, $88898888, $A3A3A331, $3333A3AA, $33333333, $33333333, $33333333, $A3333333
                LONG $44454544, $54464644, $55444644, $54544444, $44BB5544, $5BB5B444, $5BB54BBB, $B55BBBB4
                LONG $5544545B, $86455545, $AA3AC0D8, $5B55B5BA, $BB5BBB55, $BB5BBBBB, $BBBBBBBB, $BBCBBCBC
                LONG $33AAA33A, $33AA3A3A, $33333AAA, $33331333, $33333333, $33333333, $33333333, $99933333
                LONG $88888898, $98988999, $A3AA3A39, $33A3AAA3, $33333333, $33333333, $33333333, $A3333333
                LONG $54454444, $B4466464, $55446464, $BB444444, $445BB5B5, $BBBBB5B5, $BB5555BB, $B55BBB55
                LONG $5455545B, $86554545, $AA3AA988, $5BBB55BA, $5BBB5BBB, $5BBBB5BB, $BCBBBBBB, $CBBCBBBB
                LONG $A3333AAA, $AA3AA333, $33A3AA3A, $33333333, $33333333, $33313333, $33333333, $88913333
                LONG $A2088888, $99899A31, $AA3AA308, $333A3A3A, $33333333, $33333333, $33333333, $AAA33333
                LONG $44544444, $55444664, $5B444464, $5B544445, $545B555B, $B5BBB544, $BB55B55B, $BB5BBB45
                LONG $4554545B, $84455455, $3AAA398D, $55BB55BA, $BB55B55B, $BBB5BBBB, $BBBBBBBB, $CBCBCBCC
                LONG $333AA33A, $3AA3A33A, $A33A33AA, $33333333, $33333333, $33313333, $33333333, $88813333
                LONG $AC398888, $88893AA3, $A3AA3369, $3333A3AA, $33333333, $33333333, $33333333, $BA33A333
                LONG $45444544, $44466445, $54546646, $B5444444, $444B5B55, $545BB554, $BBB455BB, $B5BBB54B
                LONG $5545445B, $84554545, $33A3A888, $B5B55BBA, $B5BBBBB5, $BBBBBB5B, $BBBBBBB5, $BCBBBCBB
                LONG $3AA33A3A, $AA3A33A3, $3A3AAAA3, $3333333A, $33333333, $33333333, $33333333, $88903333
                LONG $CB188888, $78863CAA, $A33C3397, $33AA3A33, $33333333, $33333333, $33333333, $4AAA3333
                LONG $44544444, $46666444, $55554464, $44444455, $54B5B5BB, $555BB544, $BBB5BBBB, $BBBBB55B
                LONG $5454544B, $84455454, $3A3B388D, $5B555B4A, $55B5555B, $B5BBBBBB, $BBBBBB5B, $BBCBCBBB
                LONG $3AA33333, $33AAA3AA, $3AA33A3A, $33333333, $33333333, $33333333, $33333333, $88803333
                LONG $A3A3988D, $777787AA, $3A333377, $333333A3, $3A333333, $33333333, $33333333, $BABAA3A3
                LONG $5445455B, $44444454, $54455444, $44445554, $45BB5444, $5445B544, $5BB555BB, $BB5B55BB
                LONG $55555445, $85545555, $AA3A2D88, $B5555BB3, $BBB5BB55, $BBB5BBB5, $BBBBBBBB, $CBBCBCBB
                LONG $A33AA33A, $A3333A33, $A333A333, $33333333, $33333333, $33113333, $33333333, $88803333
                LONG $88208888, $77777777, $33333777, $33333333, $3AAAA3A3, $33333333, $33333333, $AAACAA33
                LONG $B5555B45, $5555BB5B, $55554B55, $45554B55, $5B55B544, $5B554455, $5BBB5BBB, $BBBB5BB5
                LONG $5444454B, $64455545, $AA3B9888, $5BB55BAA, $B5BB5B55, $5BBBB55B, $BBBBB5BB, $CBCBBBBB
                LONG $A3A33A33, $33A3A333, $333A333A, $33333333, $33333333, $33331333, $33333333, $88893333
                LONG $77778888, $77777777, $3333A777, $33A33A33, $AAA33A3A, $33333333, $A3333333, $4AA33AAA
                LONG $44642642, $44444444, $BBBBB4B4, $BB5BB5BB, $B55BBBBB, $BBBB5BBB, $BB55BB5B, $BBB55BB5
                LONG $45554465, $65554555, $A3A39D8D, $B5555BA3, $5BB5BB5B, $BB5BBBBB, $BBBBBBBB, $BCBBCBCB
                LONG $33333333, $33333333, $333333A3, $33333333, $33333333, $33333333, $33333333, $88893333
                LONG $77777888, $77777777, $3333A777, $A3A3A33A, $A3AAA3AA, $333333A3, $33333333, $44AAAA3A
                LONG $42444444, $44444446, $46464464, $44444444, $BBBBB5B5, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $5444454B, $64545554, $3AAA0888, $BBBBBBAA, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB, $BBCBCBBB
                LONG $33333333, $33333333, $33333333, $33333333, $13333333, $33333333, $33333333, $87882333
                LONG $77777777, $77777777, $A3A26777, $AA3A3AA3, $AA3AAAA3, $333333AA, $A3333333, $4AAA3AAA
                LONG $444A4446, $44244444, $44444444, $44444426, $44444444, $B5554544, $BBBBBBBB, $BBBBBBBB
                LONG $4545444B, $44554445, $A3A38888, $554554AC, $55555555, $55555555, $BBBBB5B5, $CBBBBBBB
                LONG $31111333, $33333333, $33333333, $33333333, $33133333, $33333331, $33333333, $77882333
                LONG $77777777, $77777777, $AA337777, $AAAAA3AA, $3AA3A3AA, $33333A3A, $3A333333, $243AA3AA
                LONG $42444444, $44444444, $44444444, $44444444, $44444444, $44244444, $55454444, $BBB5B5B5
                LONG $44445445, $45445554, $33A288D8, $454444AA, $54545444, $BBBB55B4, $BBBBBBBB, $BBBBBBBB
                LONG $11111113, $33333131, $33333333, $33333333, $13333333, $33333333, $33333333, $78786AA3
                LONG $77777777, $77777777, $33A37777, $3A3AAAAA, $AAAAAAAA, $3333AAAA, $A3A33333, $44AAAA33
                LONG $44424464, $44644424, $44444424, $44444444, $44444444, $54454464, $BBB55564, $BB55B554
                LONG $54444445, $44454454, $3A308886, $545B5AAA, $B5B4BB55, $BB554B55, $BBB5BBB5, $BBBBBBBB
                LONG $11111113, $33331133, $33333333, $33333333, $33333333, $33333333, $33333333, $787824C3
                LONG $77777777, $77777777, $A3A47777, $AAAA3A3A, $A3A3A3A3, $333AA3A3, $3333A333, $44AA33AA
                LONG $64444244, $46442444, $44444444, $44444444, $44444A44, $445A4445, $5B44444B, $B5BBB5BB
                LONG $4444444B, $44444444, $A330D8D6, $55B444AA, $4B454554, $BB5BB45B, $5BBBBBB5, $BBBBBB5B
                LONG $11111113, $33313311, $33333333, $33333333, $33333333, $33333333, $33333333, $7778734A
                LONG $77777777, $77777777, $A3B67777, $A3AA3AAA, $A3AAAAAA, $33A3A3A3, $A3A333A3, $44AAA333
                LONG $44464446, $44444444, $4A444442, $4A4444A4, $44444444, $6B5444B4, $B54444B4, $B4BB54BB
                LONG $45454545, $44444454, $AAA98886, $44B54AA3, $B5B5B444, $B5B55B45, $BB5BB55B, $BBBBBBBB
                LONG $11111113, $33331131, $33333333, $33333333, $33333333, $33333333, $33333333, $7777776B
                LONG $77777877, $77777777, $33377777, $AAA33333, $33A3A3A3, $A3AA3333, $3333A333, $644A33A3
                LONG $44444444, $42444442, $44444444, $44424444, $44444444, $B5444B54, $B4444B44, $5BBB4BBB
                LONG $5444444B, $44545444, $33A88886, $45444AA3, $54454B55, $4B5B45B5, $BBBB5BBB, $BBBBBBBB
                LONG $11111113, $33331111, $33333333, $33333333, $33333333, $33333333, $33333333, $77776773
                LONG $77777777, $77777777, $A3B77777, $3A3A3A33, $333AA3AA, $33333333, $33A33333, $444AAA33
                LONG $24424464, $44442444, $44424644, $44444424, $4444444A, $56445B44, $4444B44B, $BBB4BBBB
                LONG $44454444, $45444545, $A3288D84, $45544AAA, $4B5B5444, $B5545B4B, $B5BBBBB5, $BBBBBB5B
                LONG $11111113, $13113111, $33333331, $33333333, $33333333, $33333333, $B1333333, $77777774
                LONG $77777777, $77777777, $AA677777, $AAAA33A3, $33A33A33, $33333333, $33333333, $6424A333
                LONG $44444644, $44244424, $44444442, $44444444, $44444444, $444BB444, $444B5454, $BB44BBB4
                LONG $45444445, $44454444, $33288884, $444442A3, $54544B45, $B5B5B4B5, $BBB5B5B5, $BBBBBBBB
                LONG $11111113, $11331131, $33333313, $33333133, $33333333, $33333333, $A3333333, $77777777
                LONG $77777777, $77777777, $3AA77777, $3A3AA33A, $33333333, $33333333, $3A333333, $4442A333
                LONG $44244464, $44444444, $42442444, $44244244, $44444A44, $44B54444, $44B44B54, $B44BBB44
                LONG $54544444, $45445454, $33088D64, $54544A33, $54454444, $B454B554, $5BBBB5B5, $BBBBBBBB
                LONG $11111113, $33113133, $31333131, $33333331, $33333333, $33333333, $4A333333, $77777777
                LONG $77777777, $77777777, $3A777777, $33A33A3A, $3333333A, $33333333, $333A3333, $4466AA33
                LONG $44442444, $24444444, $44444444, $A4444444, $54444444, $4BB44444, $4B54BB44, $54BBB444
                LONG $44444445, $44544444, $3A9D8895, $44442AA3, $4B545454, $4B5B545B, $B5BB5BBB, $BBBBBBBB
                LONG $11111113, $13111113, $33131131, $31333311, $33333313, $33333333, $7A333333, $77777777
                LONG $77777777, $77777777, $3B777777, $3AA3A3A3, $33333333, $33333333, $A3333333, $42424A33
                LONG $42444442, $44424444, $44244444, $44444444, $44444444, $BB544444, $BB4B5444, $4BBB5444
                LONG $44454444, $44454545, $33988865, $45444A33, $54444445, $55B454B4, $BBB5B55B, $BBBBBB5B
                LONG $13111113, $31133313, $31313313, $33133133, $33333331, $33333333, $6AA33333, $77777777
                LONG $77777777, $77777777, $AA777777, $A3333333, $33333333, $3333A333, $33333333, $44444333
                LONG $44464644, $42444244, $24444244, $44244444, $4444444A, $B5444554, $54BB444B, $4BB5444B
                LONG $44544444, $45444454, $3A888864, $44442A33, $54545454, $5B4B5454, $B5BB5BB4, $BBBBBBBB
                LONG $33333333, $33313333, $33333331, $11311333, $33333313, $33333333, $74B33333, $77777777
                LONG $77777777, $77777777, $AA777777, $3A33A333, $33333333, $A3A33A33, $333333A3, $46242AAA
                LONG $44444446, $44446424, $44424446, $44442444, $44444444, $54445445, $45B444BB, $BB5444BB
                LONG $44444444, $44545444, $3288D845, $45444A33, $44444445, $45545B45, $5BBB555B, $BBBB5BBB
                LONG $33333333, $13333333, $33333333, $33133133, $33333133, $33333333, $77AA3333, $77777777
                LONG $77777777, $77777777, $34777777, $3333333A, $33333333, $3A3AA33A, $A333A333, $24444433
                LONG $44244444, $44644442, $66444444, $44444424, $54444444, $444B5544, $BB445BBB, $BB444555
                LONG $54454644, $54444544, $A0888844, $44442A33, $45454544, $B54B444B, $BB55BB45, $BBBBB5BB
                LONG $33333333, $31333333, $33333333, $31333113, $33333331, $33333333, $773A3333, $77777777
                LONG $77777777, $77777777, $A7777777, $33A33333, $3A3A333A, $A33333A3, $3A3333A3, $444242A3
                LONG $44444244, $44442444, $64444446, $44244A44, $44444444, $44B54444, $B445BB54, $55445B55
                LONG $45444444, $44545445, $39D8D645, $54444A33, $44444444, $4B454554, $B5BB555B, $BB5BBBBB
                LONG $31311313, $13113313, $33331331, $33131311, $33331333, $33333333, $777B3333, $77777777
                LONG $77777777, $77777777, $37777777, $33333333, $33333333, $33A33333, $3333333A, $442444AA
                LONG $42464464, $42444442, $44424644, $44444444, $44444444, $5B444454, $44BBB554, $4445BB5B
                LONG $44444444, $45444444, $39888644, $44442A33, $54545454, $54545444, $5B5B4B4B, $BBBB5BB5
                LONG $11111113, $31331131, $33133333, $33313133, $33333333, $33333333, $77733333, $77777777
                LONG $77777777, $77777777, $27777777, $33333333, $33333333, $33333333, $33333333, $2444242A
                LONG $44444444, $44446446, $44444442, $44424442, $444444A4, $55545445, $45BB5444, $44BB5BB5
                LONG $54444444, $44545454, $39888645, $44442A3A, $44454444, $55444544, $5B55B5B4, $BBBBBB5B
                LONG $11111113, $31111111, $13313111, $33333313, $33333333, $33333333, $7777A333, $77777777
                LONG $77777777, $77777777, $27777777, $33333333, $33333333, $33333333, $A333333A, $44444443
                LONG $644244A4, $42444244, $44464244, $24444444, $44444444, $B4444544, $5BB54555, $45BB5B54
                LONG $44444444, $54444444, $288D8444, $4444A333, $45444444, $B4454445, $BB4B4554, $BB5BBB5B
                LONG $11111113, $11111111, $31131131, $33333333, $33333333, $33333333, $7777A333, $77777777
                LONG $77777777, $77777777, $67777777, $33333333, $33333333, $33333333, $A3333333, $4424424A
                LONG $42444244, $44444444, $46244442, $44444624, $44444444, $44454445, $BB5445B5, $4BBBB44B
                LONG $44444444, $44544544, $08888445, $44442333, $54444444, $44544454, $4555B4B4, $BBBB55BB
                LONG $11111113, $11111111, $13111111, $33313313, $33333333, $33333333, $77777333, $77777777
                LONG $77777777, $77777777, $67777777, $33333333, $33333333, $33333333, $A333A3A3, $24464442
                LONG $44444444, $44424444, $24444244, $62446444, $4444A444, $45444444, $B545BB54, $4BB554BB
                LONG $44454444, $44444444, $288D6444, $4442AA33, $44444444, $54445444, $5B4B4544, $BB5BBB55
                LONG $11111113, $11111111, $13113111, $33131311, $33333331, $33333333, $77777A33, $77777777
                LONG $77777777, $77777777, $77777777, $33333333, $33333333, $33333333, $A3333333, $44446442
                LONG $24464244, $46444424, $44424464, $44244444, $44444444, $44544444, $544B5445, $45BB4BBB
                LONG $44444644, $54445444, $98886444, $444423A3, $44444444, $45454444, $45B54544, $BBBB5B5B
                LONG $11111113, $11111111, $31131131, $13311313, $33333333, $33333333, $77777A33, $77777777
                LONG $77777777, $77777777, $77777777, $333333A3, $33333333, $33333333, $A3333333, $442444A4
                LONG $42442446, $44242444, $44644442, $64424624, $44444244, $54444444, $5455B544, $4B54BBB4
                LONG $44444444, $44544444, $9D8D2544, $4444A333, $44444444, $54444454, $5B5454B4, $B555B4BB
                LONG $11111113, $31111111, $13111113, $33313131, $33333331, $33333333, $777776A3, $77777777
                LONG $77777777, $77777777, $67777777, $33333333, $33333333, $33333333, $A3333333, $64442464
                LONG $44424444, $44444446, $24444444, $44644444, $44444442, $44444444, $4B5B4445, $4445BB55
                LONG $44444444, $44444444, $88884444, $4442A3A3, $44444444, $45445444, $5454B544, $BBBB5B54
                LONG $11111113, $11111111, $11111111, $11131311, $33333313, $33333333, $777777A3, $77777777
                LONG $77777777, $77777777, $87777777, $33333333, $33333333, $33333333, $23333333, $44644424
                LONG $44464424, $46446424, $44424244, $44444444, $44424644, $44444444, $B5444454, $445BB554
                LONG $44444444, $45444444, $88884454, $44642A33, $44444444, $44444444, $B45B4445, $BB5B5B4B
                LONG $11111103, $11111111, $31111111, $33313131, $33333133, $33333333, $27773733, $77717377
                LONG $88888888, $88888888, $8D88D888, $33333330, $33333333, $33333333, $4A333333, $24446442
                LONG $46442444, $44644444, $42444444, $44444424, $44444444, $44544444, $5B545445, $445B5BBB
                LONG $44454444, $44444444, $88D84444, $4442A332, $44444444, $54444444, $4B444544, $B5B54B55
                LONG $11110111, $11111111, $11113311, $31131133, $33331331, $33333333, $77677733, $81673711
                LONG $8D888D88, $D88D8888, $88888D88, $33333330, $33333333, $33333333, $22333333, $64642464
                LONG $24444644, $44444624, $44424424, $44A44444, $44444444, $54444444, $54B54544, $44BB45BB
                LONG $44444444, $54444444, $88864544, $4444A330, $44444444, $44444444, $55454454, $5BB5B4B4
                LONG $11111113, $11111111, $11111331, $13313313, $33333313, $33333333, $71771773, $87171717
                LONG $888D8888, $8888888D, $88888888, $33333330, $33333333, $33333333, $4AA33333, $42424444
                LONG $42424464, $24242444, $44444244, $44444442, $44444242, $44445444, $5B444445, $4445BB54
                LONG $44444444, $44444444, $D8D65444, $444233A9, $44444444, $54444444, $54445444, $BB5B555B
                LONG $11111113, $11111111, $11111111, $31331111, $33333331, $33333333, $17176973, $86717171
                LONG $88888888, $88888888, $88988888, $33333330, $33333333, $33333333, $42233333, $44444626
                LONG $64644242, $64464464, $64442444, $42424244, $44444444, $45444444, $54454544, $44BB5B5B
                LONG $44444444, $44444444, $88864444, $4462A339, $44444444, $44444444, $44544454, $5B55B4B4
                LONG $11111103, $11111111, $11111111, $13133131, $33313333, $33333333, $67107071, $87171717
                LONG $02988888, $3A300209, $89A393A0, $33333339, $33333333, $33333333, $A4A33333, $66424444
                LONG $44424444, $46446442, $42464644, $44444444, $44444464, $44544444, $45544454, $444B55B4
                LONG $44444444, $44444444, $88844444, $44423338, $44444444, $44444444, $44454444, $B4B4B545
                LONG $11111113, $11111111, $11111111, $31113311, $33133131, $33313333, $70770767, $79716717
                LONG $93A08D88, $3A23A330, $80A39AA0, $33333330, $33333333, $33333333, $622A3333, $44444242
                LONG $44444424, $44644644, $44444442, $44642442, $44444442, $54444444, $B4445444, $444BBB54
                LONG $44444444, $44444444, $D8845446, $444A3318, $44444444, $44444444, $45444444, $BB4B4454
                LONG $11111113, $11111111, $11111111, $31331111, $31333331, $63333333, $11707170, $97171717
                LONG $A33A9888, $2399A319, $D0A39A38, $33333339, $33333333, $33333333, $88913333, $69988888
                LONG $44260666, $4444A442, $64424444, $44444444, $44444444, $44544444, $54554445, $444B5545
                LONG $44444444, $44444444, $8D644444, $42623308, $44444444, $44444444, $54544444, $555554B4
                LONG $11110111, $11111111, $11111111, $13133131, $33313111, $73333333, $67117716, $80717161
                LONG $A3A39888, $0B303B31, $80A30AA9, $33333339, $33333333, $33333333, $88933333, $88888888
                LONG $88888888, $60699998, $44444240, $42444244, $44444444, $44444444, $44454544, $44455445
                LONG $44444444, $44444444, $88644644, $44423398, $44446444, $44444444, $44444444, $B4B54B44
                LONG $11111113, $11111111, $11111111, $13111111, $31133113, $13333333, $11101171, $91111111
                LONG $03398888, $80003320, $99999009, $33333330, $33333333, $33333333, $89923333, $88889889
                LONG $88888888, $88888888, $99998888, $44226660, $444A444A, $5554B454, $5B444555, $444B4B44
                LONG $44444444, $44444444, $D8644444, $424A3398, $64444444, $44444444, $45445444, $5B445445
                LONG $10111103, $11111111, $11111111, $31131111, $33311331, $13333333, $11111111, $90111111
                LONG $88998988, $99889989, $99999899, $33333339, $33333333, $33333333, $98833333, $98988898
                LONG $89889898, $88888888, $88888888, $88888888, $24269099, $4A4444A4, $B5BBBBAB, $4445BBBB
                LONG $444A4444, $44444444, $88444444, $446A3388, $44444444, $44444444, $54444444, $B5BBB554
                LONG $11111013, $11111111, $11111111, $11111111, $13133131, $13333133, $11111111, $01111111
                LONG $90999999, $99909999, $99999099, $33333310, $33333333, $33333333, $8980A333, $88889888
                LONG $98888888, $98999898, $89899898, $88888898, $99989898, $99999998, $02606069, $69624262
                LONG $99999696, $98888889, $88266998, $44233288, $44444446, $44444444, $B4BB4444, $BBBB5BAB
                LONG $11111113, $11111111, $11111111, $11311111, $33113313, $13331331, $11111111, $00111111
                LONG $90099990, $09009909, $99990090, $33333330, $33333333, $33333333, $89893333, $88988889
                LONG $88898989, $98888888, $98988988, $99989989, $98898989, $98989899, $88898899, $88999899
                LONG $88889898, $99899998, $D4688988, $8D923288, $88888D88, $68988888, $99999699, $09999969
                LONG $10110111, $11111111, $11111111, $31131111, $11331331, $11333133, $11111111, $01111111
                LONG $09009009, $90990090, $99099909, $33333339, $33333333, $33333333, $89891333, $88888889
                LONG $98898888, $89889988, $98998989, $88998988, $99898988, $99898988, $89999998, $99988989
                LONG $99999999, $89989899, $8A889899, $88033088, $88888888, $88888888, $88888888, $89888888
                LONG $11111013, $11111111, $11111111, $11111111, $33113131, $11333333, $11111111, $01111111
                LONG $90990999, $09999909, $99999999, $33333330, $33333333, $33333333, $88882333, $88898988
                LONG $88988988, $98898889, $89888988, $99889899, $88989899, $98989899, $98988898, $99899898
                LONG $99989998, $99999989, $86989999, $882339D8, $98989989, $98989898, $98989988, $99999999
                LONG $11101113, $11111111, $11111111, $11311111, $33331313, $11333331, $11111111, $00111111
                LONG $90909990, $99090909, $90990990, $33333330, $33333333, $33333333, $89889333, $88988889
                LONG $88888888, $88988988, $89898889, $88899898, $89989898, $98989898, $89899998, $88988989
                LONG $88899989, $88989889, $89989999, $88233988, $98888888, $98898988, $88989889, $99988888
                LONG $11111111, $11111111, $11111111, $31131111, $13311331, $11133113, $11111111, $90111111
                LONG $09909999, $90990990, $99909909, $33333330, $33333333, $33333333, $888883A3, $89888988
                LONG $88898898, $89888889, $98898898, $99988988, $98898989, $89898989, $98988889, $99899898
                LONG $99998898, $99989898, $86898888, $8933188D, $88888888, $89889888, $99898998, $98989999
                LONG $11101103, $11111111, $11111111, $11311111, $33133113, $1113A331, $11111111, $00111111
                LONG $90990099, $09009909, $90090099, $33333330, $33333333, $3A333333, $89888033, $88898898
                LONG $88888889, $88899888, $89888988, $98998898, $89988988, $89898989, $89899989, $98998989
                LONG $89889989, $88989989, $96899999, $893A2888, $89889889, $89988898, $89898899, $99898888
                LONG $10110111, $11111111, $11111111, $31111111, $33331313, $11113313, $11111111, $01011111
                LONG $00909909, $90999099, $90909990, $33333330, $33333333, $3A333333, $98888933, $88888888
                LONG $98898888, $89888988, $98898889, $88898988, $89889899, $98989898, $89898898, $89898989
                LONG $98998899, $99898898, $69999889, $82332D88, $88888888, $89889889, $98989988, $98989999
                LONG $11011013, $11111110, $11111111, $33131111, $13131131, $11113333, $33111111, $01111311
                LONG $09009090, $01100000, $11111111, $33333331, $33333333, $3AA33333, $88888933, $89888889
                LONG $88888989, $88888889, $88988898, $98989888, $99898888, $98898898, $89898998, $89898989
                LONG $88989999, $88889989, $20898989, $8233088D, $89898988, $98989888, $88989898, $98989888
                LONG $11111113, $11111111, $11111111, $13311111, $31111113, $33313331, $A3A33333, $1111333A
                LONG $11111111, $11111111, $11111111, $33133331, $33333333, $3A333333, $8888880A, $88898988
                LONG $88888888, $88898888, $89889888, $89888898, $88898998, $89889889, $98989889, $98989898
                LONG $98989898, $89998898, $66898989, $82338888, $88988888, $98898898, $89888988, $99898899
                LONG $11111111, $11111111, $11111111, $11111111, $33311313, $AAAAAB31, $AAAAAAAA, $11113333
                LONG $11111111, $11111111, $11111111, $33133331, $33333333, $BAA33333, $88D88893, $88888888
                LONG $88888888, $88888888, $98888888, $88988988, $98898889, $98998988, $89898998, $89898989
                LONG $89898989, $98899889, $A9899898, $800988D8, $98898888, $88888889, $98898888, $98989888
                LONG $01111103, $01111111, $11111111, $11111111, $31113111, $3AAABAA1, $33AAAAAA, $1111133A
                LONG $11111111, $11111111, $11111111, $11011111, $11111111, $3CA30111, $88888883, $88889888
                LONG $88888888, $88888888, $88988888, $89888888, $98988988, $89889898, $98989889, $98989898
                LONG $98989898, $89988998, $28898989, $89998888, $89889888, $98888988, $89888898, $99888989
                LONG $10000001, $10011000, $00020000, $01001001, $00000010, $AA3A33B3, $33A3A33A, $11111133
                LONG $11111111, $11111111, $01113111, $99990900, $99999999, $09099999, $88888889, $88898888
                LONG $88888888, $88888888, $88888888, $98888988, $88898888, $98988889, $98898998, $89898988
                LONG $89898989, $89889989, $48898989, $89998886, $88988888, $89888888, $88898888, $98899888
                LONG $99909900, $99999999, $99999990, $99999999, $99999999, $333AAAC1, $3A333333, $11111133
                LONG $11111111, $31111111, $00113A33, $99999900, $99999999, $89999999, $88888889, $88988888
                LONG $88888888, $89888898, $88888888, $88889888, $89888988, $88988998, $89889889, $98989898
                LONG $98989898, $98998898, $A8989898, $999988D6, $88898888, $88888889, $98888988, $99988988
                LONG $99889889, $99898009, $98999998, $99989989, $99999999, $333333AA, $333A3333, $11111113
                LONG $00000101, $10000000, $99990000, $99999999, $99999999, $99999999, $88D88888, $8888888D
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $98889888, $89898888, $89889889
                LONG $98989898, $89899898, $08898989, $89988886, $88888888, $88888888, $88888888, $98898898
                LONG $98998999, $98989999, $89999898, $88898998, $99989998, $13113333, $33333333, $99901113
                LONG $89999999, $03009999, $99998999, $99998989, $99999999, $89999999, $88888888, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $88988889, $88889889, $98888988
                LONG $89898989, $98988989, $68989898, $9998D88A, $88888888, $88888888, $88888888, $99989889
                LONG $89898989, $98989899, $99999898, $99989899, $99889989, $10000009, $33333311, $99990011
                LONG $89901109, $30999999, $99999990, $99989999, $99899998, $88999999, $8888D888, $88888D88
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $88888888, $89988898, $88989898
                LONG $98988888, $98989898, $98898989, $98988884, $88888888, $88888888, $98888888, $98888988
                LONG $98898989, $98989898, $98889898, $89989999, $99899898, $99998988, $01000099, $98999990
                LONG $98999909, $00999999, $99999999, $99999999, $99998998, $89989999, $D8888888, $88888888
                LONG $88888888, $88888888, $88888888, $88888888, $88888888, $88888888, $98888888, $89888988
                LONG $98888899, $88888988, $88989898, $99988D6A, $88888888, $88888888, $88888888, $99888888
                LONG $98888898, $98989988, $99989898, $98998988, $98989999, $89889999, $91099999, $98889888
                LONG $99999998, $90099999, $99999999, $99999999, $99999999, $88898999, $88D888D8, $888D8888
                LONG $88888888, $88888888, $888D8888, $88888888, $88888888, $88888888, $88898888, $88898898
                LONG $88888888, $09989898, $88989899, $9998886A, $88888888, $88888888, $88888888, $88888888
                LONG $89899889, $89898898, $88898989, $89889899, $89999988, $99998999, $60998999, $9187A787
                LONG $99999989, $99109989, $99999998, $99999999, $89999999, $06009989, $88888888, $888888D8
                LONG $88888888, $88888888, $D8888888, $88D88D88, $88888888, $88888888, $88888888, $98888888
                LONG $88888898, $AB088888, $898989AA, $998888A4, $88888889, $88888888, $88888888, $99898888
                LONG $89898989, $98989989, $89989898, $98898898, $99898889, $89999898, $99889999, $26762778
                LONG $99999988, $99000999, $99999999, $99999999, $89999999, $69669608, $96662622, $88888899
                LONG $88888888, $D8888888, $88888888, $88888888, $88888888, $88888888, $88888888, $88888888
                LONG $88888988, $69988888, $88998892, $9998D8A2, $88888D89, $88888888, $88888888, $98888898
                LONG $98989899, $89898898, $89898989, $99090089, $99989999, $99999999, $88899999, $6765A788
                LONG $99999892, $99900099, $99999999, $99999999, $98999999, $88888896, $88988888, $88888686
                LONG $88899988, $88888888, $88888888, $8888D888, $88888888, $88888888, $88888888, $88898888
                LONG $88888889, $88888888, $88888988, $998888B6, $88888889, $88888888, $98888888, $88988888
                LONG $89898989, $98989989, $00989898, $3BABC320, $99900000, $09999999, $88888880, $74A46788
                LONG $999996A4, $99990009, $99999999, $99999999, $A9899999, $88888889, $8888888D, $88888888
                LONG $88888888, $88696966, $98888888, $88888868, $88888888, $88888888, $88888888, $88888888
                LONG $88888888, $88888D88, $D8888888, $998888A2, $888D8889, $88888888, $88888888, $98888888
                LONG $98989899, $89898898, $10998989, $13A32090, $89999899, $19888988, $888D892A, $6A788888
                LONG $990A3226, $99989999, $99999999, $99999999, $9A089999, $88898988, $88888888, $88888888
                LONG $98888888, $98669696, $28866666, $62222222, $88888889, $88888888, $88888888, $88888888
                LONG $88888889, $888D8888, $88888888, $998D8858, $88888890, $88888888, $88888888, $98889889
                LONG $89898989, $98989989, $89889898, $33119899, $9999033A, $98888899, $8823AAA2, $88888888
                LONG $00099688, $99999999, $99999999, $99999999, $89A08889, $99999999, $99999999, $99999999
                LONG $88988988, $98888888, $A9969990, $AAAAAAA2, $88888888, $88888888, $88888888, $88888D88
                LONG $88888888, $88888888, $88888888, $998889A6, $88888899, $88888888, $98889888, $89888988
                LONG $98989899, $89898898, $98998999, $00019998, $99903320, $99888999, $AAB20009, $8899992A
                LONG $88888888, $89989889, $99999999, $99999999, $999A2999, $99999999, $99999999, $90999999
                LONG $08999989, $99909900, $99999999, $62666690, $88888888, $8D888888, $D88D8D8D, $888D8688
                LONG $8888888D, $68888888, $06298889, $09888886, $8D88D899, $88888888, $88888888, $98898889
                LONG $98989899, $89899898, $98998989, $00109999, $99990999, $9893A099, $20900231, $2ABAAB3A
                LONG $90022022, $98899999, $99999999, $31999999, $99992200, $99990999, $99999999, $90000999
                LONG $98899988, $90999990, $99999999, $9662A606, $99999006, $99899999, $88898999, $88886088
                LONG $D8888D88, $D8888888, $D8D88888, $08888888, $88888809, $88888888, $88888898, $89888988
                LONG $89898989, $98988989, $89989989, $90999999, $99999999, $89910999, $09000119, $AA3A0909
                LONG $03ABCBBB, $99999990, $99999999, $90909009, $90009020, $99099999, $99099909, $99999090
                LONG $98899999, $99990999, $09090999, $B4666692, $99099696, $99999999, $99999999, $99999999
                LONG $99899990, $88898989, $88888889, $998D8698, $88888990, $9888888D, $99889888, $98988888
                LONG $89898989, $89989989, $98999899, $99999998, $99999999, $99998999, $99999999, $99999999
                LONG $90200909, $99999999, $99999999, $00200099, $02102002, $99999990, $99999999, $99099999
                LONG $00999009, $99099999, $90909990, $CB5468D6, $00990994, $99000909, $90990090, $09000900
                LONG $90906999, $09909909, $99099990, $90888909, $888D8999, $88888888, $88888989, $98888898
                LONG $89898989, $99899999, $99999998, $99999999, $99999899, $99999999, $99999999, $99999990
                LONG $99999999, $99999999, $90999999, $22990990, $99999999, $90990999, $99990990, $90990990
                LONG $02099990, $99999099, $89999909, $BCB56688, $0909992B, $90990909, $99099090, $99099099
                LONG $99244AA9, $99099090, $90909090, $98888900, $8D886090, $88888D88, $98898888, $98988988
                LONG $99989899, $99998998, $99999999, $99898999, $99999999, $99999999, $30999999, $9999903B
                LONG $99999999, $09909090, $99999099, $0A299999, $90990999, $99999009, $90999999, $99999999
                LONG $90099090, $09909909, $80909999, $BBCBB568, $99092BCC, $99990990, $09909909, $09909090
                LONG $92BB64BB, $00000900, $00000900, $39889900, $88666009, $88888888, $89889898, $98898888
                LONG $98998989, $99999899, $99998999, $99999999, $99999999, $99999999, $99999999, $99099999
                LONG $99999999, $99999999, $99909999, $99030999, $99099999, $99090990, $99990909, $90909090
                LONG $00099000, $99990999, $99990900, $44446888, $9999004A, $09009999, $90909090, $60900090
                LONG $0ACB4786, $09000000, $00000000, $A69BA909, $86429090, $88D8888D, $88888888, $89889889
                LONG $89899999, $98898999, $99999998, $99999999, $99999999, $99999999, $99999999, $09999999
                LONG $99999099, $90909999, $90999990, $99999909, $99999909, $09999999, $90099999, $09009999
                LONG $02909090, $90909900, $99000009, $88888888, $00000968, $90990090, $00900900, $99000900
                LONG $6CBBCCB2, $90000009, $00090900, $9882A000, $66890000, $8888D888, $88898888, $98898898
                LONG $99999899, $99999998, $99999899, $99999999, $99999999, $99999999, $99999999, $99909909
                LONG $90999999, $99999099, $99990909, $90999999, $90990999, $99909909, $99999099, $90900990
                LONG $90099000, $00900090, $90090909, $878888D8, $00009666, $90000900, $00099000, $2A000000
                LONG $BABBBB4A, $0000009A, $00000000, $88880909, $40889000, $88888892, $88888898, $99889898
                LONG $99998999, $99999999, $99999999, $99999999, $99999999, $99999999, $09909909, $90999999
                LONG $99909909, $99099990, $99099099, $09909990, $99999099, $00990999, $09009909, $00090009
                LONG $00090900, $00090909, $60000000, $88888862, $00006247, $00000000, $90000000, $44200000
                LONG $68878888, $9000001B, $00000900, $88886000, $90299000, $00000000, $00900090, $20000000
                LONG $89899989, $99899899, $99999998, $99999999, $99999999, $90999909, $90999990, $99999099
                LONG $99990999, $99990999, $00909990, $90900009, $00900900, $09009909, $00090090, $09000090
                LONG $00000009, $00000000, $26900000, $77888864, $00063647, $00000000, $10000900, $76469000
                LONG $66787878, $0000009A, $00000000, $888D9200, $00220029, $00000000, $00000000, $00000000
                LONG $98998999, $99999999, $99999999, $99999999, $99099999, $99990999, $99999099, $09009909
                LONG $09099999, $90099900, $90900000, $00009090, $09099909, $00000009, $00009000, $00009000
                LONG $00909000, $00090900, $72200000, $8D88888D, $00667747, $00900000, $22000000, $4A4A2000
                LONG $74477787, $0000006A, $00000000, $8D889000, $00220926, $00000000, $00000000, $00000000
                LONG $99999999, $99999998, $99999999, $09990909, $99990909, $09909990, $09009999, $00090990
                LONG $00090000, $00900909, $09090909, $09090000, $00000009, $90090000, $00090090, $00009009
                LONG $90000090, $00000000, $87660000, $787D88D8, $94A47774, $00000000, $02000000, $77778000
                LONG $67766777, $00000665, $00000020, $888D9200, $002009AA, $00000000, $00000000, $20000000
                LONG $99999999, $99999999, $09999999, $99099999, $90999999, $90999999, $00900009, $90900009
                LONG $09900900, $00090000, $90000000, $00909990, $00000000, $09000009, $00000000, $00900000
                LONG $00099000, $00000000, $87D62600, $78888888, $4B577777, $00000009, $00000000, $777D8820
                LONG $78876777, $00006A7B, $00000000, $88888000, $02226009, $00000000, $00000000, $00000000
                LONG $99999999, $99999999, $99999999, $99999099, $09990999, $99009009, $00900900, $00000000
                LONG $00090000, $09000090, $09000000, $00000009, $99000090, $00000900, $00000900, $00000000
                LONG $00000900, $00000000, $78886260, $77888888, $A5767747, $00000004, $00000000, $77888800
                LONG $88877777, $00016374, $00000020, $88880200, $00200298, $00000000, $10000000, $20000000
                LONG $99999999, $09099999, $99090999, $09909990, $00909909, $00990900, $90090009, $90900900
                LONG $09000900, $00000009, $00000090, $09090000, $00000900, $90090000, $00000000, $00000000
                LONG $09000000, $00000000, $88887860, $7888D887, $57778777, $00000064, $2A900000, $7B568890
                LONG $6D777777, $00001A75, $00000020, $88D92200, $00200008, $01000100, $00100100, $00000000
                LONG $99999990, $99990999, $90999909, $99099909, $99999099, $99000909, $00009090, $00099090
                LONG $09090000, $00090090, $90090000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $7788D788, $78888878, $67888777, $00000255, $03A00000, $4B47D892
                LONG $A4667777, $00010474, $00002020, $88802200, $00202008, $00010000, $01001010, $10010010
                LONG $99999990, $99099990, $09999999, $09990099, $09909999, $09909099, $09000999, $99009009
                LONG $00009009, $00000000, $00000000, $00000090, $00000000, $00000000, $00000000, $00000000
                LONG $00009000, $62200000, $88D87888, $87788787, $67887867, $010044A4, $22202001, $4B468822
                LONG $B4777777, $010107AB, $01100302, $8D822021, $00200208, $01000101, $10100001, $21000000
                LONG $09909099, $09999099, $99909099, $99999999, $99099909, $90090000, $00900000, $00000000
                LONG $00000090, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $09000000, $66AA2000, $77888888, $87878778, $77878787, $02044B47, $02000010, $44667762
                LONG $77867767, $002023A4, $10110221, $88823010, $22222298, $00010012, $01010100, $11010001
                LONG $99999990, $99909999, $90999990, $90909990, $00090009, $00900000, $00090909, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $21100000, $00000111
                LONG $00000902, $88664A22, $788968D7, $78778777, $77787877, $102BA546, $10202001, $77777822
                LONG $77677777, $12010357, $10102120, $8892A020, $00020A08, $10100010, $00100101, $10101001
                LONG $90999090, $99099909, $99999999, $90990090, $00900990, $00090000, $09000900, $00000090
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00010010, $00000000
                LONG $49029000, $887D7444, $8D922098, $87877787, $77777787, $20AA5545, $01110200, $67777760
                LONG $77777477, $10202A24, $01102320, $880A2002, $03220088, $01011102, $10011010, $10100100
                LONG $99099999, $09999999, $09090099, $09000909, $09099000, $90909009, $00090090, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000001, $00200000
                LONG $464A6902, $88878746, $88220222, $78788777, $56777777, $22277445, $10012120, $77877622
                LONG $77577777, $22020665, $10010222, $88932010, $B09AB288, $01100101, $01001011, $10010201
                LONG $99909990, $90990990, $00009900, $00009000, $00000909, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00100100, $00000001, $00000000, $20020000
                LONG $7766662A, $26888677, $78200002, $77788778, $64678787, $20226777, $20010021, $77778720
                LONG $74B47777, $0222A445, $10102302, $88803220, $A8629088, $10012123, $02010100, $21121021
                LONG $99999090, $90099909, $90900090, $90090090, $00000000, $00000090, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $01000000, $00000000, $01001000, $21000000, $A4A00202
                LONG $8687866B, $00206886, $89220000, $878D7888, $76747878, $02222667, $21212201, $77777762
                LONG $77BB7777, $024BB454, $02101210, $89889003, $B4288888, $01112090, $10211111, $21022112
                LONG $90999990, $09909000, $00009000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00010100, $01011001, $10010110, $00101111, $ABA42202
                LONG $88689664, $02022298, $D0200120, $78888888, $67674B47, $00022A77, $20002020, $77777872
                LONG $575B7777, $12654A57, $21011112, $89922000, $2B66668D, $10009896, $12112010, $32010220
                LONG $99909990, $00000990, $90090099, $00909090, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $01000010, $00000100, $01000001, $00010010, $57862320
                LONG $20089986, $00200222, $80210201, $878887D8, $88887788, $12001644, $22122221, $77787786
                LONG $76BC4757, $22B47575, $22322212, $8886ABB2, $88644466, $01229869, $20202121, $13201202
                LONG $90999099, $09090909, $00900900, $99000009, $09000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00010000, $10100001, $00000100, $20001101, $A48D8922
                LONG $20099968, $01111020, $80201001, $88D78878, $77888888, $212022A5, $2223A020, $77777772
                LONG $44B47777, $64555577, $A4299998, $988996A4, $88888996, $02209888, $20021021, $20122010
                LONG $90990990, $90909000, $09009090, $00009090, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000002, $00010000, $01101010, $01011000, $10110010, $22010000, $62998801
                LONG $06899888, $11020200, $89110112, $8888D778, $4787878D, $20021645, $2222A122, $77455662
                LONG $77757777, $BB745747, $666605B4, $09888862, $88888699, $21222988, $22022112, $32212202
                LONG $00000900, $00900090, $00090000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00100100, $10010001, $00000100, $01001001, $90220110, $89A20989
                LONG $99888918, $96999999, $60022006, $8D88888D, $76888D86, $00222244, $99869902, $747BBB56
                LONG $65765477, $5BBBB574, $66666264, $2A988966, $68880A22, $22223A99, $32222220, $21020222
                LONG $90909000, $00090009, $90009000, $90000900, $00000900, $00000000, $00000000, $00000000
                LONG $00000000, $01000100, $11001001, $00100100, $01110110, $01001010, $08990221, $08823220
                LONG $20888022, $A6606020, $06899824, $88887678, $78777788, $66698667, $4B544466, $774445B5
                LONG $77577557, $6644B547, $66666666, $22398906, $86882220, $21121229, $02121213, $32222223
                LONG $09000900, $00009090, $00000090, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00010000, $00010001, $00000100, $01010010, $10001001, $12110100, $21298990, $22989212
                LONG $229D9020, $BB222222, $88876655, $8777D886, $88876688, $CCBBBB58, $76BCBCBC, $55757568
                LONG $46774775, $66666665, $4B446666, $12229888, $86692A20, $12102222, $22221212, $22222202
                LONG $90909000, $09000900, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00100100, $10100020, $00111000, $10101001, $10211010, $92001001, $12012098, $12309892
                LONG $08888211, $46792000, $744BB546, $7788788D, $88778778, $BBBCBCB6, $6884CBCB, $46777446
                LONG $64667556, $44466666, $95BBBBB5, $12222968, $8D822002, $22222396, $02012222, $A4A22222
                LONG $00090090, $90909090, $00909900, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000001, $00100101, $10000001, $01010100, $01000101, $80320110, $93321229, $10013098
                LONG $29988210, $7870AA22, $54677747, $8877D876, $88786888, $CBCBBCB6, $86686BCB, $44446766
                LONG $66666474, $26866666, $88866626, $00096998, $86886299, $22022246, $5A242222, $BCBBBB4A
                LONG $90909000, $12320000, $23A3A232, $00000090, $00000000, $00000000, $00000000, $00100000
                LONG $01000000, $00000000, $10111100, $10101010, $00001010, $98922202, $98020121, $01002220
                LONG $96299221, $78449696, $77777677, $78D66656, $88867888, $BBCCBCB5, $888884BB, $78866888
                LONG $8D88888D, $D8DD8D88, $888D8D8D, $20223688, $66666422, $A44AABCB, $BBBBBBBB, $BBCBCBCC
                LONG $00009900, $A331A3AA, $BBABAAAA, $000093AB, $00000000, $00000000, $00000000, $10000000
                LONG $00101000, $10010001, $01000101, $01010101, $20210001, $12989222, $20988990, $32311002
                LONG $88D88022, $D6C4448D, $54466678, $865B5B45, $78D88D88, $66464444, $88888866, $D8888888
                LONG $88888D88, $9989888D, $88888696, $66066068, $46666662, $BCCBCBBB, $BBCBCBBC, $BBBBBBBB
                LONG $003AA000, $1A200000, $CBACBAA2, $001AABCA, $00000000, $00000000, $00010101, $00001001
                LONG $10000010, $01000100, $10011010, $01101010, $31223011, $88608892, $22202998, $90222310
                LONG $88888888, $8682A666, $66467688, $88688874, $88888888, $88D8D88D, $D8888888, $99888888
                LONG $26069099, $22A22222, $8D8886A2, $66666666, $55666666, $CBBCBBBB, $BBBBBBBB, $46445555
                LONG $232AA302, $23A22A20, $BBCA3BCA, $000BCCBB, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10010010, $01000100, $21010010, $92212010, $99888888, $202122A0, $888802A2
                LONG $88888888, $88888888, $88888888, $88888888, $8888D88D, $88986888, $69698886, $A2222069
                LONG $222A24AA, $66666262, $688D8666, $66666666, $B5566666, $5BB5BBBB, $64644545, $66666666
                LONG $A223A200, $AAAAABBA, $AABACBAC, $02ACBBB3, $22012000, $123A3323, $00000000, $02100000
                LONG $20021002, $00000000, $00111000, $00001201, $D8902320, $A2288888, $23221222, $8888D822
                LONG $888D8D88, $888DD888, $888888D8, $88888888, $88888888, $66666668, $26626626, $96968666
                LONG $66688888, $66666666, $88886666, $64666666, $55546646, $66646444, $66666666, $46666666
                LONG $BAA22223, $BBBCA3CA, $CC3BACBB, $ACBBCBCB, $AA3A3A22, $ABA3AAAA, $0022023B, $00202000
                LONG $00000120, $00220202, $12020012, $22202122, $88899233, $222888D8, $22102012, $88D88803
                LONG $8688888D, $29066886, $66568688, $6626A688, $66666766, $66666666, $66666666, $66666666
                LONG $86866666, $88668788, $6D8D6666, $66666666, $66666666, $66664646, $44646464, $BBB55544
                LONG $A3A3200A, $3CAAAAA3, $AABACBA3, $AABCBABB, $AAAA3AA2, $3332AA3A, $023BAABA, $20002020
                LONG $01220200, $20202002, $01200200, $33221011, $29898890, $24A88260, $4AAAAA4A, $8D86444B
                LONG $BBBB5888, $6664CBBB, $86555666, $66666666, $66666666, $66666666, $66666666, $66666666
                LONG $66666666, $66666666, $66688886, $66666866, $44544666, $55554445, $BBB5B5B5, $BBBBBBBB
                LONG $33A22322, $A3AA3A33, $BCABABAC, $2BCBBCB3, $32320120, $3AA3A3A3, $A3AA3A3B, $0222202A
                LONG $00000022, $01021210, $02011103, $82232A22, $02292298, $BCBB6896, $CBBBBBBB, $865BBCBB
                LONG $BBCBB888, $58884CBC, $866BB5BB, $66666666, $66666866, $66666666, $66666666, $86886666
                LONG $68686868, $86886866, $56666668, $4666665B, $B5454466, $BCBBBBBB, $BBBBBBBC, $555BBBBB
                LONG $22021200, $AA3AA3A3, $AB3AAAAA, $BCBBBBCA, $02000002, $A3222000, $3A332A33, $3A33AA33
                LONG $A3AAAAAA, $2211AAA2, $23321120, $98899022, $88333A20, $5BBB6888, $BBBBBBBB, $654BB5BB
                LONG $B5BB58D8, $46666455, $86865445, $66866886, $66666668, $66866666, $86688866, $86688888
                LONG $66668686, $68688866, $B6666666, $BBBBBBBB, $BBCBBBBB, $BBCBBCBC, $4555BBBB, $66666444
                LONG $00002002, $33222222, $A33AA3A3, $CBBCAAAC, $0000003B, $32000202, $222A3AA2, $322223A2
                LONG $AABBAAA3, $23BBAAAA, $89002332, $BA269888, $89AAABAA, $66688D88, $66666666, $88686666
                LONG $68886888, $88888888, $6D688886, $8868886D, $88888886, $D6688888, $68866888, $66687886
                LONG $86866866, $66464686, $B5544446, $BBBBBCBB, $BBBBBBBB, $455555BB, $66666646, $46466666
                LONG $02000000, $20200000, $A3A1A323, $ABABACAA, $22000ABC, $00002033, $02000000, $02020002
                LONG $AA333A3A, $BBBCCA3C, $2888892A, $BCBCBA22, $82BACABC, $8868888D, $88866686, $66866686
                LONG $66666666, $86866668, $88886868, $86D68686, $66868688, $66866666, $66466466, $44544444
                LONG $55554644, $BBBBBBBB, $BBBBBB5B, $B5BBB5BB, $64445445, $66666664, $64666666, $4644B564
                LONG $00909090, $02000000, $12220000, $33A33212, $002ABABB, $00020200, $20020002, $02020200
                LONG $23020000, $9992AB31, $AAA88D88, $CBAAAA3A, $9ACBCBCB, $26260609, $66666966, $66668696
                LONG $88666666, $86868886, $66866866, $44446686, $55555545, $B5555555, $BBBBB5BB, $5686BBBB
                LONG $BBBBB5B5, $5BBBBBBB, $455555BB, $66464646, $44466666, $44444646, $ABA44464, $444ABAAB
                LONG $09000900, $00000000, $00000000, $A3222200, $023A3333, $00200000, $01201000, $20110022
                LONG $02202220, $D8888922, $22088888, $02221220, $622223A2, $66226066, $66666626, $88868666
                LONG $66868666, $66666666, $66666666, $BBBBB556, $BCBBBBBB, $BBBCBCBB, $455BBBBB, $55466644
                LONG $44454555, $64644444, $66666664, $44446444, $54644464, $22AAABAA, $4AB4ABA4, $62442442
                LONG $90090090, $00000009, $20202000, $00222020, $00202000, $00000000, $20000000, $02002010
                LONG $01211020, $89088921, $966688D8, $66060626, $26266666, $66666666, $66666666, $66666666
                LONG $66666666, $66666666, $46666666, $5BB55BB5, $4555555B, $44444445, $66666666, $66666666
                LONG $66666666, $66666666, $42444466, $62AAAA42, $22426626, $464A2262, $4A2A2242, $46222442
                LONG $00990900, $00090900, $02000009, $00220000, $00000000, $00000000, $00000000, $10000000
                LONG $00000000, $88208900, $66998888, $22606066, $69620602, $66666666, $66666666, $66666666
                LONG $66666666, $66666666, $66666666, $66666666, $66666666, $66466644, $66646666, $66666666
                LONG $66646666, $46646646, $66060626, $26222262, $62062260, $22262222, $22422626, $24224224
                LONG $90000000, $09000000, $00090000, $20000000, $00002000, $00000000, $00000000, $00000000
                LONG $00000000, $00200000, $00090900, $00000000, $99000202, $69868699, $66966966, $66666666
                LONG $66666666, $66666666, $66666666, $44666666, $AAAAAA4A, $AABBBBBB, $62626AA2, $64426466
                LONG $66466264, $69666666, $06060600, $26060606, $60600006, $00022022, $22022220, $A2222222
                LONG $00009000, $00000000, $00000000, $00000000, $02000200, $00020220, $02000020, $02200220
                LONG $00000020, $00000000, $20020200, $00200000, $22220202, $90602022, $88899999, $86888888
                LONG $68686886, $68868688, $68868888, $3ACAAA06, $A3AA3222, $223B3AAA, $96090609, $99900900
                LONG $06906690, $66996996, $60906069, $00602609, $02026606, $22002062, $20220020, $22222222
                LONG $02A90090, $00000099, $00000000, $00000000, $20000020, $02000000, $22002002, $23220020
                LONG $20202020, $00000222, $02000002, $20020202, $00200020, $22000202, $02222222, $86898606
                LONG $62666068, $A4222242, $AAA62466, $000AAAA3, $22006090, $22000222, $22222222, $22060262
                LONG $20600009, $02020622, $09669600, $90060062, $22202060, $02206220, $22202202, $22222222
                LONG $90200900, $00090900, $00020000, $00000000, $00000202, $02002020, $20022000, $20020220
                LONG $02002020, $20022000, $20222220, $00200020, $20200202, $20202020, $22220220, $22222222
                LONG $22222202, $22222202, $22202002, $22222222, $222222A2, $22222222, $2222222A, $22222222
                LONG $2222222A, $22222222, $22002022, $22222202, $22222222, $22222222, $22222222, $A2222222
                LONG $00900090, $00000000, $00000000, $02020000, $00000000, $00200000, $00020020, $20202002
                LONG $20020020, $22202000, $22000000, $22020202, $22222220, $22222220, $22222222, $22222222
                LONG $22222222, $22222222, $22222222, $A2222222, $2A222222, $2A2222A2, $22222222, $22222222
                LONG $22222222, $22222222, $22222222, $22222222, $22222222, $22222222, $22222222, $22222222
                LONG $90009000, $00009090, $20000000, $00000200, $02002000, $20000000, $22002020, $22020220
                LONG $22222020, $00200220, $22200220, $02222220, $22222022, $22222223, $22222222, $22222220
                LONG $22222222, $22222222, $22222222, $22222A22, $A222A222, $22222222, $2A222222, $22242222
                LONG $22422422, $422422A2, $22A22222, $2A22A222, $22422224, $24224224, $2AAA22A2, $A2A2AA2A
                LONG $00090000, $00000000, $00000000, $00000000, $00200202, $A0020002, $00220002, $00202002
                LONG $22202202, $20222022, $00202202, $22220202, $22222222, $22222122, $22222222, $22222222
                LONG $22222222, $22222232, $22222222, $22222222, $22222A22, $22A2A222, $A222A22A, $2AA22AAA
                LONG $A2A2222A, $22A22A2A, $2A222222, $A2A2A242, $2A2AAA22, $22422AA2, $AA2AAA2A, $AA3A2AA2
                LONG $00000000, $00000000, $02000000, $02002000, $02000000, $00200220, $22200202, $22020220
                LONG $22022020, $22222022, $22222222, $22222222, $22122222, $22212212, $22222222, $22222222
                LONG $22222222, $22222221, $22232222, $22222222, $22222222, $2A2A22AA, $2A2AA2A2, $2422AA2A
                LONG $222A2A2A, $2A2AAAA2, $422A242A, $A2AAA232, $A2A22242, $AA2AAA2A, $A2AAA2AA, $A2AA2A2A
                LONG $00000092, $00000000, $00000000, $20000000, $20002000, $22020200, $00022020, $02202002
                LONG $20222022, $22222202, $22222222, $22222222, $22222122, $21222222, $22222222, $22223222
                LONG $A2222222, $22223222, $32222223, $22422A2A, $2A222224, $A32224A2, $22222A22, $22222222
                LONG $22222242, $AAA2222A, $AAA2A2A2, $2A2A2AAA, $242A42A2, $24A242A2, $2A4A424A, $A2AA22AA
                LONG $09000000, $00000000, $00000200, $02000000, $00200200, $00020020, $22002020, $20220220
                LONG $22220202, $22022222, $22222222, $22222222, $22222222, $22222122, $22222222, $22222222
                LONG $22222222, $A2232232, $22A2A222, $AA2A22A2, $22AA2222, $2AA2222A, $A2AA2AAA, $AAA22422
                LONG $A242AA2A, $A2AA42A2, $22A2AAAA, $AAA2A2AA, $AAAAAAAA, $AAAAAAAA, $AA2AAAA2, $AAAAAA2A
                LONG $00009000, $00000000, $20200000, $00202000, $00020002, $22202202, $20220202, $22222020
                LONG $12222222, $22222022, $22221222, $22222222, $21221222, $22212221, $22222222, $22222222
                LONG $22222222, $2A2AAA22, $22222222, $22222222, $A2222422, $22224222, $2A2222A2, $AAAAAAA2
                LONG $AA2A2AA2, $AA22A2A4, $AAA4AAAA, $4AA2A222, $AAAAAAAA, $AAAAAAAA, $AA2AAAAA, $A22222A2
                LONG $00000000, $00000000, $02000000, $00000200, $22000000, $00002000, $20200202, $22200220
                LONG $20222022, $22222222, $12212121, $21212222, $22222222, $23222222, $22222222, $2222A222
                LONG $222A2222, $AAAA2A2A, $22222AAA, $22222222, $A222A222, $A2AA2AA2, $AAAA2A2A, $2AAA2AA4
                LONG $AAA2AAAA, $22AAAAA2, $22222AAA, $222A42A4, $A4AAA422, $AAAAAAA2, $A4AAAAA4, $4A2AAAAA
                LONG $09000990, $00000000, $00000000, $02000020, $00200200, $22220222, $02022020, $20222002
                LONG $22220222, $22122122, $32232222, $22322322, $22223232, $32232232, $32AA23A2, $22232A3A
                LONG $A2A2AA2A, $A2A2A3AA, $A4AAAAAA, $2A2A24A2, $A42A2A24, $4AAAA2A2, $A2A24AAA, $A2A4AA2A
                LONG $AA2AAAAA, $2AAAAAAA, $AA2A2AA2, $A4A2A2A2, $AAAAA24A, $AAAA4A4A, $A2AAAA2A, $32A2A2A2
                LONG $00090000, $00000009, $00020000, $00000000, $00002002, $20020002, $02002020, $22002202
                LONG $20222220, $20210220, $22222200, $32232232, $22322122, $23223223, $3A322222, $2AA2A222
                LONG $AAAA3AA3, $A2AA2AAA, $22AA2A2A, $A2AAA2AA, $2AA42AAA, $2A2AA2A4, $AAAAA2AA, $AA22AA22
                LONG $AAA2AA2A, $22224AAA, $24AA2224, $AAA2A42A, $2224AAA2, $AAAA2A2A, $AAA22AAA, $AAA2A2AA
                LONG $90009002, $00000000, $20200000, $00202000, $20000202, $02002220, $20220020, $20220220
                LONG $02101200, $22022202, $12221222, $22122122, $22222223, $22222222, $23223222, $22222222
                LONG $22222222, $4AA24222, $AAA2A4A2, $A2A2AA2A, $AA2AA2A2, $A3AA4A2A, $AAA2AAAA, $2AAA2AAA
                LONG $42AA2A2A, $AAAA22A2, $A2AAAAAA, $A2AAA2A2, $4AAAA2A4, $A2AAAAAA, $2A4AA2AA, $A2A42AAA
                LONG $00000000, $20000000, $00000000, $20020002, $00020000, $02000200, $20200000, $10200200
                LONG $12022202, $12222022, $22202212, $23222222, $23322122, $12322332, $22322A22, $A22A23A2
                LONG $A22A2A2A, $A22A222A, $2A2AA2A2, $A2AA2A2A, $AAA3A2AA, $AAAAAAAA, $AAAAAAA2, $4A2AAAAA
                LONG $A2AAAAAA, $AAA2AAAA, $AA2AAA2A, $2AAA2A2A, $A2AAAAAA, $A242AA2A, $AAAA2AA2, $AA232A22
                LONG $00000090, $00000000, $00000000, $00200000, $20000000, $20000000, $02002002, $02020022
                LONG $21220220, $22122222, $22122122, $32233221, $AA2A2232, $23232AA3, $A32A232A, $AAA2AA2A
                LONG $2AA2A3A2, $2AA2AAAA, $42A2AA3A, $AAA2AAAA, $2AAAAAA2, $AAA2A2AA, $A2AA22A2, $AAAAAA2A
                LONG $AAAAAAAA, $A2AAAAAA, $3AAAA2AA, $AA2AAAAA, $AAAAAA2A, $AA2AAA4A, $2222AA3A, $AAA2A22A
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $02020220, $20221220
                LONG $22222222, $223A2222, $23232232, $23A22A32, $3333AA3A, $32A22232, $2A222A23, $2A2A22A2
                LONG $2AA2A2AA, $A2AAAA2A, $AAAA2A2A, $AAAAA2A2, $A2AAA2AA, $AAAAAA2A, $A4AAAA2A, $AA2AA2A2
                LONG $AA42AA2A, $A4A2A2AA, $A3A2AA2A, $2AA222A2, $A2A2AA2A, $2AAAAAA2, $AAAAAAAA, $A2AAAA2A
                LONG $90909090, $00000900, $00020000, $00000000, $02000000, $22002020, $20202000, $22222202
                LONG $22222222, $22222222, $23212122, $3223A23A, $22222222, $22232322, $A2222222, $A3A2AAA3
                LONG $A2A2AA22, $A2A2AAAA, $AAAAA2AA, $AAAA2A2A, $AAA3AAAA, $A2AAAAA2, $AAAA4AAA, $AAAAAAA4
                LONG $A2AAAAA2, $222A2A42, $2A222A2A, $A22AAAAA, $AAAA2AA2, $2A2AAAAA, $AAAAA222, $AAAAAAAA
                LONG $00000000, $00000009, $20000002, $02200200, $20002022, $20220220, $22222202, $22202022
                LONG $21221222, $222A2232, $22222222, $22232222, $22A22223, $23A2222A, $22A2AAAA, $22A2222A
                LONG $AAAA22AA, $AA2A2A24, $AA4AAA4A, $AA2A4A4A, $AAA2AA2A, $AAAAAAAA, $AAA2AAAA, $A4AAAAAA
                LONG $AAAA4AAA, $AAA4AAAA, $A2AAAAA2, $2AA42AAA, $AAAAAAAA, $A2AAAAAA, $2A2A22A4, $AA222AAA
                LONG $00090902, $90000000, $00000000, $20002020, $20220200, $20202002, $22220222, $12222222
                LONG $22212222, $22222221, $23223222, $22322122, $A323A2A1, $A32AA332, $A3222A32, $A42A22A2
                LONG $2AAAAAA2, $AAAA4242, $A42AA2AA, $2A4AAAAA, $AAAA4AAA, $4AAAA4AA, $A4AAAAAA, $AA2AA4AA
                LONG $A242AA4A, $AAAAAAAA, $A4AAA4AA, $A24AAAA4, $AAAAAAAA, $AAAAAAAA, $AAAAAAAA, $A2AAAAAA
                LONG $00000000, $20220222, $02202020, $22200002, $02020220, $02202220, $20220202, $22222220
                LONG $21222222, $02222222, $22222222, $22222222, $22222222, $3A232A2A, $AAA3222A, $2A22AA22
                LONG $A22A22AA, $A4AAAA3A, $AAA24A22, $A2AAAAA2, $4A4AA424, $AAA4AAAA, $4AAA4AAA, $AAA4AAAA
                LONG $4AA4AAAA, $A222A42A, $AAAA2AAA, $4AAAA4AA, $4AA24A22, $AAAA4AAA, $A4AAAAAA, $AAAAAA4A
                LONG $00009020, $22022002, $00002222, $00202202, $00202002, $22020002, $02002020, $00200222
                LONG $22220222, $22220022, $23222222, $322232A3, $A2A22222, $A2AAA2A2, $2222AAAA, $A2AAA2A2
                LONG $A22AAA2A, $22A2AA22, $A4AAAA4A, $AA4AA4A4, $AAAA4A24, $4AAAAAAA, $AA4A4A4A, $AAAAAAA4
                LONG $A2A4A2A4, $AA4AAAAA, $A4A4AAAA, $AAA4AAA4, $2A42AA4A, $2A42AAA2, $AAA4AAAA, $AAA4AAAA
                LONG $00092092, $00200020, $00000000, $20202020, $20020220, $20200200, $22220200, $22022020
                LONG $10122210, $22322220, $A222A2A2, $23A22A22, $3222A22A, $32A22222, $A2AA2222, $2AA2AA22
                LONG $AAA2AAA2, $2AAAAAAA, $AAAA4AA4, $AAAAAAAA, $A4A4AA4A, $AAAAAAA4, $24AAA4AA, $22424A42
                LONG $A4AAAAA2, $44AA4244, $AAA4AA4A, $AAAAAA4A, $AAAAAAAA, $AA2AAA2A, $AAAAAAA2, $AAAAAAAA
                LONG $00920000, $00022020, $20200000, $00220020, $20002002, $02022000, $11002020, $20201212
                LONG $21220220, $A2322221, $23A22222, $A223A222, $2AA22AA2, $2A2A22A2, $AA22A2A2, $A22AA24A
                LONG $2A2A3222, $AA2A22A3, $A22AA22A, $A4AA4A22, $2A42A44A, $AA4AAA4A, $4AAA4AAA, $4A4AA4AA
                LONG $AAA4A4AA, $AAA42AAA, $AAAAAAA2, $A4AAAAAA, $AAAAAAAA, $2A222AAA, $242222AA, $AA4A2222
                LONG $00000002, $00000000, $00000200, $20000200, $20200000, $20202220, $02220202, $22122000
                LONG $22212021, $22222222, $22222222, $22A22232, $A22A3222, $A3A2AA2A, $2AA2A2AA, $AAAAAAAA
                LONG $3AAA2AAA, $2AA2A3AA, $AAA2AA2A, $AAAAA2AA, $A4A4AAA4, $AAA4A4A4, $A4AAA4A4, $AAAA4A4A
                LONG $2AA2AAAA, $A2AA42A4, $4AAAAAAA, $2A2A2A2A, $AAAAAA2A, $AAAAA4AA, $AAAAAA24, $AAAAAA4A
                LONG $92906006, $00060060, $02000000, $00002000, $20200200, $02020020, $20202020, $12221222
                LONG $12222222, $22222222, $A22A2232, $A2232A23, $22A2A2A3, $AAA4AAA2, $AAAAAAA2, $A4AA4AAA
                LONG $A2AAAA4A, $A2A2AA2A, $322A32A3, $AAAA3A23, $AAAA44AA, $4AAAAAAA, $4AA4A4AA, $24A4AA4A
                LONG $AA4A4244, $2242A42A, $AAAA44A4, $AAAAAAAA, $2422AA2A, $AAA4AAA2, $A4A4AAAA, $AA4A4AA4
                LONG $00009000, $00000000, $00000000, $00200200, $20202002, $22020220, $22222220, $22022212
                LONG $22222021, $22222222, $22222222, $2A222222, $AA33A23A, $2A22AA2A, $AA4AAAAA, $A2AAAA2A
                LONG $AA24AA2A, $2AAA2AA2, $23A22A2A, $A24AA2AA, $A4AAAAAA, $AAAA4AAA, $4AA4AAA4, $AA2A2422
                LONG $42222222, $AA2AAAAA, $2424AA22, $AAAA4A4A, $AAAAAAAA, $A4A4AAA4, $4AA4AA4A, $A4A44A44
                LONG $06000002, $09009060, $20020000, $22002020, $02020020, $20202002, $22222022, $02202222
                LONG $22222220, $22222222, $222A2222, $22322222, $2AAA2A22, $A2AAA2AA, $AAAA2222, $4A2A4A24
                LONG $44A2A4AA, $AA2A4A4A, $AA2AAAA2, $AAA2A323, $4AA2AAA4, $AA4AAA4A, $AAAAAAAA, $424AA2AA
                LONG $2242A4AA, $24224222, $A224AAA4, $24A4AAA2, $AAA4A44A, $AAAAAAAA, $AA4AA4AA, $AAAAA4AA
                LONG $00006006, $00200000, $22200006, $22222022, $20222222, $22222222, $22222222, $32122222
                LONG $22232222, $2A22A222, $A32A2AAA, $2A22A3A2, $22A2A2A3, $AA2A322A, $22AAAAA2, $AAA2AAAA
                LONG $AA4AAAA2, $A4AAAAAA, $3AAAAAAA, $A4AAAAAA, $AAAAAA2A, $AAAA4AA2, $AAAAAAAA, $AAAA4AAA
                LONG $AAA4AAAA, $22AAA2A4, $A422222A, $A2A2A24A, $4AA2AAAA, $AAAAA4A2, $A4AAAAAA, $4A4AAAAA


Sprite_Size     BYTE $20
Sprite_Data     WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020



                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
}


{
' Girl Image
Palette_Data    LONG $1D044B03, $029A4AAA, $BDCC9C9B, $020207AE

Image_Data      LONG $00000000, $00000000, $10000000, $10110100, $11111101, $11111111, $11111111, $11111111
                LONG $11111111, $11111111, $11111111, $11111111, $21121121, $21121211, $22221221, $22222222
                LONG $22222222, $22222222, $22222222, $22222222, $22222322, $23232323, $23232323, $33333333
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $01100000, $01010101, $00101011, $01010101, $01010101, $11111011
                LONG $11111111, $21111111, $12121211, $12121212, $11211212, $21212112, $21212211, $22122121
                LONG $22222222, $22222222, $22222222, $32323222, $32323222, $32322222, $32323232, $33332322
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $01000000, $11101101, $11011101, $10111011, $11111111, $11111110
                LONG $11111111, $11211211, $11111111, $21112111, $12112111, $12121212, $22122122, $12221222
                LONG $22212212, $22222222, $22222222, $22222222, $22232232, $23233232, $23232323, $33333233
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $10110000, $01010110, $01100110, $11011101, $11101010, $01101111
                LONG $11111111, $11111111, $12121212, $12111212, $11211211, $12112121, $12212212, $22222212
                LONG $21222222, $22222212, $22222222, $23232322, $23223223, $22322323, $32332323, $23323323
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $01000100, $10111101, $11011101, $01110110, $01111111, $11111111
                LONG $21111111, $11112111, $11111111, $11221121, $12121122, $21212112, $21222121, $22121222
                LONG $22222222, $22212222, $22222222, $32222223, $22322232, $33223222, $23323232, $33332333
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $10110010, $01101001, $10111011, $11101101, $11011101, $11111101
                LONG $11111111, $21211111, $12121211, $12111212, $21211211, $21212121, $12221221, $12212212
                LONG $22222212, $22222222, $22222222, $22232322, $23232322, $32232323, $33223232, $33333332
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $10000000, $01001010, $11010110, $01010110, $01011011, $11110111, $11110111
                LONG $11111111, $11111211, $11111111, $21121111, $11121121, $12121212, $22122112, $22112121
                LONG $12222222, $22222222, $22222222, $32322222, $23222232, $23323232, $22332323, $23232323
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $01000000, $10100100, $01101011, $11101011, $11101101, $10111011, $11111110
                LONG $11111111, $12111121, $21212121, $12112121, $22112112, $21212111, $12212221, $21211112
                LONG $22121221, $22222212, $22222222, $22222232, $22323222, $22232223, $33232323, $33333232
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $01111001, $10110110, $01011101, $10110111, $11101110, $11110111
                LONG $11111111, $11121111, $11111112, $11121112, $11121121, $12121122, $11122122, $12222141
                LONG $12222212, $22212122, $32222222, $32323222, $23222323, $23323232, $23232323, $32332333
                LONG $33333323, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $01000000, $11000010, $01101100, $10111011, $11111010, $11111101, $11111111
                LONG $21111111, $11111111, $12121211, $21212121, $12211211, $22121211, $01215551, $00555006
                LONG $21111110, $21121211, $22222222, $22222222, $22323232, $32232322, $32323232, $23323322
                LONG $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $00111101, $10101011, $11010110, $01010111, $01011011, $11111101
                LONG $11111111, $21211211, $21111111, $12111211, $21121121, $51111212, $75557777, $77755105
                LONG $11840456, $11111111, $22222211, $32322232, $32232222, $23222232, $32323223, $33232323
                LONG $33333323, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $10000101, $11011101, $10101011, $11111010, $11111111, $11111111
                LONG $11111111, $11111112, $12121211, $11211121, $11112112, $77500000, $77777777, $77776557
                LONG $66060057, $11000066, $22222111, $22222222, $22223232, $32323223, $32322322, $23332332
                LONG $33323332, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $10000000, $10100000, $01111000, $01101011, $11111110, $10101101, $11101010, $11111111
                LONG $11111111, $21121211, $11111121, $12122112, $75500011, $77777777, $77777777, $66577777
                LONG $57776666, $65577755, $21111100, $32222222, $32322222, $23222322, $23233232, $32323223
                LONG $23333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00000000, $11000011, $11011010, $01010101, $11111011, $10111111, $11111111
                LONG $12111111, $11111111, $21212111, $50111121, $77775555, $77777777, $77777777, $77777777
                LONG $77566655, $77775777, $55550657, $22322221, $22223232, $32223223, $32322322, $23232332
                LONG $33332323, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $11000000, $01011100, $10110101, $11101110, $01011110, $11110101, $11111011
                LONG $11111111, $12121112, $11111121, $55555511, $77777777, $77777777, $77777777, $77777777
                LONG $76557777, $77777777, $77757777, $32222115, $23232222, $22322232, $32323232, $32323232
                LONG $23233332, $33333323, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $01010000, $10100010, $11011010, $01010101, $11101011, $01111111, $11111111
                LONG $11111111, $11111211, $22121111, $77777751, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $57777777, $22222110, $32222232, $23232222, $23232223, $33232323
                LONG $33323232, $33232333, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $10000000, $00000100, $01101000, $00110111, $11111011, $10111101, $11111010, $11111111
                LONG $11111111, $11211111, $51112112, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77755777, $77777777, $22221105, $22323222, $32223232, $32323232, $22323232
                LONG $32332323, $33333232, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $10000001, $10010010, $11101100, $01010110, $11101011, $10101111, $11111111
                LONG $11111111, $11112112, $77511211, $77777777, $77777777, $57577777, $77777755, $77777777
                LONG $77777777, $77777777, $77777777, $22222105, $23222223, $23232222, $23222322, $33232323
                LONG $23232332, $32332323, $33333333, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $10100000, $00100100, $10101011, $11101101, $10111110, $11111010, $11111101
                LONG $11111111, $12111111, $77751112, $77777777, $77777777, $65557777, $77555756, $77777777
                LONG $77777777, $77777777, $77777777, $22221577, $22222322, $32222323, $22332232, $23232323
                LONG $33233223, $33323232, $33333323, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $10000000, $00000010, $11010001, $11011010, $01110110, $11010111, $11111111, $11111111
                LONG $11111111, $11121121, $77775111, $77777777, $77775777, $55577777, $55567756, $77777777
                LONG $77777777, $77777777, $77777777, $22205777, $22222222, $23232222, $23223222, $23232323
                LONG $23223323, $32332323, $33323332, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $10000000, $01010100, $01101101, $11011011, $01111101, $11101011, $11111111
                LONG $11111111, $21111111, $77777511, $77777777, $77777577, $77777777, $67775665, $77777775
                LONG $77777777, $77777777, $77777777, $05577777, $32223211, $22222232, $32232323, $32323232
                LONG $32332232, $23223232, $32332323, $33333333, $33333333, $33333333, $33333333, $33333333
                LONG $00000000, $00100001, $10101010, $11011010, $10110110, $11101110, $10111111, $11111110
                LONG $11111111, $11121211, $77777751, $77777777, $77777777, $77777777, $77756555, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $22221555, $23232222, $22322222, $23223223
                LONG $32322323, $32332323, $33323232, $33233332, $33232333, $33333333, $33333333, $33333333
                LONG $00000000, $10000010, $01010100, $01101011, $11011011, $10111011, $11111110, $11111111
                LONG $11111111, $12111112, $77777775, $77777777, $57777777, $77777775, $77565577, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $32157777, $22222222, $23232323, $32323232
                LONG $23233232, $23223232, $22232323, $32332233, $32333232, $33232332, $33333333, $33333333
                LONG $00000000, $00010000, $10101001, $10110101, $01101101, $11011101, $11101111, $11111111
                LONG $11111111, $11121111, $77777775, $77777577, $67777777, $75575756, $75557777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $20777777, $23222323, $22222222, $22322323
                LONG $32222323, $32323232, $33323232, $23223323, $33232333, $33323233, $33333333, $33333333
                LONG $00000000, $01000001, $01010010, $10101110, $10110110, $01110111, $01111101, $11111111
                LONG $11111111, $01111111, $77777756, $75555577, $57775557, $56565566, $55577777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $07777777, $22322222, $23232323, $23223222
                LONG $23323222, $23232323, $22232323, $32332232, $32323232, $23233322, $23232323, $33333323
                LONG $00000000, $10000000, $11010100, $11011001, $01101101, $11011101, $11110111, $11111111
                LONG $11111111, $51121121, $77777755, $55665577, $77756577, $65765565, $55777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $57777777, $22222232, $32222222, $32232232
                LONG $22232323, $32322222, $33323232, $32323232, $23232323, $33232233, $33233232, $33323223
                LONG $00000000, $00101000, $01001001, $01101110, $11011011, $11110111, $11101111, $11111111
                LONG $11111111, $60111111, $77777755, $66055777, $77757775, $55777557, $55657777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $57777777, $22222231, $22322322, $22323223
                LONG $23222232, $32232323, $22232322, $23232323, $32323232, $22323322, $23232323, $32332332
                LONG $00000000, $10000010, $10110100, $10110101, $10110101, $10111010, $11111010, $11111110
                LONG $11111111, $54112111, $77777775, $56555557, $77777775, $77777777, $55566577, $77777756
                LONG $77777777, $77777777, $77777777, $77777777, $57777777, $22223232, $32222222, $22222322
                LONG $32323223, $23223222, $33232232, $23232232, $32323233, $32322332, $23223232, $23232233
                LONG $00000000, $00010001, $10101010, $01010110, $11101111, $11101111, $11111111, $11111111
                LONG $11111111, $50111111, $77777775, $55555555, $77777777, $77777777, $55777657, $77757777
                LONG $77777777, $77777777, $77777777, $77777777, $57777777, $22222222, $22232222, $23232232
                LONG $22222322, $22322232, $22323223, $32232323, $22232222, $23223223, $32332323, $32323322
                LONG $00000000, $01000100, $11010101, $11111101, $01011010, $10111101, $11011010, $11111101
                LONG $11111111, $60112111, $77777775, $77775565, $77777777, $77777777, $55777777, $77777555
                LONG $77777777, $77777777, $77777777, $77777777, $07777777, $22222222, $32222222, $32222222
                LONG $32323222, $23222322, $23222232, $22323222, $23322323, $32232232, $23223222, $23232223
                LONG $00000000, $00010000, $10101001, $01001010, $11101101, $11101011, $11110111, $11111111
                LONG $11111111, $55011111, $55777777, $77776557, $77777777, $77777777, $55777777, $57777776
                LONG $55777775, $77777555, $77777777, $77777777, $77777777, $22222220, $22222222, $22322322
                LONG $22222232, $32232223, $22323222, $23222323, $23223232, $22323223, $22232232, $22322322
                LONG $10001000, $01000010, $01010100, $10110111, $01110111, $01111101, $10111101, $11110111
                LONG $11111111, $56011111, $57777775, $77775575, $77777777, $77777777, $77777777, $77577777
                LONG $77555777, $77777777, $77777777, $77777777, $77777777, $22222215, $22222222, $32222222
                LONG $23232222, $22223222, $32222322, $22322222, $32322223, $22222322, $32322323, $32232232
                LONG $00000000, $00001000, $10101010, $01101101, $11011101, $10110111, $11011111, $11111111
                LONG $11111111, $55611111, $57755775, $77777775, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $22222215, $22222222, $22232222
                LONG $22222222, $32322232, $22232232, $32223232, $22232322, $32323223, $22232222, $22322322
                LONG $00000000, $10000100, $01010000, $11010101, $10101010, $11101110, $11111010, $11111110
                LONG $11111111, $55501111, $77555777, $77777556, $77775777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $22222221, $22222222, $22222222
                LONG $32222232, $22223222, $32322222, $22232222, $22322223, $22222232, $23222323, $23223223
                LONG $00000000, $00000000, $10101010, $01101110, $11110111, $10110101, $11111111, $10111111
                LONG $11111111, $56510111, $77757555, $77777557, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $07777777, $22222221, $22222222, $22222222
                LONG $22222222, $22222222, $22223223, $23222322, $32223232, $23232222, $22232222, $22232232
                LONG $00000000, $00001000, $01000100, $10110011, $10111010, $11011110, $10101010, $11101111
                LONG $11111111, $55511111, $77755655, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $10777777, $22222222, $22222222, $22222222
                LONG $22222222, $23222222, $32322222, $22322232, $23222222, $22223232, $32223232, $23223222
                LONG $00000000, $10000100, $01010000, $01101101, $11010111, $11101011, $11111111, $11111110
                LONG $11111111, $75500111, $77755565, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $21577777, $12212222, $22222222, $22222222
                LONG $22222222, $22232222, $22222222, $22232222, $22232323, $32322222, $22322222, $22322232
                LONG $00000000, $00000000, $10101010, $11011010, $10111010, $10111110, $11011010, $10111011
                LONG $11111111, $55600111, $77766557, $77777777, $77777777, $77777777, $77777577, $77777777
                LONG $77777777, $77777777, $77777777, $05777777, $22157775, $22222122, $22221222, $22222222
                LONG $22222222, $22222222, $22222222, $22222322, $32222222, $22222232, $23222323, $22223222
                LONG $00000000, $01000000, $10010000, $01010110, $11010111, $11101011, $01110111, $11101111
                LONG $11111110, $57501111, $77565577, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $75777777, $22107777, $22222221, $12222212, $22222222
                LONG $22222222, $22222222, $22222222, $23222222, $22222222, $22323222, $22222222, $22322223
                LONG $00000000, $00000000, $10101010, $11101010, $01101010, $10111101, $10111010, $10111111
                LONG $11111111, $75501111, $75655557, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $22221577, $12122222, $22122222, $22222222
                LONG $22222222, $22222222, $22222222, $22222222, $22222222, $22222222, $22222232, $22222222
                LONG $00000000, $10000000, $01010100, $01011101, $10111111, $11010111, $11101111, $11111101
                LONG $11110110, $77511111, $75755575, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $12222207, $22222122, $22222222, $12221212
                LONG $22222222, $22222222, $22222222, $22222222, $22222222, $23222222, $22222223, $22222222
                LONG $00000000, $00100000, $11010101, $10110010, $01010100, $10111101, $01011010, $10101011
                LONG $11111111, $55011111, $77565755, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $22122207, $22221222, $12212121, $22122222
                LONG $12212122, $22222222, $22222222, $22222222, $22222222, $22222222, $22222222, $22222222
                LONG $01000000, $11001000, $10101010, $11011110, $11101011, $11101011, $11110111, $11111110
                LONG $10111101, $50111111, $75555557, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $12222215, $21212212, $22222222, $22222222
                LONG $22222221, $21212122, $22222212, $22222222, $22222222, $22222222, $22222222, $22222222
                LONG $00100000, $00100010, $11010101, $01100010, $01011101, $01111101, $10111101, $01101111
                LONG $11101111, $61111111, $55556555, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $22121220, $22222222, $21221212, $22212121
                LONG $22122222, $22222221, $21221222, $22122212, $22212222, $22222222, $22222222, $22222222
                LONG $10000000, $10000000, $00111010, $11011101, $10110110, $10101011, $11101011, $10111010
                LONG $11111101, $01111111, $77755555, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $12222207, $12121212, $22122222, $21222222
                LONG $12221212, $12212222, $22222222, $12221222, $21222212, $12212121, $12122222, $21221212
                LONG $00000000, $01000000, $11100010, $10101010, $01101101, $11011101, $11011110, $11110111
                LONG $11110111, $11111111, $77777750, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $22122157, $22222222, $12222121, $12221222
                LONG $22122222, $22122212, $12212212, $22222222, $22221222, $22122222, $22222212, $22222222
                LONG $10100000, $10100010, $00010100, $10110101, $11010110, $10110110, $01110101, $01011101
                LONG $10111111, $11111110, $77777775, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777557, $77777777, $77777777, $77777777, $12222157, $21212121, $22121222, $22122212
                LONG $12221221, $21221222, $22122122, $21212212, $21212221, $12221221, $12121222, $21212122
                LONG $00000100, $00000000, $10001000, $01001010, $01101011, $11011011, $10101110, $11101111
                LONG $11111011, $51111111, $77777777, $77777777, $77777777, $77777777, $77565657, $57777777
                LONG $77775655, $77777777, $77777777, $77777777, $21212157, $12121212, $22212212, $22212221
                LONG $22122122, $22222122, $22222221, $22221221, $22222122, $22122122, $22222122, $22222212
                LONG $00000000, $00000000, $01110101, $10110101, $10110110, $01101101, $01111011, $10110101
                LONG $11011101, $01111111, $77777777, $77777777, $77777777, $57777777, $56664186, $66575775
                LONG $77777666, $77777777, $77777777, $77777777, $12122157, $21212121, $21122121, $21221212
                LONG $12212221, $21212212, $21212122, $12122222, $21212212, $12222222, $21212212, $12121221
                LONG $00000000, $00000000, $10000000, $01101010, $01101101, $11011011, $11010110, $11111111
                LONG $11110111, $11111101, $77777775, $77777777, $77777777, $46577777, $66556640, $66656755
                LONG $77756666, $77777777, $77777777, $77777777, $12121077, $12121212, $11221212, $12212212
                LONG $22122212, $12222122, $12222222, $22221212, $12222222, $21211212, $22222122, $22222212
                LONG $00000000, $00000000, $01010000, $01010101, $10110101, $10101010, $10111011, $01101010
                LONG $01111101, $01110111, $77777775, $77777777, $77777777, $44657777, $77567644, $67756767
                LONG $76666666, $77777777, $77777777, $77777777, $21222077, $21212121, $22112121, $21212121
                LONG $12212121, $21212212, $22121212, $12122222, $21211212, $12122122, $21121221, $12121221
                LONG $00000000, $00000000, $00000000, $10100001, $11010110, $01110101, $11010101, $10111111
                LONG $11010111, $01111101, $77777775, $77777777, $77777777, $88644667, $66676648, $77766666
                LONG $66666666, $77777777, $77777577, $77777777, $12111577, $12121212, $11221212, $12121212
                LONG $21212122, $21211221, $21212121, $22212121, $21222121, $21211211, $22121122, $21212121
                LONG $00000000, $00000000, $01001000, $01011100, $10101001, $10101010, $01101111, $11010101
                LONG $01111010, $01101111, $77777775, $77777777, $67777777, $98844884, $64676898, $77666666
                LONG $66646666, $77777766, $77777777, $77777777, $11221577, $11211111, $12111121, $21212121
                LONG $12121211, $12122112, $12221212, $11221222, $12111212, $12112112, $11212111, $11212112
                LONG $00000000, $00000000, $10000000, $10100000, $10110110, $01011101, $11010101, $10111110
                LONG $11101111, $11111101, $77777770, $77777777, $86657777, $A9998888, $8866499A, $64484448
                LONG $64888466, $75666666, $77777777, $77777777, $12115777, $12112122, $21121212, $11121111
                LONG $11211212, $12121121, $22112121, $12211211, $21122112, $21121121, $12111121, $12121121
                LONG $00000000, $00000000, $00000000, $01010010, $01010101, $11101011, $01111110, $11101011
                LONG $01011010, $11111111, $77777751, $77777777, $99846777, $AAAABA99, $A9469ABB, $48999999
                LONG $88889884, $64444448, $77755776, $77777777, $21207777, $21121211, $12121111, $21212122
                LONG $21121121, $21112121, $11212121, $11122121, $11211212, $12112112, $21121211, $11111211
                LONG $00000000, $00000000, $00000000, $10001001, $11101010, $01010101, $10101011, $01011101
                LONG $11101111, $11011011, $77777750, $77777777, $99988657, $BBBBBBA8, $A869BBBB, $99AA999A
                LONG $9A99A998, $48998889, $77557666, $77777777, $12157777, $12111121, $11212121, $12111211
                LONG $12112112, $12211212, $21121112, $22111212, $21121121, $21211211, $12111121, $12121121
                LONG $00000000, $00000000, $00000000, $00100100, $10010101, $10111110, $11011010, $11110110
                LONG $10110101, $11101110, $77777750, $57777557, $AAA98488, $BBCCCCCB, $949CCCCC, $BBA989AC
                LONG $AAAAA999, $88999999, $55577544, $77777775, $21077777, $21122111, $12111211, $11212112
                LONG $12121121, $11112111, $11212121, $11212111, $12112111, $11111121, $11212111, $11112112
                LONG $00000000, $00000000, $00000000, $01001001, $10110101, $11100010, $10110101, $10101101
                LONG $11111110, $11111101, $77777750, $96777777, $ABBA4489, $CCCCCCCB, $89CCBCCC, $BA99ACBA
                LONG $9ABBAABB, $8999999A, $75576648, $77777777, $21077777, $11211121, $11212112, $12121121
                LONG $21211211, $12121121, $12111112, $21121122, $11111121, $21212111, $11111121, $11211111
                LONG $00000000, $00000000, $00000000, $01010000, $01000100, $00111101, $01101111, $11011011
                LONG $01010101, $11011011, $75777751, $9A877775, $CCBA9889, $CCCCCCCB, $9CCCCCBB, $AABCBBA9
                LONG $BA999AA9, $9999AAAB, $75766648, $77777777, $11577777, $12112112, $21111211, $21111211
                LONG $11121121, $11211211, $11212121, $12111211, $21212111, $11111121, $12121211, $12111212
                LONG $00000000, $00000000, $00000000, $00000000, $10110101, $11001011, $10110101, $01101101
                LONG $11111011, $11111111, $75777750, $99A85777, $CCCBA889, $CDCCCCCC, $BCCCCBCC, $BBBBAAAA
                LONG $AABBBABB, $899999A9, $67656448, $77777775, $21577777, $11211121, $12121121, $12121121
                LONG $21112111, $21121121, $12111211, $11212112, $11111212, $11212111, $11111112, $11121111
                LONG $00000000, $00000000, $00000000, $00100000, $01001000, $10110101, $01011010, $10111011
                LONG $10101101, $11101010, $77577751, $AAAA9577, $CCCCB989, $CCCCCCCC, $BCCCCCCC, $BBBBBCAB
                LONG $ABABBBBB, $9999AAAA, $76644488, $77777576, $11577777, $21121111, $11112111, $11111112
                LONG $11211212, $12112111, $11121121, $21111121, $12121111, $21111112, $12112111, $11111211
                LONG $00000000, $00000000, $00000000, $10000100, $01010010, $01101101, $10110101, $01010110
                LONG $11011111, $10111111, $77557750, $9AAAA967, $CCCCBA99, $CCCCCCCC, $CCCCCCCC, $BBBCBBBC
                LONG $BABBBBBB, $99AAAAAA, $66488889, $77775567, $21577777, $11111211, $11211112, $12121211
                LONG $12111111, $11111112, $11211111, $11212111, $11111121, $11111211, $11111121, $11211112
                LONG $00000000, $00000000, $00000000, $00000000, $01010100, $10010101, $11001110, $11101011
                LONG $10110101, $11101010, $77757751, $AABAA985, $CCCBBBA9, $CCCCCCCC, $CCCCCCDC, $CCCBCCCC
                LONG $ABBBBBBB, $9AAAAAAB, $64488899, $77766666, $10777777, $12121112, $12112121, $11111112
                LONG $11121212, $21212121, $50112121, $21111055, $12112111, $12121111, $11212111, $11111111
                LONG $00000000, $00000000, $00000000, $00000000, $00100000, $11010101, $01110010, $01111101
                LONG $11101110, $11111101, $77757501, $AABBA996, $CCCCCBAA, $CDCCCCCC, $CCDCCCCD, $BBCCCCCC
                LONG $BBBBBBCC, $AAAAABBB, $48888989, $77677664, $20577777, $11111211, $11111111, $12121211
                LONG $12111111, $11111111, $50111111, $11211555, $11111121, $11111112, $11111111, $11111111
                LONG $00005000, $00000000, $00000000, $00000000, $11010001, $00101010, $10101101, $10101010
                LONG $10111011, $11010110, $57775501, $BBBBBA98, $CCCCBCBB, $CCCDCDCC, $CCCDCDCC, $CCCCCCCC
                LONG $BBBBBCBB, $AAABABAB, $88889999, $77766644, $11577777, $21121111, $11212121, $11111111
                LONG $11112112, $12111111, $55501121, $11110775, $11211111, $11111111, $11111111, $11111111
                LONG $00000000, $00000000, $00000000, $00000000, $00100100, $11011101, $11011011, $01010101
                LONG $11010101, $10101101, $67777551, $BBBBBA98, $CCCCCCCB, $CDCCCCCC, $CCCCCCCD, $BCCCCCCC
                LONG $BCBCCBCC, $AAABBBBB, $888999AA, $77666444, $11577777, $11111111, $21111111, $11111111
                LONG $11111111, $11111111, $55555111, $12117775, $11111111, $11111111, $11111111, $11111111
                LONG $00000000, $00000000, $00000000, $00000000, $11010000, $01100101, $10101100, $11111010
                LONG $01101110, $11111011, $57777750, $BBBBBBA8, $CCCCCCBC, $CCCDCCDC, $CDCDCCDC, $CCCCCCCC
                LONG $CBCBCBCB, $ABBBBBBC, $888999AA, $76664448, $21577777, $12111121, $11111111, $11211121
                LONG $11121111, $11111111, $75555501, $11157777, $11111111, $11111111, $11111111, $10110111
                LONG $05000000, $00000000, $00000000, $00100000, $01000100, $00110100, $01010111, $01001111
                LONG $10110101, $11010101, $65777750, $CBBBBAA9, $DCCCCCCC, $DCCCDCCC, $CCCCDCCC, $CCCCCCCC
                LONG $CCBCBCBC, $AAABBBCB, $66644889, $76644446, $11577777, $11111111, $11111111, $11111111
                LONG $12101111, $11111111, $77557701, $11077777, $11111111, $11111111, $11111111, $11011010
                LONG $00000000, $00000000, $00000000, $00000000, $00100010, $11000011, $10101000, $10110010
                LONG $11011011, $11111110, $47777755, $CCCBBBB9, $BBBBBBCB, $CCDCCCCB, $CCCCCCDC, $BCCCCCCC
                LONG $BBBBCBCC, $08199AAB, $65555606, $76446666, $10777777, $11111111, $11111111, $11111111
                LONG $15555111, $11111111, $77777750, $10577777, $11111111, $01010111, $01010101, $01101111
                LONG $00050000, $00000000, $00000000, $00000000, $00000000, $00011100, $01111011, $01101101
                LONG $10110110, $11010101, $07777755, $BCBBBBB9, $918199AA, $BBAAA929, $CCCCCCBB, $CCCCCCCC
                LONG $9AABBBBB, $65666048, $48888884, $56466666, $15777777, $11111111, $11111111, $11111111
                LONG $77755011, $11111105, $77777750, $15777777, $10111111, $11111110, $11111111, $11011010
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $01000001, $10000100, $10010110
                LONG $01011011, $11101101, $07777775, $9ABBBBBA, $99992919, $64040489, $CBBAA988, $BCCCCCCC
                LONG $4499ABBB, $98440466, $81899999, $66466641, $15777777, $11111111, $11111111, $11111111
                LONG $77775011, $11105777, $77777750, $05777777, $11111111, $10101011, $10101010, $10110111
                LONG $00000000, $00000000, $00000000, $00000000, $01000000, $00100000, $01110101, $11101011
                LONG $10101000, $10110110, $65777775, $A99BBCB9, $BBCCBBBA, $88999AAA, $BAA99998, $BBCCCCCC
                LONG $488999AB, $66444448, $55575556, $64446665, $10777777, $11111111, $11111111, $11111111
                LONG $77750111, $15577777, $77775551, $07777777, $01010111, $11111101, $11011111, $01011010
                LONG $05000500, $00000500, $00000000, $00000000, $00000000, $10001000, $01001000, $10010101
                LONG $01010111, $11010101, $67777755, $BBABBCC9, $92AAABBB, $99888889, $BBAAAA99, $BBCCCCCB
                LONG $888889AA, $77755564, $77777777, $66446577, $15777777, $11111111, $11111111, $11111111
                LONG $77750111, $77777777, $77777555, $57777777, $11111011, $10101011, $01101010, $10100101
                LONG $00050000, $00000000, $00000000, $00000000, $00000000, $00000000, $00010010, $00100000
                LONG $10101000, $11101010, $57777775, $9ABBCCB9, $55550119, $55555555, $BBAA9980, $ABCCCCCC
                LONG $5484888A, $55005555, $77775555, $64066577, $10755775, $11111111, $11111111, $10101101
                LONG $77501111, $77777777, $77777777, $57777777, $01010111, $01011101, $01010101, $00001011
                LONG $50000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $10001001
                LONG $01010010, $10110101, $46657750, $89BCCCCA, $55777555, $65555555, $BBAA8055, $ABCCCCCC
                LONG $56444489, $75123B26, $77541100, $44446657, $10046776, $11011011, $01010110, $11111011
                LONG $75111110, $77777777, $77777777, $57777777, $10111110, $11101011, $10101111, $00100100
                LONG $00000500, $05000050, $00000000, $00000000, $00000000, $00000000, $00000000, $01000100
                LONG $00101000, $10010101, $99AA6750, $8ABCBCBA, $22457756, $39155111, $BBA98489, $ABCCDCCC
                LONG $86484488, $1013CC39, $77619221, $44884446, $11466776, $11101111, $11111011, $10111111
                LONG $11005011, $77777775, $77777777, $57777777, $01010110, $01010101, $00010000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00010000
                LONG $10000101, $11101010, $9ABCB050, $ABCCCCCB, $CC396549, $B3900023, $CBBA999A, $ABCCCDCC
                LONG $A9999889, $93B3C3BA, $46754192, $48488844, $10466566, $10111010, $10101110, $11101010
                LONG $55575501, $77775005, $77777777, $57777777, $01101110, $01010101, $01001011, $00000000
                LONG $00000500, $00500000, $00000005, $00000000, $00000000, $00000000, $00000000, $00100001
                LONG $01010000, $01001010, $AABCCB10, $CCCCCCCB, $CC3819AB, $CCC3333C, $CCCBBBAB, $ABCDCCCC
                LONG $3A298999, $08499992, $88466666, $44881989, $10666666, $11110111, $11111011, $00111111
                LONG $55555500, $77755577, $77777777, $77777777, $10011010, $10100010, $00000000, $00000000
                LONG $00500000, $00005005, $00000500, $00000000, $00000000, $00000000, $00000000, $10000000
                LONG $10000100, $01010100, $A9ABCC31, $CCCCCCCB, $2188ABCC, $A32AA3B3, $DCCBB3A2, $ABCCCCDC
                LONG $AA992999, $999AAAAA, $99999999, $64889999, $10466666, $10101101, $10101110, $55011010
                LONG $55555775, $77777755, $77777777, $57777777, $10100110, $00000100, $00000000, $00000000
                LONG $00000500, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00100100
                LONG $00000000, $00101010, $ABAABCA1, $CCCCCCBB, $AABCCCCC, $99999999, $CCCCCBBA, $BBCCCCCC
                LONG $3A3A9999, $AAB3AABA, $99AAAAAA, $44889999, $10066666, $11110110, $11110101, $77501111
                LONG $57557775, $77777777, $77777777, $77777777, $01011010, $01010010, $00000000, $00000000
                LONG $50000000, $50000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $01000000, $01010000, $CCBBBB20, $CCCCCCBB, $CCCCCCCC, $BCBBBBBC, $CCDCCCBC, $BBCCDCCC
                LONG $AA999999, $BBABBB3A, $AAA3AA3B, $648889AA, $01066666, $01011011, $01011111, $77750101
                LONG $77555777, $77777757, $77777777, $77777777, $00000110, $00001001, $00000000, $00000000
                LONG $00050050, $00050500, $00000500, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $10000000, $10001010, $CCBBCB10, $CCCCCCBC, $CCDCCDCD, $CCCCCCCC, $CDCCDCCC, $BBCCCCCD
                LONG $9392989A, $ABBBB3AA, $AABABBBB, $6448999A, $11066466, $10110110, $10110010, $77775011
                LONG $55057777, $77775777, $77777777, $57777777, $01000100, $00000100, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00100000, $CCCCCA10, $CCCCCBBB, $DCCCDCCC, $CCCCCCDC, $CCCCCCDC, $ACCCCCCC
                LONG $2999999A, $BBB3AAA9, $ABABBBBB, $6648899A, $00066666, $11101101, $01101101, $77777501
                LONG $01157777, $77777755, $77777777, $57777777, $00001010, $00000000, $00000000, $00000000
                LONG $05005000, $50000050, $50500000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $10000000, $BCCCC200, $CCCCBBBB, $CCDCCCCC, $CCCCDCCC, $CDCCDCCC, $BBCCCCCC
                LONG $9999899A, $BBBBAA39, $A3BBBBBB, $66488999, $11106666, $10110100, $10110110, $77777551
                LONG $11057777, $77500011, $77777777, $05777777, $00000001, $00000000, $00000000, $00000000
                LONG $00000050, $00050000, $00000500, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00010000, $CCCCB100, $CCCBBBBC, $CCCCCDCC, $DCDCCCCD, $CCCCCCDC, $BCCCDCCC
                LONG $9988899A, $BBAAAAA9, $AAAAABBB, $66648999, $01006666, $01001010, $11001101, $77777750
                LONG $01057777, $77551100, $77777777, $05777777, $00000000, $00000000, $00000000, $05000000
                LONG $05005000, $05000500, $00000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $CCCB1000, $CCBBBBBC, $CCCCCCCC, $CCCCCDCC, $CCCCCCCC, $CCCCCCCC
                LONG $888889AB, $A3AB3AA9, $9AAAABAB, $66644899, $10005566, $10110100, $01010010, $77777775
                LONG $00157777, $77750101, $77777777, $10577777, $00000000, $00000000, $00000000, $50000000
                LONG $00000000, $50000000, $05000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $CCB20000, $CBBBABCC, $DCCCCCCC, $CDCDCCCC, $CCCCCCCD, $BCCCCCCC
                LONG $889899AB, $AABAAA99, $99AAAAAA, $66664899, $00106655, $01000010, $00101001, $77777775
                LONG $00105777, $77775510, $77777777, $00577777, $00000000, $00000000, $00000000, $00050000
                LONG $00505050, $00005005, $00000000, $00000050, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $CA100000, $CBBBBBCC, $CCCCCCCB, $CCCCCCDC, $CCCCCCCC, $BCCDCCCC
                LONG $988889AB, $BAAAAA98, $999AAAAA, $66664489, $00000065, $10101001, $50101100, $77777777
                LONG $10015777, $77757550, $77777777, $00057777, $00000000, $00000000, $00000000, $05000050
                LONG $00000000, $00500000, $00000000, $05000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $C2400000, $BBBBBBCC, $CCCCCCBC, $CCDCDCCC, $CCCCCCCC, $BCBCCCCC
                LONG $89884489, $AAAAA999, $9999AAAA, $66664488, $00010555, $00010100, $55100010, $77777777
                LONG $01000577, $77555775, $77777777, $00057777, $00000000, $00000000, $00000000, $50505005
                LONG $50050050, $50000000, $00505050, $00050000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $91000000, $BBBBAA99, $CCCCCBCB, $CCCCCCCC, $ACCDCCCC, $9AABB989
                LONG $98866689, $AAAAAA99, $899999AA, $66664448, $01000005, $00100000, $75000000, $77777777
                LONG $50010577, $77555577, $77777777, $00055777, $00000000, $00000000, $05005000, $55500000
                LONG $00500000, $00050500, $50000000, $00000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $21000000, $BBBB2502, $CCCBCBBB, $DCCCCCCC, $ACCCCDCC, $99ABBBAA
                LONG $88848888, $AAAAA999, $89999A9A, $66666448, $00000065, $10000000, $55010010, $77777777
                LONG $75501057, $57555777, $77777777, $00000777, $00000000, $00000000, $00000000, $50505000
                LONG $50005050, $05000000, $00000000, $00000500, $05000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $B1000000, $BBBA151A, $CCCCBCBB, $CCCDCCCC, $CCCDCCCC, $ABBCCBBB
                LONG $9999899A, $9AAAA999, $88999999, $56664444, $00000415, $00000001, $75000000, $77777777
                LONG $77500055, $77555555, $77777777, $00005777, $00000000, $50000000, $05000500, $55550505
                LONG $00000000, $00000000, $00005000, $50000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $DB000000, $BBB91503, $CCBBCCBB, $DCCCCCCC, $CCCCCDCC, $BBBCCCCC
                LONG $999999AB, $A9A99999, $48899999, $66666644, $00105541, $00000000, $77501000, $75557777
                LONG $77750157, $77555555, $77777777, $00005777, $50000000, $05050000, $50050050, $55555050
                LONG $00505050, $00005005, $00500005, $00050005, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $DB200000, $BBA2503C, $CBCCCBBB, $CCCCCCCC, $CCCCCCCC, $BBBBCCCC
                LONG $99A999BC, $99A9AA99, $88899999, $06646444, $00006661, $00000000, $57550000, $77777555
                LONG $75777505, $77555757, $77777777, $00000577, $00050000, $00000050, $05500500, $55550505
                LONG $05000500, $05050000, $00000000, $00000500, $00005000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $A9910000, $BBA059CD, $CBCBBCBB, $CCCCCCCC, $CCCDCCCD, $CBBBCCCC
                LONG $99AAAABB, $999A9999, $48889899, $66666444, $00005508, $00000000, $75755000, $57777777
                LONG $55555555, $77555777, $77777775, $00000057, $00000050, $05050000, $55050005, $55555555
                LONG $00500050, $00000505, $50005005, $05000000, $00000000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $BAA10000, $CB2503DC, $BCBCCBCC, $CCCCCCCC, $CCCCCCCC, $BBCBCBCC
                LONG $999AAAAA, $99999999, $44889989, $55666444, $00000105, $50000000, $77777755, $55555555
                LONG $77777777, $57775577, $77777777, $00000057, $50500000, $50505050, $50555505, $55555550
                LONG $50005005, $05000000, $00500000, $00000000, $05000500, $00000500, $00000000, $00000000
                LONG $00000000, $00000000, $BAA15000, $BA051ADC, $CCBCBCBB, $CCCCCCCC, $BBBCCCCC, $9AAABBBB
                LONG $88888999, $99988488, $48888898, $00666644, $00000000, $75500000, $55557777, $77777755
                LONG $77777777, $77775777, $77777777, $50000057, $00005000, $50505050, $55550550, $55555555
                LONG $00050050, $00005050, $00000050, $00000050, $00000005, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $C3100000, $A100613C, $CBCBCCBC, $CCCCCCCB, $AA99ABCC, $848AAAAA
                LONG $88999888, $99946448, $88488899, $00066444, $00000000, $77755000, $77775557, $77777777
                LONG $77777777, $77777777, $77777777, $00000057, $05050000, $55550505, $55555555, $55555555
                LONG $50505050, $00500000, $50005000, $05005000, $00000000, $05000000, $00000000, $00000000
                LONG $00000000, $00000000, $11000000, $20000001, $CBCBCBCB, $CCCCCCCC, $BBAAAABB, $BBCBCCCC
                LONG $9AABBCCB, $99999898, $44881989, $00056644, $00000000, $55777550, $75777775, $77777775
                LONG $77777777, $77757557, $77777777, $00500055, $50505050, $50505550, $55555550, $55555555
                LONG $05000505, $00005000, $00500005, $00000000, $50050000, $00000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $CBCCBBB9, $CCCCCCBB, $CCCCCCCC, $CCCCCCCC
                LONG $88ABCCCC, $99999988, $48484899, $00006664, $50000000, $77775775, $77575777, $77777777
                LONG $57777777, $77775777, $57777777, $50005000, $50505050, $55555505, $55555555, $55555555
                LONG $00505050, $00500005, $50000500, $00500050, $00000000, $00005000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $50000000, $BCBBBB90, $CBCBCBCC, $BBCBCCCC, $CCCCCCCB
                LONG $9888A3CC, $88898998, $44488818, $00505666, $75000000, $55777755, $77777775, $77777777
                LONG $75777777, $77775777, $05777777, $00500000, $55550550, $55555550, $55555555, $55555555
                LONG $00005005, $05005050, $00505005, $00005000, $00000005, $50000000, $00000000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $CBBBB200, $BCBCCBCB, $BBBBCBCB, $A2AA3A3B
                LONG $99998889, $98989999, $44484889, $00005566, $77550000, $77555557, $77777777, $77777777
                LONG $57777777, $77777755, $00577777, $55005050, $55055500, $55555555, $55555555, $55555555
                LONG $50505050, $00050000, $00000050, $00000505, $00050000, $00000005, $00500500, $05000005
                LONG $00000000, $00000000, $00000000, $00000000, $BBBA1500, $CBCBBCBC, $BBBCBCBC, $9AAAABBB
                LONG $99999AAA, $99999899, $66448488, $00005556, $77750000, $77777555, $77777777, $77777777
                LONG $77577777, $77777777, $50057777, $50550000, $55550555, $55555555, $55555555, $55555555
                LONG $00050505, $50005050, $50505000, $05050000, $05000500, $05000000, $00000000, $00000000
                LONG $00000005, $00000000, $00000000, $00000000, $BA200000, $CBCCCBCB, $BBBBCBCB, $BBBBBBBB
                LONG $9A9AAAAB, $99899999, $66444888, $00000576, $57755000, $77777775, $77777777, $77777777
                LONG $75777777, $77777777, $05005777, $50055050, $55555550, $55555555, $55555555, $55555555
                LONG $55005055, $00500500, $00000050, $00000050, $00000050, $00000500, $00000050, $00500500
                LONG $00000000, $00000000, $00000000, $00000000, $B9050000, $BCBBCBBB, $CCCCCCCC, $BBCCCCCC
                LONG $AABAABBB, $8999999A, $66664448, $00005677, $70215000, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $50005777, $55505050, $55555555, $55555555, $55555555, $55555555
                LONG $00050050, $05005005, $05055500, $00000500, $05000000, $00000000, $05005000, $00000000
                LONG $00000000, $00000000, $00000000, $00000000, $B1000000, $CCCBBBBB, $CCCCCCCB, $BCCCCCCC
                LONG $ABABBBBB, $88899999, $66666444, $00000576, $0BC85500, $57777777, $77777777, $77777777
                LONG $77777777, $77777777, $50500557, $55550555, $55555055, $55555555, $55555555, $55555555
                LONG $05050505, $50050050, $50500050, $50500050, $00050000, $05050005, $00000000, $00000000
                LONG $00500000, $05000005, $05000000, $00000000, $A1000000, $BBBBBBCB, $CCCCCCCC, $BBCCCCCC
                LONG $AABBBBBB, $4899999A, $67666664, $00005777, $C3551000, $75557752, $77777777, $77777777
                LONG $77777777, $77777777, $05050005, $55555505, $55555555, $55555555, $55555555, $55555555
                LONG $00505550, $05005055, $00055050, $00050005, $00000500, $00000500, $50000000, $05005000
                LONG $00000500, $00000500, $00000000, $00000000, $A1500000, $BBBBCCCB, $CCCCCBCB, $BBBBCCCC
                LONG $AAABBCBB, $44889999, $66666444, $00005777, $002CA050, $77777509, $77777777, $77777777
                LONG $77777777, $57777777, $50505005, $55555055, $55555555, $55555555, $55555555, $55555555
                LONG $55050055, $00505000, $50505005, $05000500, $00050000, $00000000, $00050005, $00000000
                LONG $00000000, $00000000, $00000500, $00000500, $A1000000, $BBCCBCBC, $CCCCBBBB, $BBBBBBBC
                LONG $9AAABBBB, $44488999, $77666664, $00005777, $1CD25755, $77775577, $77777777, $77777777
                LONG $77777777, $77777777, $55550555, $55505550, $55555555, $55555555, $55555555, $55555555
                LONG $00505505, $05055055, $05050050, $50005005, $05000050, $00050000, $00000500, $00000050
                LONG $00505005, $00000000, $00000000, $00500000, $B9050000, $CCBCBCBB, $BBBBBBBB, $BBBBBBBB
                LONG $999AABBB, $44448889, $66666644, $00057777, $55577755, $77777775, $77777777, $77777777
                LONG $77777777, $57777777, $55055505, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $00050550, $50500500, $50505500, $00500500, $00000000, $00000500, $00000000, $00050000
                LONG $00000000, $00500000, $50000000, $00000000, $BA000000, $BCCBCBBB, $BABABBBC, $AAAAAAAA
                LONG $899999AA, $44444448, $77666444, $00557776, $55757775, $77777777, $77777777, $77777777
                LONG $77777777, $55577777, $55550500, $55555505, $55555555, $55555555, $55555555, $55555555
                LONG $55505055, $05050050, $05050050, $50050050, $00505050, $50000005, $50050000, $05000500
                LONG $50000000, $50000500, $00000000, $00000000, $BB905000, $CBCCBCBB, $BBBBBCBC, $999AAAAB
                LONG $44888989, $48884444, $66766644, $55777777, $75555555, $77777777, $77777777, $77777777
                LONG $77777777, $05577777, $55055555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $05055555, $50555505, $50550500, $05050505, $00000000, $00050000, $00000050, $00000050
                LONG $00000505, $00000000, $00050000, $50000500, $BBA15550, $CBCBCCCB, $BBBCCBCB, $99A9AABA
                LONG $44488898, $88889888, $77666644, $55777767, $77775555, $77777777, $77777777, $77777777
                LONG $77777777, $55577777, $55505050, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $50505055, $00505055, $50005005, $05005050, $00000500, $00000000, $00000000
                LONG $00050000, $00050005, $05000000, $00000000, $BBBB3205, $CCCBCBBB, $BCBCBCCC, $9A9AAABB
                LONG $88888999, $88999898, $66666448, $55567777, $77777555, $77777777, $77777777, $77777777
                LONG $77777777, $50505577, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $50505555, $05550555, $55055505, $00550550, $00050005, $00000005, $05000000, $00500005
                LONG $00000000, $00000000, $00000500, $50050000, $CBBBBB20, $CBCCBBCB, $CBCCCCCB, $AAABBBBC
                LONG $989999AA, $89999999, $76766644, $55557666, $77777755, $77777777, $77777777, $77777777
                LONG $77777777, $55550557, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55055050, $05500550, $50005005, $50000500, $55555500, $00555555, $00005000
                LONG $00000505, $00500000, $00050000, $15500500, $BBBBBBC2, $CCCBCBBC, $CCCCCCCC, $AABBBBCB
                LONG $999AAA9A, $89899999, $66666448, $55555766, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $05555557, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $05505555, $50055005, $00500050, $05550050, $81888814, $00118181, $05050555
                LONG $00050000, $00005005, $50000000, $BA910050, $BBBBBBBB, $CCBCBCCB, $CCCCCCCC, $BBBBBCBC
                LONG $A9A9AAAB, $899999A9, $66666448, $55555666, $77777775, $77777777, $77777777, $77777777
                LONG $77777777, $55550557, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $05505555, $55055505, $00550550, $05050505, $A9980550, $AAAAAAAA, $99AAAAAA, $01111199
                LONG $05500000, $50000000, $00500000, $BBBAA980, $CBBBBBBC, $CBCBCBBC, $CCCCCCCC, $BBBBCBCC
                LONG $AAAAAABB, $8899AA9A, $66666648, $55576666, $77775777, $77777777, $77777777, $77777777
                LONG $77777777, $55555557, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $05550555, $55055055, $50005000, $BBBAA900, $BABBBBBB, $AAAAAABA, $A9A99A99
                LONG $18999A9A, $00000500, $98100505, $CBBBBBBA, $BBCBBBBC, $CCBCBCCC, $CCCCCCCC, $BBCCBCBC
                LONG $AAAAABBB, $8999A9AA, $46664444, $55556664, $77777757, $77777777, $77777777, $77777777
                LONG $77777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $50555555, $05055050, $05005505, $05500505, $BBBBBBA9, $ABBBBBBB, $AAAABAAB, $9A9AAAAA
                LONG $A9A99999, $6550099A, $BAAA9981, $CBCBCBBB, $CBBCBABC, $BCBCBCCC, $CCCCCCCC, $BBBBCCCB
                LONG $AABBBBBB, $8899AAAA, $48444448, $77556444, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55505555, $00550050, $A8050050, $BCBCBBBB, $BBBBBBBC, $ABABABAB, $AAAAAAAA
                LONG $999999AA, $86758AA9, $BBAA9A99, $CCBCBBBB, $CBCBBBCC, $CBCBCBCB, $CCCCCBCC, $BBCCCBCC
                LONG $BBBABBBB, $88999AAB, $48884444, $75564448, $77777775, $77777777, $77777777, $77777777
                LONG $57777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55055555, $50005050, $55500550, $BA805500, $CBCCBCBB, $BBBBCBCC, $BABBBBBB, $AAAAAAAA
                LONG $9999AAAA, $95759999, $BAAA999A, $CBCCBBBB, $CBCBABCC, $BCCCCCCC, $CCCCCCCC, $CCCBBCCC
                LONG $BBBBBBBB, $8889AAAB, $89889888, $75554898, $77777555, $77777777, $77777777, $77777777
                LONG $55777777, $50505555, $66555555, $55555656, $55555555, $55555555, $55555555, $55555555
                LONG $50555555, $05550555, $00055005, $BBB90505, $CCCCCCBB, $BBBCBCCB, $ABABABBB, $ABABABAB
                LONG $99AAAAAB, $96769998, $AAA9AAAA, $BCBBBBBB, $BCBABCCC, $CCBCBCBC, $BCCCCBCB, $BBCBCBCC
                LONG $BBBBBBCC, $99989ABB, $99999999, $75555899, $77777777, $77777777, $77777777, $77777777
                LONG $40577777, $66666664, $66666666, $55666666, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $50555050, $50505505, $BBBA9500, $CCCCCCCC, $BCBCCCCC, $BABBBBAB, $BABABABB
                LONG $99AAAABA, $85709989, $AAABBBBB, $CBCBBBAA, $CBABCCCB, $CCCCCCCB, $CBCBCCCC, $CBCBCBCB
                LONG $BBBCBCBC, $8989AABB, $99999989, $75555089, $77777777, $77775777, $77777777, $77777777
                LONG $66577777, $76666666, $66666676, $66664666, $55555556, $55555555, $55555555, $55555555
                LONG $05505055, $05605055, $05006050, $CBBBA150, $CCCCCCCC, $BBCCCCCC, $BBBABBBB, $ABABBBBA
                LONG $9AAAAAAA, $47548989, $BBBBBCBA, $BBABAAAB, $BAACCBBB, $CBCCCBCC, $CCCCCCCC, $BCBCBCBC
                LONG $CCCCBCBB, $98999ABB, $9AA9A999, $55556609, $77777555, $77777777, $77777777, $77777777
                LONG $66657777, $66766666, $46666666, $64444444, $55555566, $55555555, $55555555, $55555555
                LONG $50555555, $55055555, $50505056, $CCBBB945, $CCCCCCCC, $CCCCCCCC, $BBABBBBB, $BBBBBABB
                LONG $AAAABABB, $47549999, $BBCCCCB9, $ABABABBB, $A3CBBABB, $CCCCBCCA, $BCBCCCCC, $CBBCBCBC
                LONG $CCCCBCBC, $9989AABC, $9AAAAAA9, $57556846, $77775757, $55555577, $77777557, $77777777
                LONG $66657777, $66666666, $46466666, $44446446, $55556644, $55555555, $55555555, $55555555
                LONG $05555555, $50500505, $50050500, $BCBCBB10, $CCCCCCCC, $CCCCCCCC, $BABBBBBC, $BABBABBA
                LONG $9AAAAABA, $6758899A, $CCBBBBA9, $BBBBBBCB, $BBAAAAAB, $BCCBCBAB, $CBCCCCCC, $BCBBCBBB
                LONG $CCCCCCBC, $99989ABB, $42BBAAA9, $75554998, $77777755, $77777757, $77577777, $77777777
                LONG $66677777, $66666566, $44464666, $44444444, $55566444, $55555555, $55555555, $55555555
                LONG $55050555, $55655055, $55005050, $CCCBBB91, $CCCCCCCC, $CCCCCCCC, $BBBBBBCB, $ABBBBBBB
                LONG $AAAABABB, $6768999A, $BCCBBBA8, $BBBBCBCC, $AAAABBBB, $CBBBBBB3, $BCCCCCBC, $BCBBBBBC
                LONG $CCCCCCBC, $A9999AAC, $889BBBAA, $775558A9, $77777575, $77777755, $77777777, $77777777
                LONG $66657777, $66666666, $44446666, $44444444, $55664444, $55555555, $55555555, $55555555
                LONG $05555555, $05050505, $05050505, $CCBCBBA8, $CCCCCCCC, $CCCCCCCC, $ABBBBBCC, $BBBABABA
                LONG $AAAAABAB, $655899A9, $CBBBBAA8, $BBBCBCCB, $ABBBBBBA, $BBCBAAAA, $CCCCCCCB, $CBBBBBBB
                LONG $CCCCCCCB, $AA9999AB, $A989ABBB, $5555559A, $57777777, $75757775, $77777775, $77777777
                LONG $66667777, $66666666, $44444466, $44444444, $56644444, $55555555, $55555555, $55555555
                LONG $55055555, $50505565, $05050050, $CBCCBBB1, $CCCCCCCC, $CCCCCCCC, $BBBBBCCC, $BBBBABAB
                LONG $AAAABABB, $57499A9A, $CCBBBBA8, $BBCBCBCB, $BBBBABBB, $CCAAAAAB, $BBCCBBAB, $CBBBBBBC
                LONG $CCCCCCCC, $BAA999AB, $BAA992BB, $5755778A, $75777775, $75755577, $77777777, $77777777
                LONG $66667777, $66666666, $44444466, $48484444, $56644444, $55555555, $55555555, $55555555
                LONG $56550555, $65055050, $05005605, $CCBBBBB9, $CCCCCCCC, $CCCCCCCC, $ABBBCBCC, $BABABBBB
                LONG $AAABABBB, $55499AAA, $BBBBBBA9, $CCBCCCBC, $ABBBBBBC, $AAAABBBB, $BCCBBABB, $BBBBBBBB
                LONG $BCCCCCCC, $BBAAA99A, $AAABAA9A, $7755774A, $77577775, $57777757, $77777777, $77777777
                LONG $66667777, $66666666, $44444446, $44444848, $66444444, $55555555, $55555555, $55555555
                LONG $55555055, $50565555, $05500500, $CCCCBCBA, $CCCCCCCC, $CCCCCCCC, $BBBCBCCC, $ABABABAB
                LONG $AAABBBAB, $5549A9A9, $CBBBABB8, $BCCBBCCB, $BBBBBBCB, $AABBBBAB, $BBAAAB3A, $CBBBABBB
                LONG $BCCCCCCC, $99AAABAA, $BABAAAAA, $5755774A, $57777777, $77575775, $77777777, $77777777
                LONG $46677777, $66666666, $48444444, $84888848, $66644444, $55555555, $55555555, $55555555
                LONG $05055555, $05500505, $15055055, $CBCBCBB9, $CDCCCCCC, $CCCCCCCC, $BBCBCCCC, $AABBBABB
                LONG $ABABABBA, $55499AAA, $BBBBBBB8, $CBBCCBCB, $BBBBBCBC, $BBBBABBB, $AAABBBBB, $CCBABAAA
                LONG $BCCCCCCC, $BAA9AAAC, $BAABABAB, $7757770A, $75557777, $77777757, $77777777, $77777777
                LONG $66677777, $46666666, $84844444, $48488888, $66664444, $55555555, $55555555, $55555555
                LONG $55555555, $56055560, $15500050, $CCBCBBBA, $CCCCCCCC, $CCCCCCCC, $BBBCCCCC, $ABABABBB
                LONG $BAABBAAA, $768AAAAA, $BBABBBA1, $BCCCBCBB, $BBBCBCBC, $BBABBBBB, $9ABBBBBB, $CBBBAAAA
                LONG $AACCCCCC, $AABAAAA9, $ABBABABA, $55777758, $77557777, $77577575, $77777777, $77777777
                LONG $66657777, $64666666, $88484444, $84888888, $66444444, $55555555, $55555555, $55555555
                LONG $50505555, $05505055, $05065505, $BCCBCBB2, $CCCCCCCC, $CCCCCCCC, $BCCBCCCC, $BBBBBBBB
                LONG $BBBBAAAA, $768A9AAA, $BBBBBCB8, $CBBBCBBB, $CBCBCBCB, $BBBBBBBB, $ABCBBBBB, $BBBAAA99
                LONG $BBABCCCB, $BBABBABB, $ABBBBBBB, $77777556, $55557775, $77775775, $77777775, $77777777
                LONG $66667777, $44666666, $88884844, $48888888, $66644448, $55555555, $55555555, $55555555
                LONG $65555555, $50565505, $05005060, $CBCBBBB9, $CCCCCCCC, $CCCCCCCC, $CBCCCCCC, $ABABABBB
                LONG $ABBA99AA, $749AAABB, $BBBBBCB0, $BCCBCBCB, $BBBBBCBC, $BABBBBCB, $CCBBBBBB, $CCBBCBAB
                LONG $BBCBBAAC, $BBBBBBBB, $9BBBBABB, $77775775, $55555777, $77777777, $77777777, $77777777
                LONG $66677777, $44446646, $88888484, $84888888, $66664444, $55555555, $55555555, $55555555
                LONG $50505055, $55050550, $05550505, $CCBCCBB9, $DCCCCCCC, $CCCCCDCC, $BCBCCCCC, $BABBBBBB
                LONG $BBA99AAB, $54AAAAAB, $BBBBBCB4, $CBBBBCBB, $BCBBBBCB, $BBBBBCBB, $CBBBBBAB, $3CCCCCCC
                LONG $CCCCBB99, $BBBBBBBB, $9BBBBBBB, $77755775, $55555777, $77755555, $77777777, $77777777
                LONG $46677777, $44444466, $88888884, $88888888, $66464444, $55555555, $55555555, $55555555
                LONG $55555555, $60505505, $55006050, $CBCBBBA1, $CCCCCCCC, $CCCCCCCC, $CBCCCCCC, $ABBBBBCB
                LONG $BA98AAAA, $689AABBB, $BBBBCCB4, $BBCBCBBC, $BBBBBCBB, $ABBBBBBB, $BBBBBBBB, $483CCDCC
                LONG $CBBCCCB9, $CBBBBBBB, $759BBBBB, $77777755, $55555577, $77575555, $77777777, $77777777
                LONG $46665777, $44444644, $88888848, $48888888, $66664444, $55555555, $55555555, $55555555
                LONG $05505555, $05555655, $50550555, $CBCBBBA1, $CCCCCCCC, $CCCCCCCC, $BCCCCCCC, $BBBBBCBC
                LONG $BA89AABB, $68AAABBC, $CBCBCCB8, $CBBCBCCB, $BCBCBBBC, $BBBBBBBC, $C3BBABAB, $98AC33CD
                LONG $BCCBCBCB, $BBCBCBCC, $56ABBBBB, $75777777, $55555577, $77777777, $77777777, $77777777
                LONG $64666777, $44444446, $88888884, $88888888, $56664448, $55555555, $55555555, $55555555
                LONG $50555555, $50605055, $50055005, $CBCBBB90, $CCCCCCCC, $CCCCCDCC, $CCCCCCCC, $BBBBBBCB
                LONG $A88AAAAB, $68AABBBB, $BCBCBCB8, $BBBBCBBC, $BBBBCBCB, $BBBBBBBB, $CBABAABB, $99CD33CD
                LONG $BBCBBCBB, $BBBBBBBB, $70ABBBBB, $77777777, $77555557, $77555777, $77777777, $77777777
                LONG $44467777, $84444444, $88888488, $48888888, $56666444, $55555555, $55555555, $55555555
                LONG $55055055, $05555505, $55600650, $BCBBCA90, $CCCCCCCC, $CCCDCCCC, $CBCCCCCC, $BBBCBCBC
                LONG $989AAABA, $689AABBB, $CBCBCCC9, $CBCBCBCB, $CBCBBCBC, $BBBBBCBB, $CCBAABBB, $982CCC33
                LONG $BBBCBBBB, $BBBBBBBB, $770BBBBB, $77777775, $55755557, $77775555, $77777777, $77777777
                LONG $44665777, $44444444, $88888848, $84888888, $56646444, $55555555, $55555555, $55555555
                LONG $05555555, $65050565, $50055505, $CCBBBA15, $DCCCCCCC, $CCCCCCCC, $BCCCCCCC, $ABBBBBBC
                LONG $889AAAAB, $449AABBA, $BCBCBCCA, $BCBCBCBC, $CBCBCBCB, $BBBBBBBC, $B3ABBBBB, $B98833CC
                LONG $BBBBBBBB, $BBBBBBBB, $774ABBBB, $77777757, $55555555, $77777575, $77777777, $77777777
                LONG $46446777, $84444444, $88888884, $48888888, $56666444, $55555555, $55555555, $55555555
                LONG $55505555, $05655505, $05505055, $BBBBB205, $CCCCCCCC, $CCDCCCCC, $CCCCCCCC, $BBBBCCCB
                LONG $89AAAABB, $468AABA9, $CCCCCCCB, $CBCBCBCB, $BCBCBCBC, $BBBCBCBC, $AABABBBB, $BBA989B3
                LONG $BBBBBBBB, $BBBBBBBB, $55558ABB, $77777777, $75755555, $77777777, $77777777, $77777777
                LONG $44465777, $44444444, $88888888, $44888888, $55664444, $55555555, $55555555, $55555555
                LONG $50555055, $50505550, $00506505, $CCBBA855, $CCCCCCCC, $DCCCCCCC, $CCCCCCCC, $BBBBBBCC
                LONG $89AAAABA, $9649AA98, $BCCCCCCB, $BCBCBCBC, $BBCBCBCB, $BBBBCBCB, $BABBBBBB, $BBBBBAAB
                LONG $BBCCBBBB, $BBBBBBBB, $755759CC, $57777757, $77575755, $77777757, $77777777, $77777777
                LONG $44446777, $84444444, $88888884, $44888888, $55666644, $55555555, $55555555, $55555555
                LONG $55555555, $05555055, $56555065, $BCBB9150, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $BBBBCCBC
                LONG $999AAAAB, $B4649888, $CBCCCBCC, $CBCBCBCB, $CBCBCBCB, $BBCBBBBB, $BBBBBBBB, $BBBBBABA
                LONG $CBBBCCCB, $CBBBBBCB, $5777509B, $57777775, $77777555, $77757777, $77777777, $77777777
                LONG $44466777, $48444444, $88888888, $44448888, $55566644, $55555555, $55555555, $55555555
                LONG $55555555, $65050650, $50000550, $CBBB9050, $CCCCCCCB, $CCCCCCCC, $CCCCCCCC, $BBBBCBCC
                LONG $99AAAABB, $B9664888, $BCCCBCCB, $BCBCBCBC, $BCBCBCBC, $BBBBBBBB, $BBBBBBBB, $BBBBABAB
                LONG $CBCCCBBB, $ACCCCCBB, $77755776, $55577777, $75755755, $77777775, $77777777, $77777777
                LONG $44467777, $84444464, $88888884, $44488888, $55566644, $55555555, $55555555, $55555555
                LONG $56505555, $50565555, $05556505, $BBBA1505, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $BBBCBCBC
                LONG $9A9AAABA, $BA475648, $CCBCCCBB, $CBCBCBCB, $BCBCCBCB, $BBBBBBCC, $BBBBBBCB, $BBBBBBBB
                LONG $BBBBBBBB, $09AABBBB, $55757557, $55577777, $57757555, $77777757, $77777777, $77777777
                LONG $65777777, $84844444, $88888484, $44844888, $55566464, $55555555, $55555555, $55555555
                LONG $50555555, $55550505, $50605550, $BBB91550, $CCCCCBBC, $CCDCCCCC, $CCCCCCCC, $BBBBCBCB
                LONG $99AAAABB, $BA857648, $BCCCBCCB, $CCBCBCBC, $CBCBCBCC, $BBBBBBBB, $BBBBBBBB, $BBBBABBA
                LONG $BBBBBBBB, $77559BCB, $77777775, $55557777, $75757555, $57577775, $77777757, $77777777
                LONG $77777777, $44844657, $88888848, $44484488, $55556664, $55555555, $55555555, $55555555
                LONG $55555555, $65055555, $05550505, $CBB90506, $CCCCCCCB, $CCCCCCCC, $CCCCCCCC, $BBBBCBCC
                LONG $A9AAABAB, $AA857689, $CCCCCBBB, $BBCBCCBC, $BCBCBCBC, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $BBBBBBBB, $55778BBB, $77777557, $55557777, $77575775, $77757777, $77775577, $77777777
                LONG $77777777, $84467777, $48484844, $44444844, $55556466, $55555555, $55555555, $55555555
                LONG $05505555, $50550555, $55005650, $BBA15550, $CCCBCCBC, $CCCCCCCC, $CCCCCCCD, $BBCCCCCB
                LONG $99AAAABB, $A9857489, $BCBCBCBB, $CCCCBCBC, $CCBCBCCC, $BBBBBBCB, $BBBBBBBB, $BBBBBBBB
                LONG $8BBBBBBB, $77755575, $77755777, $55557777, $57575755, $57577777, $77577575, $77777777
                LONG $77777777, $66777777, $48484848, $44444448, $55555666, $55555555, $55555555, $55555555
                LONG $55555555, $55505555, $50555055, $BBA15005, $CCCCCBCB, $CCCCCCCC, $CCCCCCCC, $AAAABBBC
                LONG $AAAAAAAA, $AA456499, $CCCCBBBB, $BCBCCBCC, $BBCBCBBC, $BBBBBCBC, $BBBBBBBB, $BBBBBBBB
                LONG $54ABBBBB, $75577757, $57577777, $55555777, $75755757, $75777577, $77755755, $77777777
                LONG $77777777, $77777777, $44448446, $64444444, $55555664, $55555555, $55555555, $55555555
                LONG $55555555, $05655555, $65065556, $BB905550, $CCBCCBBB, $CCCCCCCC, $419BCCDC, $57555566
                LONG $46655577, $A9676884, $CCBCCBBA, $CCBCBCBC, $CBCCCCCB, $BBBBBBBC, $BBBBBBBB, $BBBBBBBB
                LONG $559BBBBB, $57775555, $77777755, $55555777, $77755575, $55557757, $77775555, $77777777
                LONG $77777777, $77777777, $44844677, $64444444, $55555666, $55555555, $55555555, $55555555
                LONG $55555555, $55550555, $50550505, $BA905065, $CCCCBCCB, $CCDCCCCC, $7577554A, $55555575
                LONG $77757555, $98677777, $BCCBBBBA, $CBCBCCBC, $BCCBBCBC, $BBBBBBBB, $BBBBBBBB, $BBBBBBBB
                LONG $5709BBBB, $77757755, $77777777, $75555577, $77577557, $55775777, $75757555, $77777777
                LONG $77777777, $77777777, $44447777, $66444444, $55555664, $55555555, $55555555, $55555555
                LONG $55555555, $55055555, $05505555, $BB155505, $CCCBCBBB, $568BCCCC, $55555557, $75555555
                LONG $75575575, $94577555, $CCBBBBAA, $BCCBCBCB, $CCBCCCCC, $BBBBBBCB, $BBBBBBBB, $BBBBBBBB
                LONG $777759BC, $77577757, $77777777, $55555577, $57575557, $55557577, $77555555, $77777777
                LONG $77777777, $77777777, $44677777, $64444444, $55555566, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $56055605, $BA155055, $CCBCBCBB, $55756ACC, $55555555, $57555555
                LONG $57555755, $85755775, $BBBBBAA9, $CBBCBCBC, $BBCBCBBB, $BBBBBCBC, $BBBBBBBB, $CBBBBBBB
                LONG $777576AC, $57777757, $77777777, $55555557, $77557575, $55555775, $77575555, $77777777
                LONG $77777777, $77777777, $46777777, $66464444, $55555556, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $05550555, $BA155605, $BCCBBBCB, $55555758, $55555555, $75757555
                LONG $57575575, $57575577, $CBBBAA98, $BCCCBCCB, $CBCBCCCC, $BBBBBBBB, $BBBBBBBB, $BCBBBBBB
                LONG $7755558A, $77777557, $77777775, $75555555, $57755555, $55555777, $77755555, $77777777
                LONG $77777777, $77777777, $67777777, $46644444, $55555556, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $BA450555, $58BCCBBB, $55555557, $55555555, $55575575
                LONG $75775755, $57575755, $BBBAA994, $BCBBCBCB, $BCBCBBBC, $ABBBBBBB, $BBBABABB, $68BCCBBB
                LONG $75555777, $77777577, $77777757, $57555557, $77757575, $55575575, $77575555, $77777777
                LONG $77777777, $77777777, $77777777, $64666446, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55055550, $BA055506, $576ACCCB, $55555555, $75555555, $75757555
                LONG $57555757, $77755777, $BBAA9986, $CCBCCBCB, $CBCBCCCB, $BBABBBBB, $BBABBBBA, $759AAABB
                LONG $55577775, $77775777, $77777777, $55555555, $77555557, $55755755, $77775555, $77777777
                LONG $77777777, $77777777, $77777777, $66664466, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $05555555, $BA055555, $55778BBB, $55555555, $55555555, $55555555
                LONG $75775755, $57577757, $BAA99845, $BBCBBCBB, $BBBCBBBC, $ABABBBBB, $BBABAAAB, $5555556A
                LONG $55777555, $77777775, $57777775, $57575555, $77777555, $55557577, $77555555, $77777777
                LONG $77777777, $77777777, $77777777, $56644467, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $BA055655, $555554BC, $75555555, $55575555, $77777757
                LONG $77577777, $77757775, $BA998465, $CBCCBBBB, $BBCBCCBC, $BABABBBB, $AABAAAAA, $77755758
                LONG $57775577, $77777755, $57777777, $55555555, $75555755, $57555555, $77775555, $77777777
                LONG $77777777, $77777777, $77777777, $56666467, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $C9455050, $5555554A, $55755555, $57555775, $55575777
                LONG $75755555, $75775777, $AA994657, $BCBBBBBB, $BBBBBBCB, $AAABABBB, $6ABAAAAA, $75575777
                LONG $57777775, $77777757, $57777777, $55755555, $77775555, $55557577, $77777575, $77777777
                LONG $77777777, $77777777, $77777777, $55664657, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $BA055555, $55555758, $57555555, $55755557, $55555555
                LONG $75757755, $77777775, $A9984657, $BBBCBBBA, $BBBBBBBB, $AAAABABB, $58AAAAAA, $77577755
                LONG $77777577, $77775555, $55777777, $55555555, $75555555, $55555555, $77775755, $77777777
                LONG $77777777, $77777777, $77777777, $55566667, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $9A055555, $55555555, $55575555, $55555555, $55555555
                LONG $77775757, $77777757, $A9946577, $BCBCBBBA, $BBBBBBBB, $AAABABBB, $557649AA, $77775555
                LONG $77775777, $77775757, $55777777, $55755555, $57575555, $55755777, $77777577, $77777777
                LONG $77777777, $77777777, $77777777, $55566657, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $51455555, $55555557, $55555555, $55555755, $77575555
                LONG $75755575, $77757777, $A9845777, $BBBBBBBA, $BBBBBBBB, $AAAAAAAB, $75577749, $77757757
                LONG $75577777, $77775777, $55577777, $55557555, $75755555, $55555555, $77757755, $77777777
                LONG $77777777, $77777777, $77777777, $55556657, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $57575575, $55555757
                LONG $75557555, $75777775, $99867777, $BBBBBBBA, $ABBBBBBB, $9AAAAAAA, $75557774, $75575777
                LONG $57777777, $77557777, $55577777, $55555555, $57577755, $75557557, $77777577, $77777777
                LONG $77777777, $77777777, $77777777, $55555657, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55575755, $55555555
                LONG $55555575, $77777757, $98665777, $BCBBBBAA, $BABBABBB, $568AAAAA, $55557777, $57757777
                LONG $77777777, $77777775, $55577777, $55555555, $57575555, $55775577, $77777757, $77777777
                LONG $77777777, $77777777, $77777777, $55555557, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $75755755, $55555575, $57757555
                LONG $57757557, $77755555, $98657777, $BBCBBBBA, $ABABBBBB, $77769AAA, $55577775, $75577775
                LONG $77777775, $77777777, $55577777, $55555555, $75755555, $77557757, $77777575, $77777777
                LONG $77777777, $77777777, $77777777, $55555557, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $75557555, $77777775
                LONG $75777777, $75775777, $94577777, $CBBBBBAA, $AAABBBBB, $557759AA, $55777755, $77777755
                LONG $77777757, $77777777, $55557777, $55555555, $55575755, $57775755, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $55555557, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $75555555, $77755575, $55555577
                LONG $57555555, $57557557, $86577777, $BCBBBBA9, $AAAAABBB, $5577769A, $57777555, $77777555
                LONG $77777757, $77777777, $55557777, $55555555, $57775555, $75577577, $77777575, $77777777
                LONG $77777777, $77777777, $77777777, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55777755, $55555555
                LONG $57555555, $75775777, $46777775, $BBBBBA99, $ABABBBBB, $55777758, $77775555, $77775557
                LONG $77777557, $77777777, $55557777, $55555755, $75555757, $77757775, $77777775, $77777777
                LONG $77777777, $77777777, $57777777, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55557555, $55555777, $55555555
                LONG $75575555, $57757777, $67777757, $BBBBBA38, $69BABBBB, $55577775, $77755555, $77775577
                LONG $77777777, $77777777, $55555777, $55555555, $55777555, $75575575, $77777777, $77777777
                LONG $77777777, $77777777, $55777777, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $75555555, $75555555, $55555757
                LONG $77557555, $77777575, $57777577, $BBBBBA94, $5709BBBB, $55577755, $77555577, $77755577
                LONG $77777777, $77777777, $55555777, $55575555, $57555757, $77757757, $77777777, $77777777
                LONG $77777777, $77777777, $55577777, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77775555, $77777777
                LONG $57777777, $77575777, $57775577, $BBBBA996, $57751ABB, $55577775, $77555777, $77755777
                LONG $77775777, $77777777, $55555777, $55555555, $57577775, $77777575, $77777777, $77777777
                LONG $77777777, $77777777, $55555777, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55577555, $75777755
                LONG $77777777, $77777777, $77757777, $BBAA3946, $5555751A, $55777755, $77557775, $77755777
                LONG $77777777, $77777777, $55555777, $55555555, $75757555, $77777775, $77777777, $77777777
                LONG $77777777, $77777777, $55555577, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77757577
                LONG $77777777, $77777777, $77777777, $583BA905, $57755575, $75777775, $77577777, $77755777
                LONG $77777777, $77777777, $55555777, $55757555, $57575755, $77575757, $77777777, $77777777
                LONG $77777777, $77777777, $55555557, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77555555, $77777757
                LONG $77777777, $77777777, $77777777, $57582867, $55755555, $75577775, $77777775, $77757777
                LONG $77777777, $77777777, $55555777, $75555755, $77577775, $77757757, $77777777, $77777777
                LONG $77777777, $77777777, $55555557, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55775555, $57777777
                LONG $75555557, $77777777, $77777777, $55555657, $55777555, $77777775, $77777777, $77757777
                LONG $77777777, $77777777, $55555777, $55755555, $55755575, $77775775, $77777777, $77777777
                LONG $77777777, $77777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77557555, $75577777
                LONG $57577775, $77777775, $77777777, $55555555, $55577577, $75777775, $77777777, $77757777
                LONG $77777777, $77777777, $55555777, $57555755, $57555777, $77775755, $77777777, $77777777
                LONG $77777777, $57777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77775755, $77755777
                LONG $77577777, $77777777, $55777777, $77555555, $55775755, $77777775, $77777777, $77757777
                LONG $77777777, $77777777, $55555777, $55555555, $75755575, $77577557, $77777777, $77777777
                LONG $77777777, $55777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $75777575, $77777555
                LONG $77775577, $77777777, $55555777, $75777555, $55577775, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $55555777, $55575555, $75575577, $77757775, $77777777, $77777777
                LONG $77777777, $55777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $75555555, $57577557, $77777777
                LONG $77777755, $77775777, $75555557, $57555775, $55777777, $77777775, $77777777, $77777777
                LONG $77777777, $77777777, $55555777, $55557755, $57757557, $77777557, $77777777, $77777777
                LONG $77777777, $55577777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $57555555, $75557755, $55557777
                LONG $57777777, $77777755, $55755555, $75777555, $55777757, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $55555777, $75577555, $57575575, $77575775, $77777777, $77777777
                LONG $77777777, $55557777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55755555, $77755755, $77755557
                LONG $55555777, $75777777, $75575777, $57755757, $55777775, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $55577777, $55755755, $57755757, $77777757, $77777777, $77777777
                LONG $77777777, $55557777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55575555, $77775577, $55555777
                LONG $75555555, $75775577, $55557777, $77775757, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $55775777, $75575555, $75777777, $77777777, $77777777, $77777777
                LONG $77777777, $55555777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77775555, $77777555
                LONG $77577755, $55777757, $75775777, $77577575, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $57757777, $75555555, $77777775, $77777757, $77777777, $77777777
                LONG $77777777, $55555577, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $75555555, $55557555, $55557777
                LONG $77755777, $57777775, $57557775, $77777775, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $55577777, $75575557, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $55555557, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77755555, $75775557
                LONG $57777577, $57775775, $75577775, $77777557, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $57777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $57555555, $55777555, $77757755
                LONG $75777755, $57775777, $75777755, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $57777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $75555555, $55555555, $75577755, $57755777
                LONG $57577775, $77777777, $57777555, $77775757, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $57777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $77555775, $55575557
                LONG $75557777, $77577577, $77775557, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $57777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55755555, $55775577, $75575555
                LONG $77575777, $77777777, $57575757, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $55777777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555
                LONG $55555555, $55555555, $55555555, $55555555, $55555555, $55575555, $55577555, $77557555
                LONG $75757777, $75777777, $77775777, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777, $77777777
                LONG $55577777, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555, $55555555


Sprite_Size     BYTE $20
Sprite_Data     WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
'               WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020



                WORD $0000, $0000, $0000, $0000, $0000, $2007, $200B, $200C, $200E, $2010, $2011, $2012, $2014, $2015, $2015, $2016
                WORD $2017, $2018, $2019, $2019, $201A, $201B, $201B, $201C, $201C, $201D, $201D, $201E, $201E, $201F, $201F, $2020
                WORD $0000, $0000, $0000, $0000, $0000, $1F07, $1F0B, $1F0C, $1F0E, $1F10, $1F11, $1F12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1E07, $1E09, $1E0C, $1E0E, $1E10, $1E11, $1E12, $1F14, $1F15, $1F15, $1F16
                WORD $1F17, $1F18, $1F19, $1F19, $1F1A, $1F1B, $1F1B, $1F1C, $1F1C, $1F1D, $1F1D, $1F1E, $1F1E, $1F1F, $1F1F, $1F20
                WORD $0000, $0000, $0000, $0000, $0000, $1D07, $1D09, $1D0C, $1D0E, $1E10, $1E11, $1E12, $1E14, $1E14, $1E15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1C09, $1C0B, $1D0E, $1D10, $1D11, $1D12, $1D13, $1D14, $1D15, $1E16
                WORD $1E17, $1E18, $1E18, $1E19, $1E1A, $1E1B, $1E1B, $1E1C, $1E1C, $1E1D, $1E1D, $1E1E, $1E1E, $1E1F, $1E1F, $1E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1B09, $1B0B, $1C0D, $1C0F, $1C10, $1C12, $1C13, $1D14, $1D15, $1D16
                WORD $1D17, $1D18, $1D18, $1D19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1A08, $1B0B, $1B0D, $1B0F, $1B10, $1C12, $1C13, $1C14, $1C15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1D1A, $1D1A, $1D1B, $1D1C, $1D1C, $1D1D, $1D1D, $1D1E, $1D1E, $1D1F, $1D1F, $1D20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $1908, $190A, $1A0D, $1A0F, $1B10, $1B12, $1B13, $1B14, $1B15, $1C16
                WORD $1C17, $1C18, $1C18, $1C19, $1C1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $180A, $190C, $190E, $1A10, $1A11, $1A13, $1A14, $1B15, $1B16
                WORD $1B17, $1B18, $1B18, $1B19, $1B1A, $1C1A, $1C1B, $1C1C, $1C1C, $1C1D, $1C1D, $1C1E, $1C1E, $1C1F, $1C1F, $1C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1709, $180C, $180E, $1910, $1911, $1A13, $1A14, $1A14, $1A16
                WORD $1A17, $1A17, $1B18, $1B19, $1B1A, $1B1A, $1B1B, $1B1C, $1B1C, $1B1D, $1B1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $1609, $170B, $170D, $180F, $1810, $1812, $1913, $1914, $1916
                WORD $1916, $1A17, $1A18, $1A19, $1A1A, $1A1A, $1A1B, $1A1C, $1A1C, $1A1D, $1A1D, $1B1E, $1B1E, $1B1F, $1B1F, $1B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $150A, $160D, $170F, $1710, $1812, $1813, $1814, $1815
                WORD $1916, $1917, $1918, $1919, $1919, $1A1A, $1A1B, $1A1B, $1A1C, $1A1D, $1A1D, $1A1E, $1A1E, $1A1F, $1A1F, $1A20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $140A, $150C, $160E, $1610, $1711, $1713, $1714, $1815
                WORD $1816, $1817, $1818, $1818, $1919, $191A, $191B, $191B, $191C, $191C, $191D, $191E, $191E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130B, $150E, $1510, $1611, $1613, $1613, $1715
                WORD $1716, $1817, $1818, $1818, $1819, $181A, $181B, $181B, $181C, $181C, $181D, $181E, $181E, $191F, $191F, $1920
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $130D, $140F, $1410, $1512, $1613, $1615
                WORD $1615, $1616, $1718, $1718, $1719, $171A, $171A, $181B, $181C, $181C, $181D, $181E, $181E, $181F, $181F, $1820
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $110C, $120E, $1310, $1411, $1413, $1514
                WORD $1515, $1516, $1617, $1618, $1619, $1619, $171A, $171B, $171C, $171C, $171D, $171E, $171E, $171F, $171F, $1720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100D, $110F, $1210, $1312, $1414
                WORD $1415, $1516, $1517, $1518, $1518, $1619, $161A, $161B, $161C, $161C, $161D, $161E, $161E, $161F, $161F, $1620
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $100E, $1010, $1111, $1213
                WORD $1314, $1316, $1316, $1417, $1418, $1419, $141A, $151B, $151B, $151C, $151D, $151D, $151E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0F0F, $1010, $1012
                WORD $1114, $1215, $1316, $1317, $1318, $1319, $141A, $141A, $141B, $141C, $141D, $141D, $141E, $151F, $151F, $1520
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0E10, $0F11
                WORD $1013, $1014, $1116, $1117, $1218, $1218, $131A, $131A, $131B, $131C, $131C, $131D, $141E, $141F, $141F, $1420
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0D10
                WORD $0E12, $0F14, $1015, $1016, $1017, $1018, $1119, $111A, $121B, $121C, $121C, $121D, $121E, $121E, $121F, $1220
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0C11, $0D13, $0E15, $0E16, $0F17, $0F18, $1019, $101A, $101B, $101B, $101C, $111D, $111E, $111E, $111F, $1120
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0B13, $0C15, $0D16, $0D17, $0E18, $0E19, $0F1A, $0F1B, $0F1C, $101D, $101E, $101E, $101F, $1020
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0A14, $0A15, $0B17, $0C18, $0C19, $0D1A, $0D1B, $0D1C, $0E1D, $0E1D, $0E1E, $0E1F, $0E20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0916, $0917, $0A18, $0A19, $0B1B, $0B1B, $0B1C, $0C1D, $0C1E, $0C1F, $0C20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0819, $081A, $091B, $091C, $091D, $091E, $0B1F, $0B20
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $071D, $071E, $071F, $0720
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000
                WORD $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000



}